@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Author: aCaB <acab@clamav.net>
@@ -37,18 +37,24 @@
 #include <time.h>
 #include <libmilter/mfapi.h>
 
+// libclamav
 #include "clamav.h"
+#include "default.h"
 
-#include "shared/output.h"
-#include "shared/optparser.h"
-#include "shared/misc.h"
-#include "libclamav/default.h"
+// shared
+#include "output.h"
+#include "optparser.h"
+#include "misc.h"
 
 #include "connpool.h"
 #include "netcode.h"
 #include "clamfi.h"
 #include "whitelist.h"
 
+#ifndef _WIN32
+#include <sys/wait.h>
+#endif
+
 struct smfiDesc descr;
 struct optstruct *opts;
 
@@ -83,25 +89,14 @@ int main(int argc, char **argv)
     const struct optstruct *opt;
     time_t currtime;
     mode_t umsk;
-    int ret;
+    pid_t parentPid = getpid();
+#ifndef _WIN32
+    int dropPrivRet = 0;
+#endif /* _WIN32 */
 
     sigset_t sigset;
     struct sigaction act;
-
-    sigfillset(&sigset);
-    sigdelset(&sigset, SIGUSR1);
-    sigdelset(&sigset, SIGFPE);
-    sigdelset(&sigset, SIGILL);
-    sigdelset(&sigset, SIGSEGV);
-#ifdef SIGBUS
-    sigdelset(&sigset, SIGBUS);
-#endif
-    pthread_sigmask(SIG_SETMASK, &sigset, NULL);
-    memset(&act, 0, sizeof(struct sigaction));
-    act.sa_handler = milter_exit;
-    sigfillset(&(act.sa_mask));
-    sigaction(SIGUSR1, &act, NULL);
-    sigaction(SIGSEGV, &act, NULL);
+    const char * user_name = NULL;
 
     cl_initialize_crypto();
 
@@ -127,7 +122,7 @@ int main(int argc, char **argv)
         printf("\n");
         printf("                       Clam AntiVirus: Milter Mail Scanner %s\n", get_version());
         printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-        printf("           (C) 2020 Cisco Systems, Inc.\n");
+        printf("           (C) 2021 Cisco Systems, Inc.\n");
         printf("\n");
         printf("    %s [-c <config-file>]\n\n", argv[0]);
         printf("\n");
@@ -163,6 +158,10 @@ int main(int argc, char **argv)
     }
     free(pt);
 
+    if ((opt = optget(opts, "User"))->enabled){
+        user_name = opt->strarg;
+    }
+
     if ((opt = optget(opts, "Chroot"))->enabled) {
         if (chdir(opt->strarg) != 0) {
             logg("!Cannot change directory to %s\n", opt->strarg);
@@ -261,11 +260,11 @@ int main(int argc, char **argv)
             }
         }
 
-        if ((opt = optget(opts, "User"))->enabled) {
+        if (NULL != user_name) {
             struct passwd *user;
-            if ((user = getpwnam(opt->strarg)) == NULL) {
+            if ((user = getpwnam(user_name)) == NULL) {
                 logg("ERROR: Can't get information about user %s.\n",
-                     opt->strarg);
+                     user_name);
                 logg_close();
                 optfree(opts);
                 return 1;
@@ -299,40 +298,6 @@ int main(int argc, char **argv)
         }
     }
 
-    if (geteuid() == 0 && (opt = optget(opts, "User"))->enabled) {
-        struct passwd *user = NULL;
-        if ((user = getpwnam(opt->strarg)) == NULL) {
-            fprintf(stderr, "ERROR: Can't get information about user %s.\n", opt->strarg);
-            optfree(opts);
-            return 1;
-        }
-
-#ifdef HAVE_INITGROUPS
-        if (initgroups(opt->strarg, user->pw_gid)) {
-            fprintf(stderr, "ERROR: initgroups() failed.\n");
-            optfree(opts);
-            return 1;
-        }
-#elif HAVE_SETGROUPS
-        if (setgroups(1, &user->pw_gid)) {
-            fprintf(stderr, "ERROR: setgroups() failed.\n");
-            optfree(opts);
-            return 1;
-        }
-#endif
-        if (setgid(user->pw_gid)) {
-            fprintf(stderr, "ERROR: setgid(%d) failed.\n", (int)user->pw_gid);
-            optfree(opts);
-            return 1;
-        }
-
-        if (setuid(user->pw_uid)) {
-            fprintf(stderr, "ERROR: setuid(%d) failed.\n", (int)user->pw_uid);
-            optfree(opts);
-            return 1;
-        }
-    }
-
     logg_lock    = !optget(opts, "LogFileUnlock")->enabled;
     logg_time    = optget(opts, "LogTime")->enabled;
     logg_size    = optget(opts, "LogFileMaxSize")->numarg;
@@ -401,8 +366,9 @@ int main(int argc, char **argv)
 
     multircpt = optget(opts, "SupportMultipleRecipients")->enabled;
 
+#ifndef _WIN32
     if (!optget(opts, "Foreground")->enabled) {
-        if (daemonize() == -1) {
+        if (-1 == daemonize_parent_wait(user_name, logg_file)) {
             logg("!daemonize() failed\n");
             localnets_free();
             whitelist_free();
@@ -411,10 +377,28 @@ int main(int argc, char **argv)
             optfree(opts);
             return 1;
         }
-        if (chdir("/") == -1)
+        if (chdir("/") == -1) {
             logg("^Can't change current working directory to root\n");
+        }
     }
 
+    sigfillset(&sigset);
+    sigdelset(&sigset, SIGUSR1);
+    sigdelset(&sigset, SIGFPE);
+    sigdelset(&sigset, SIGILL);
+    sigdelset(&sigset, SIGSEGV);
+#ifdef SIGBUS
+    sigdelset(&sigset, SIGBUS);
+#endif
+    pthread_sigmask(SIG_SETMASK, &sigset, NULL);
+    memset(&act, 0, sizeof(struct sigaction));
+    act.sa_handler = milter_exit;
+    sigfillset(&(act.sa_mask));
+    sigaction(SIGUSR1, &act, NULL);
+    sigaction(SIGSEGV, &act, NULL);
+
+#endif /* _WIN32 */
+
     maxfilesize = optget(opts, "MaxFileSize")->numarg;
     if (!maxfilesize) {
         logg("^Invalid MaxFileSize, using default (%d)\n", CLI_DEFAULT_MAXFILESIZE);
@@ -435,18 +419,60 @@ int main(int argc, char **argv)
     if ((opt = optget(opts, "PidFile"))->enabled) {
         FILE *fd;
         mode_t old_umask = umask(0002);
+        int err = 0;
 
         if ((fd = fopen(opt->strarg, "w")) == NULL) {
             logg("!Can't save PID in file %s\n", opt->strarg);
+            err = 1;
         } else {
             if (fprintf(fd, "%u\n", (unsigned int)getpid()) < 0) {
                 logg("!Can't save PID in file %s\n", opt->strarg);
+                err = 1;
             }
             fclose(fd);
         }
         umask(old_umask);
+
+#ifndef _WIN32
+        if (0 == err){
+            /*If the file has already been created by a different user, it will just be
+             * rewritten by us, but not change the ownership, so do that explicitly.
+             */
+            if (0 == geteuid()){
+                struct passwd * pw = getpwuid(0);
+                int ret = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
+                if (ret){
+                    logg("!Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
+                    err = 1;
+                }
+            }
+        }
+#endif /*_WIN32*/
+
+        if (err){
+            localnets_free();
+            whitelist_free();
+            logg_close();
+            optfree(opts);
+            return 2;
+        }
     }
 
+#ifndef _WIN32
+    dropPrivRet = drop_privileges(user_name, logg_file);
+    if (dropPrivRet){
+        optfree(opts);
+        return dropPrivRet;
+    }
+
+    /* We have been daemonized, and initialization is done.  Signal
+     * the parent process so that it can exit cleanly.
+     */
+    if (parentPid != getpid()){ //we have been daemonized
+        daemonize_signal_parent(parentPid);
+    }
+#endif
+
     return smfi_main();
 }
 
