@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Author: aCaB <acab@clamav.net>
@@ -36,9 +36,9 @@
 #include <time.h>
 #include <netdb.h>
 
-#include "shared/optparser.h"
-#include "shared/output.h"
-#include "shared/misc.h"
+#include "optparser.h"
+#include "output.h"
+#include "misc.h"
 
 #include "connpool.h"
 #include "netcode.h"
