@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Author: Tomasz Kojm <tkojm@clamav.net>
@@ -37,19 +37,22 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include "shared/optparser.h"
-#include "shared/misc.h"
-
 #include "clamav-config.h"
-#include "libclamav/str.h"
-#include "libclamav/clamav.h"
-#include "libclamav/others.h"
-#include "libclamav/readdb.h"
-#include "libclamav/bytecode.h"
-#include "libclamav/bytecode_detect.h"
 #include "target.h"
+
+// libclamav
+#include "clamav.h"
+#include "str.h"
+#include "others.h"
+#include "readdb.h"
+#include "bytecode.h"
+#include "bytecode_detect.h"
 #include "fpu.h"
 
+// shared
+#include "optparser.h"
+#include "misc.h"
+
 #ifndef _WIN32
 extern const struct clam_option *clam_options;
 #else
@@ -204,7 +207,7 @@ static void help(void)
     printf("\n");
     printf("                       Clam AntiVirus: Configuration Tool %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2020 Cisco Systems, Inc.\n");
+    printf("           (C) 2021 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    --help                 -h         Show this help\n");
     printf("    --version              -V         Show version\n");
