@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Török Edvin
@@ -37,13 +37,15 @@
 #endif
 #include <errno.h>
 
-#include "libclamav/clamav.h"
+// libclamav
+#include "clamav.h"
 
-#include "shared/optparser.h"
-#include "shared/output.h"
-#include "shared/misc.h"
+// shared
+#include "optparser.h"
+#include "output.h"
+#include "misc.h"
 
-#include "others.h"
+#include "clamd_others.h"
 #include "server.h"
 #include "tcpserver.h"
 
