@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -35,13 +35,14 @@
 #include <time.h>
 #include <signal.h>
 
+// libclamav
 #include "clamav.h"
 
-#include "libclamav/clamav.h"
-#include "shared/output.h"
-#include "shared/misc.h"
-#include "shared/optparser.h"
-#include "shared/actions.h"
+// shared
+#include "output.h"
+#include "misc.h"
+#include "optparser.h"
+#include "actions.h"
 
 #include "client.h"
 
@@ -62,16 +63,18 @@ int main(int argc, char **argv)
 {
     int ds, dms, ret, infected = 0, err = 0;
     struct timeval t1, t2;
-    time_t starttime;
+    time_t date_start, date_end;
+
     struct optstruct *opts;
     const struct optstruct *opt;
+    char buffer[26];
 #ifndef _WIN32
     struct sigaction sigact;
 #endif
 
     if ((opts = optparse(NULL, argc, argv, 1, OPT_CLAMDSCAN, OPT_CLAMSCAN, NULL)) == NULL) {
         mprintf("!Can't parse command line options\n");
-        return 2;
+        exit(2);
     }
 
     if (optget(opts, "help")->enabled) {
@@ -81,7 +84,8 @@ int main(int argc, char **argv)
 
     if ((clamdopts = optparse(optget(opts, "config-file")->strarg, 0, NULL, 1, OPT_CLAMD, 0, NULL)) == NULL) {
         logg("!Can't parse clamd configuration file %s\n", optget(opts, "config-file")->strarg);
-        return 2;
+        optfree(opts);
+        exit(2);
     }
 
     if (optget(opts, "verbose")->enabled) {
@@ -102,6 +106,24 @@ int main(int argc, char **argv)
         exit(0);
     }
 
+    if (optget(opts, "ping")->enabled && !optget(opts, "wait")->enabled) {
+        int16_t ping_result = ping_clamd(opts);
+        switch (ping_result) {
+            case 0:
+                ret = 0;
+                break;
+            case 1:
+                ret = (int)CL_ETIMEOUT;
+                break;
+            default:
+                ret = (int)CL_ERROR;
+                break;
+        }
+        optfree(opts);
+        optfree(clamdopts);
+        exit(ret);
+    }
+
     if (optget(opts, "infected")->enabled)
         printinfected = 1;
 
@@ -141,9 +163,7 @@ int main(int argc, char **argv)
     sigaction(SIGPIPE, &sigact, NULL);
 #endif
 
-    time(&starttime);
-    /* ctime() does \n, but I need it once more */
-
+    date_start = time(NULL);
     gettimeofday(&t1, NULL);
 
     ret = client(opts, &infected, &err);
@@ -151,6 +171,9 @@ int main(int argc, char **argv)
 
     /* TODO: Implement STATUS in clamd */
     if (!optget(opts, "no-summary")->enabled) {
+        struct tm tmp;
+
+        date_end = time(NULL);
         gettimeofday(&t2, NULL);
         ds  = t2.tv_sec - t1.tv_sec;
         dms = t2.tv_usec - t1.tv_usec;
@@ -167,6 +190,26 @@ int main(int argc, char **argv)
             logg("Not moved: %d\n", notmoved);
         }
         logg("Time: %d.%3.3d sec (%d m %d s)\n", ds, dms / 1000, ds / 60, ds % 60);
+
+#ifdef _WIN32
+        if (0 != localtime_s(&tmp, &date_start)) {
+#else
+        if (!localtime_r(&date_start, &tmp)) {
+#endif
+            logg("!Failed to get local time for Start Date.\n");
+        }
+        strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
+        logg("Start Date: %s\n", buffer);
+
+#ifdef _WIN32
+        if (0 != localtime_s(&tmp, &date_end)) {
+#else
+        if (!localtime_r(&date_end, &tmp)) {
+#endif
+            logg("!Failed to get local time for End Date.\n");
+        }
+        strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
+        logg("End Date:   %s\n", buffer);
     }
 
     logg_close();
@@ -182,7 +225,7 @@ void help(void)
     mprintf("\n");
     mprintf("                      Clam AntiVirus: Daemon Client %s\n", get_version());
     mprintf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    mprintf("           (C) 2020 Cisco Systems, Inc.\n");
+    mprintf("           (C) 2021 Cisco Systems, Inc.\n");
     mprintf("\n");
     mprintf("    clamdscan [options] [file/directory/-]\n");
     mprintf("\n");
@@ -194,6 +237,8 @@ void help(void)
     mprintf("                                       (this help is always written to stdout)\n");
     mprintf("    --log=FILE          -l FILE        Save scan report in FILE\n");
     mprintf("    --file-list=FILE    -f FILE        Scan files from FILE\n");
+    mprintf("    --ping              -p A[:I]       Ping clamd up to [A] times at optional interval [I] until it responds.\n");
+    mprintf("    --wait              -w             Wait up to 30 seconds for clamd to start. Optionally use alongside --ping to set attempts [A] and interval [I] to check clamd.\n");
     mprintf("    --remove                           Remove infected files. Be careful!\n");
     mprintf("    --move=DIRECTORY                   Move infected files into DIRECTORY\n");
     mprintf("    --copy=DIRECTORY                   Copy infected files into DIRECTORY\n");
