@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -22,7 +22,10 @@
 #ifndef __CLIENT_H
 #define __CLIENT_H
 
-#include "shared/optparser.h"
+#define CLAMDSCAN_DEFAULT_PING_INTERVAL 1
+#define CLAMDSCAN_DEFAULT_PING_ATTEMPTS 31
+
+#include "optparser.h"
 
 enum {
     CONT,
@@ -36,5 +39,6 @@ enum {
 int client(const struct optstruct *opts, int *infected, int *err);
 int get_clamd_version(const struct optstruct *opts);
 int reload_clamd_database(const struct optstruct *opts);
+int16_t ping_clamd(const struct optstruct *opts);
 
 #endif
