@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -30,7 +30,7 @@
 #endif
 
 /* must be first because it may define _XOPEN_SOURCE */
-#include "shared/fdpassing.h"
+#include "fdpassing.h"
 #include <stdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -52,12 +52,15 @@
 #include <netdb.h>
 #endif
 
-#include "libclamav/clamav.h"
-#include "libclamav/others.h"
-#include "shared/actions.h"
-#include "shared/output.h"
-#include "shared/misc.h"
-#include "shared/clamdcom.h"
+// libclamav
+#include "clamav.h"
+#include "others.h"
+
+// shared
+#include "actions.h"
+#include "output.h"
+#include "misc.h"
+#include "clamdcom.h"
 
 #include "proto.h"
 #include "client.h"
@@ -148,11 +151,13 @@ static int send_stream(int sockd, const char *filename)
 
     if (filename) {
         if ((fd = safe_open(filename, O_RDONLY | O_BINARY)) < 0) {
-            logg("~%s: Access denied. ERROR\n", filename);
+            logg("~%s: Failed to open file. ERROR\n", filename);
             return 0;
         }
-    } else
+    } else {
+        /* Read stream from STDIN */
         fd = 0;
+    }
 
     if (sendln(sockd, "zINSTREAM", 10)) {
         close(fd);
@@ -196,7 +201,7 @@ static int send_fdpass(int sockd, const char *filename)
 
     if (filename) {
         if ((fd = open(filename, O_RDONLY)) < 0) {
-            logg("~%s: Access denied. ERROR\n", filename);
+            logg("~%s: Failed to open file\n", filename);
             return 0;
         }
     } else
@@ -263,8 +268,12 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
     struct RCVLN rcv;
     STATBUF sb;
 
-    if (filename && chkpath(filename))
-        return 0;
+    if (filename) {
+        if (1 == chkpath(filename)) {
+            goto done;
+        }
+    }
+
     recvlninit(&rcv, sockd);
 
     switch (scantype) {
@@ -273,17 +282,20 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
         case ALLMATCH:
             if (!filename) {
                 logg("Filename cannot be NULL for MULTISCAN or CONTSCAN.\n");
-                return -1;
+                infected = -1;
+                goto done;
             }
             len = strlen(filename) + strlen(scancmd[scantype]) + 3;
             if (!(bol = malloc(len))) {
                 logg("!Cannot allocate a command buffer: %s\n", strerror(errno));
-                return -1;
+                infected = -1;
+                goto done;
             }
             sprintf(bol, "z%s %s", scancmd[scantype], filename);
             if (sendln(sockd, bol, len)) {
                 free(bol);
-                return -1;
+                infected = -1;
+                goto done;
             }
             free(bol);
             break;
@@ -304,11 +316,15 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
         *printok = 0;
         if (errors)
             (*errors)++;
-        return len;
+        infected = len;
+        goto done;
     }
 
     while ((len = recvln(&rcv, &bol, &eol))) {
-        if (len == -1) return -1;
+        if (len == -1) {
+            infected = -1;
+            goto done;
+        }
         beenthere = 1;
         if (!filename) logg("~%s\n", bol);
         if (len > 7) {
@@ -328,7 +344,8 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
                          (scantype < 0 || scantype > MAX_SCANTYPE) ? "unidentified" : scancmd[scantype]);
                 else
                     logg("Failed to parse reply: \"%s\"\n", bol);
-                return -1;
+                infected = -1;
+                goto done;
             } else if (!memcmp(eol - 7, " FOUND", 6)) {
                 static char last_filename[PATH_MAX + 1] = {'\0'};
                 *(eol - 7)                              = 0;
@@ -369,18 +386,23 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
     if (!beenthere) {
         if (!filename) {
             logg("STDIN: noreply from clamd\n.");
-            return -1;
+            infected = -1;
+            goto done;
         }
         if (CLAMSTAT(filename, &sb) == -1) {
             logg("~%s: stat() failed with %s, clamd may not be responding\n",
                  filename, strerror(errno));
-            return -1;
+            infected = -1;
+            goto done;
         }
         if (!S_ISDIR(sb.st_mode)) {
             logg("~%s: no reply from clamd\n", filename);
-            return -1;
+            infected = -1;
+            goto done;
         }
     }
+
+done:
     return infected;
 }
 
@@ -395,59 +417,83 @@ struct client_serial_data {
 
 /* FTW callback for scanning in non IDSESSION mode
  * Returns SUCCESS or BREAK on success, CL_EXXX on error */
-static int serial_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
+static cl_error_t serial_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
 {
+    int status = CL_EOPEN;
+
     struct client_serial_data *c = (struct client_serial_data *)data->data;
     int sockd, ret;
-    const char *f = filename;
+    const char *f       = filename;
+    char *real_filename = NULL;
 
     UNUSEDPARAM(sb);
 
-    if (chkpath(path))
-        return CL_SUCCESS;
+    if (reason != visit_directory_toplev) {
+        if (CL_SUCCESS != cli_realpath((const char *)path, &real_filename)) {
+            logg("*Failed to determine real filename of %s.\n", path);
+            logg("*Quarantine of the file may fail if file path contains symlinks.\n");
+        } else {
+            path = real_filename;
+        }
+    }
+
+    if (chkpath(path)) {
+        status = CL_SUCCESS;
+        goto done;
+    }
     c->files++;
     switch (reason) {
         case error_stat:
             logg("!Can't access file %s\n", path);
             c->errors++;
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case error_mem:
             logg("!Memory allocation failed in ftw\n");
             c->errors++;
-            return CL_EMEM;
+            status = CL_EMEM;
+            goto done;
         case warning_skipped_dir:
             logg("^Directory recursion limit reached\n");
+            /* fall-through */
         case warning_skipped_link:
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case warning_skipped_special:
             logg("^%s: Not supported file type\n", path);
             c->errors++;
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case visit_directory_toplev:
-            if (c->scantype >= STREAM)
-                return CL_SUCCESS;
-            f        = path;
-            filename = NULL;
+            if (c->scantype >= STREAM) {
+                status = CL_SUCCESS;
+                goto done;
+            }
+            f = path;
         case visit_file:
             break;
     }
 
     if ((sockd = dconnect()) < 0) {
-        if (filename) free(filename);
         c->errors++;
-        return CL_EOPEN;
+        goto done;
     }
     ret = dsresult(sockd, c->scantype, f, &c->printok, &c->errors);
-    if (filename) free(filename);
     closesocket(sockd);
     if (ret < 0) {
         c->errors++;
-        return CL_EOPEN;
+        goto done;
     }
     c->infected += ret;
-    if (reason == visit_directory_toplev)
-        return CL_BREAK;
-    return CL_SUCCESS;
+    if (reason == visit_directory_toplev) {
+        status = CL_BREAK;
+        goto done;
+    }
+
+    status = CL_SUCCESS;
+done:
+    free(filename);
+    return status;
 }
 
 /* Non-IDSESSION handler
@@ -554,43 +600,64 @@ static int dspresult(struct client_parallel_data *c)
 
 /* FTW callback for scanning in IDSESSION mode
  * Returns SUCCESS on success, CL_EXXX or BREAK on error */
-static int parallel_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
+static cl_error_t parallel_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
 {
+    cl_error_t status = CL_EOPEN;
+
     struct client_parallel_data *c = (struct client_parallel_data *)data->data;
-    struct SCANID *cid;
-    int res = CL_CLEAN;
+    struct SCANID *cid             = NULL;
+    int res                        = CL_CLEAN;
+
+    char *real_filename = NULL;
 
     UNUSEDPARAM(sb);
+    UNUSEDPARAM(path);
 
-    if (chkpath(path))
-        return CL_SUCCESS;
+    if (reason != visit_directory_toplev) {
+        if (CL_SUCCESS != cli_realpath((const char *)filename, &real_filename)) {
+            logg("*Failed to determine real filename of %s.\n", filename);
+            logg("*Quarantine of the file may fail if file path contains symlinks.\n");
+        } else {
+            free(filename);
+            filename = real_filename;
+        }
+    }
+
+    if (chkpath(filename)) {
+        goto done;
+    }
     c->files++;
     switch (reason) {
         case error_stat:
-            logg("!Can't access file %s\n", path);
+            logg("!Can't access file %s\n", filename);
             c->errors++;
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case error_mem:
             logg("!Memory allocation failed in ftw\n");
             c->errors++;
-            return CL_EMEM;
+            status = CL_EMEM;
+            goto done;
         case warning_skipped_dir:
             logg("^Directory recursion limit reached\n");
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case warning_skipped_special:
-            logg("^%s: Not supported file type\n", path);
+            logg("^%s: Not supported file type\n", filename);
             c->errors++;
+            /* fall-through */
         case warning_skipped_link:
         case visit_directory_toplev:
-            return CL_SUCCESS;
+            status = CL_SUCCESS;
+            goto done;
         case visit_file:
             break;
     }
 
     while (1) {
         /* consume all the available input to let some of the clamd
-	 * threads blocked on send() to be dead.
-	 * by doing so we shouldn't deadlock on the next recv() */
+	     * threads blocked on send() to be dead.
+	     * by doing so we shouldn't deadlock on the next recv() */
         fd_set rfds, wfds;
         FD_ZERO(&rfds);
         FD_SET(c->sockd, &rfds);
@@ -598,31 +665,20 @@ static int parallel_callback(STATBUF *sb, char *filename, const char *path, enum
         FD_SET(c->sockd, &wfds);
         if (select(c->sockd + 1, &rfds, &wfds, NULL, NULL) < 0) {
             if (errno == EINTR) continue;
-            free(filename);
             logg("!select() failed during session: %s\n", strerror(errno));
-            return CL_BREAK;
+            status = CL_BREAK;
+            goto done;
         }
         if (FD_ISSET(c->sockd, &rfds)) {
             if (dspresult(c)) {
-                free(filename);
-                return CL_BREAK;
+                status = CL_BREAK;
+                goto done;
             } else
                 continue;
         }
         if (FD_ISSET(c->sockd, &wfds)) break;
     }
 
-    cid = (struct SCANID *)malloc(sizeof(struct SCANID));
-    if (!cid) {
-        free(filename);
-        logg("!Failed to allocate scanid entry: %s\n", strerror(errno));
-        return CL_BREAK;
-    }
-    cid->id   = ++c->lastid;
-    cid->file = filename;
-    cid->next = c->ids;
-    c->ids    = cid;
-
     switch (c->scantype) {
 #ifdef HAVE_FD_PASSING
         case FILDES:
@@ -636,13 +692,32 @@ static int parallel_callback(STATBUF *sb, char *filename, const char *path, enum
     if (res <= 0) {
         c->printok = 0;
         c->errors++;
-        c->ids = cid->next;
-        c->lastid--;
-        free(cid);
-        free(filename);
-        return res ? CL_BREAK : CL_SUCCESS;
+        status = res ? CL_BREAK : CL_SUCCESS;
+        goto done;
     }
-    return CL_SUCCESS;
+
+    cid = (struct SCANID *)malloc(sizeof(struct SCANID));
+    if (!cid) {
+        logg("!Failed to allocate scanid entry: %s\n", strerror(errno));
+        status = CL_BREAK;
+        goto done;
+    }
+
+    cid->id   = ++c->lastid;
+    cid->file = filename;
+    cid->next = c->ids;
+    c->ids    = cid;
+
+    /* Give up ownership of the filename to the client parralel scan ID list */
+    filename = NULL;
+
+    status = CL_SUCCESS;
+
+done:
+    if (NULL != filename) {
+        free(filename);
+    }
+    return status;
 }
 
 /* IDSESSION handler
