@@ -52,18 +52,21 @@
 #include <sys/uio.h>
 #endif
 
-#include "libclamav/clamav.h"
-#include "shared/optparser.h"
-#include "shared/output.h"
-#include "shared/misc.h"
-#include "shared/actions.h"
+// libclamav
+#include "clamav.h"
+#include "str.h"
+#include "others.h"
 
-#include "libclamav/str.h"
-#include "libclamav/others.h"
+// shared
+#include "optparser.h"
+#include "output.h"
+#include "misc.h"
+#include "actions.h"
 
 #include "communication.h"
 #include "client.h"
 #include "protocol.h"
+#include "socket.h"
 
 #include "../clamonacc.h"
 
@@ -81,13 +84,11 @@ void onas_print_server_version(struct onas_context **ctx)
  * Returns 0 if clamd is local, non zero if clamd is remote */
 int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
 {
-    int s, ret;
+    int ret;
     const struct optstruct *opt;
     CURL *curl;
     CURLcode curlcode;
     char *ipaddr = NULL;
-    struct addrinfo hints, *info, *p;
-    int res;
     int64_t timeout;
 
     timeout = optget((*ctx)->clamdopts, "OnAccessCurlTimeout")->numarg;
@@ -132,7 +133,11 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
 
         curlcode = curl_easy_perform(curl);
         if (CURLE_OK != curlcode) {
-            logg("!ClamClient: could not connect to remote clam daemon, %s\n", curl_easy_strerror(curlcode));
+            if (optget((*ctx)->opts, "ping")->enabled || optget((*ctx)->opts, "wait")->enabled) {
+                logg("~ClamClient: Initial connection failed, %s. Will try again...\n", curl_easy_strerror(curlcode));
+            } else {
+                logg("!ClamClient: Could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+            }
             *err = CL_EARG;
             return ret;
         }
@@ -154,6 +159,142 @@ int onas_check_remote(struct onas_context **ctx, cl_error_t *err)
     return ret;
 }
 
+/* pings clamd at the specified interval the number of time specified
+ * return 0 on a succesful connection, 1 upon timeout, -1 on error */
+int16_t onas_ping_clamd(struct onas_context **ctx)
+{
+
+    uint64_t attempts           = 0;
+    uint64_t interval           = 0;
+    char *attempt_str           = NULL;
+    char *interval_str          = NULL;
+    char *errchk                = NULL;
+    uint64_t i                  = 0;
+    const struct optstruct *opt = NULL;
+    CURL *curl                  = NULL;
+    CURLcode curlcode;
+    cl_error_t err = CL_SUCCESS;
+    int b_remote   = 0;
+    uint16_t ret   = 0;
+    int64_t timeout;
+
+    if (ctx == NULL) {
+        logg("!null parameter was passed\n");
+        return -1;
+    }
+
+    timeout = optget((*ctx)->clamdopts, "OnAccessCurlTimeout")->numarg;
+
+    b_remote = onas_check_remote(ctx, &err);
+    if (CL_SUCCESS != err) {
+        logg("*ClamClient: could not check to see if daemon was remote... PINGing again...\n");
+    }
+
+    if (!b_remote) {
+        curlcode = onas_curl_init(&curl, optget((*ctx)->clamdopts, "LocalSocket")->strarg, 0, timeout);
+    } else {
+        curlcode = onas_curl_init(&curl, optget((*ctx)->clamdopts, "TCPAddr")->strarg, (*ctx)->portnum, timeout);
+        if (CURLE_OK != curlcode) {
+            logg("!ClamClient: could not setup curl with tcp address and port, %s\n", curl_easy_strerror(curlcode));
+            /* curl cleanup done in onas_curl_init on error */
+            ret = -1;
+            goto done;
+        }
+    }
+
+    /* ping command takes the form --ping [attempts[:interval]] */
+    opt = optget((*ctx)->opts, "ping");
+
+    if (opt) {
+        attempt_str = cli_strdup(opt->strarg);
+        if (attempt_str) {
+            if (NULL == attempt_str) {
+                logg("!could not allocate memory for string\n");
+                ret = -1;
+                goto done;
+            }
+            interval_str = strchr(attempt_str, ':');
+            if ((NULL != interval_str) && (interval_str[0] != '\0')) {
+                interval_str[0] = '\0';
+                interval_str++;
+                interval = cli_strntoul(interval_str, strlen(interval_str), &errchk, 10);
+                if (interval_str + strlen(interval_str) > errchk) {
+                    logg("^interval_str would go past end of buffer\n");
+                    ret = -1;
+                    goto done;
+                }
+            } else {
+                interval = ONAS_DEFAULT_PING_INTERVAL;
+            }
+            attempts = cli_strntoul(attempt_str, strlen(attempt_str), &errchk, 10);
+            if (attempt_str + strlen(attempt_str) > errchk) {
+                logg("^attempt_str would go past end of buffer\n");
+                ret = -1;
+                goto done;
+            }
+        } else {
+            attempts = ONAS_DEFAULT_PING_ATTEMPTS;
+            interval = ONAS_DEFAULT_PING_INTERVAL;
+        }
+    }
+
+    do {
+        curlcode = curl_easy_perform(curl);
+        if (CURLE_OK != curlcode) {
+            logg("*ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
+        } else if (CURLE_OK == onas_sendln(curl, "zPING", 5, timeout)) {
+
+            if (!optget((*ctx)->opts, "wait")->enabled) {
+                logg("PONG\n");
+            } else {
+                logg("*ClamClient: Connected.\n");
+            }
+
+            ret = 0;
+            goto done;
+        }
+
+        if (i + 1 < attempts) {
+            if (optget((*ctx)->opts, "wait")->enabled) {
+                if (interval == 1)
+                    logg("*Will try again in %lu second\n", interval);
+                else
+                    logg("*Will try again in %lu seconds\n", interval);
+            } else {
+                if (interval == 1)
+                    logg("PINGing again in %lu second\n", interval);
+                else
+                    logg("PINGing again in %lu seconds\n", interval);
+            }
+            sleep(interval);
+        }
+        i++;
+    } while (i < attempts);
+
+    /* timed out */
+    ret = 1;
+    if (optget((*ctx)->opts, "wait")->enabled) {
+        logg("Wait timeout exceeded; Could not connect to clamd\n");
+    } else {
+        logg("PING timeout exceeded; No response from clamd\n");
+    }
+
+done:
+    if (curl) {
+        curl_easy_cleanup(curl);
+    }
+
+    if (attempt_str) {
+        free(attempt_str);
+    }
+
+    attempt_str  = NULL;
+    interval_str = NULL;
+    errchk       = NULL;
+
+    return ret;
+}
+
 /**
  * @brief initialises a curl connection for the onaccess client; curl must be initialised globally before use
  *
@@ -177,8 +318,10 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
 
     if (!port) {
 
+#if ((LIBCURL_VERSION_MAJOR > 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 40))
         /* "ipaddr" is actually our unix socket path here */
         curlcode = curl_easy_setopt(*curl, CURLOPT_UNIX_SOCKET_PATH, ipaddr);
+#endif
         if (CURLE_OK != curlcode) {
             logg("!ClamClient: could not setup curl with local unix socket, %s\n", curl_easy_strerror(curlcode));
             curl_easy_cleanup(*curl);
@@ -270,11 +413,6 @@ cl_error_t onas_setup_client(struct onas_context **ctx)
 
     opts = (*ctx)->opts;
 
-    if (optget(opts, "verbose")->enabled) {
-        mprintf_verbose = 1;
-        logg_verbose    = 1;
-    }
-
     if (optget(opts, "infected")->enabled) {
         (*ctx)->printinfected = 1;
     }
@@ -308,6 +446,9 @@ cl_error_t onas_setup_client(struct onas_context **ctx)
     remote = (*ctx)->isremote | optget(opts, "stream")->enabled;
 #ifdef HAVE_FD_PASSING
     if (!remote && optget((*ctx)->clamdopts, "LocalSocket")->enabled && (optget(opts, "fdpass")->enabled)) {
+        if (onas_set_sock_only_once(*ctx) == CL_EWRITE) {
+            return CL_EWRITE;
+        }
         logg("*ClamClient: client setup to scan via fd passing\n");
         (*ctx)->scantype = FILDES;
         (*ctx)->session  = optget(opts, "multiscan")->enabled;
@@ -340,8 +481,8 @@ int onas_get_clamd_version(struct onas_context **ctx)
     CURLcode curlcode;
     cl_error_t err = CL_SUCCESS;
     int b_remote;
-    int len, sockd;
-    struct RCVLN rcv;
+    int len;
+    struct onas_rcvln rcv;
     int64_t timeout;
 
     timeout = optget((*ctx)->clamdopts, "OnAccessCurlTimeout")->numarg;
@@ -363,11 +504,11 @@ int onas_get_clamd_version(struct onas_context **ctx)
         }
     }
 
-    onas_recvlninit(&rcv, curl);
+    onas_recvlninit(&rcv, curl, 0);
 
     curlcode = curl_easy_perform(curl);
     if (CURLE_OK != curlcode) {
-        logg("*ClamClient: could not connect to clam daemon, %s\n", curl_easy_strerror(curlcode));
+        logg("*ClamClient: could not connect to clamd, %s\n", curl_easy_strerror(curlcode));
         return 2;
     }
 
@@ -408,7 +549,8 @@ int onas_client_scan(const char *tcpaddr, int64_t portnum, int32_t scantype, uin
     CURL *curl        = NULL;
     CURLcode curlcode = CURLE_OK;
     int errors        = 0;
-    int sockd, ret;
+    int ret;
+    static bool disconnected = false;
 
     *infected = 0;
 
@@ -425,9 +567,16 @@ int onas_client_scan(const char *tcpaddr, int64_t portnum, int32_t scantype, uin
 
     curlcode = curl_easy_perform(curl);
     if (CURLE_OK != curlcode) {
-        logg("!ClamClient: could not establish connection, %s\n", curl_easy_strerror(curlcode));
+        if (!disconnected) {
+            logg("!ClamClient: Connection to clamd failed, %s.\n", curl_easy_strerror(curlcode));
+            disconnected = true;
+        }
         return CL_ECREAT;
     }
+    if (disconnected) {
+        logg("~ClamClient: Connection to clamd re-established.\n");
+        disconnected = false;
+    }
 
     if ((ret = onas_dsresult(curl, scantype, maxstream, fname, fd, timeout, &ret, err, ret_code)) >= 0) {
         *infected = ret;
