@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -24,9 +24,12 @@
 
 #include <curl/curl.h>
 
-#include "shared/optparser.h"
+#include "optparser.h"
 #include "../clamonacc.h"
 
+#define ONAS_DEFAULT_PING_INTERVAL 1
+#define ONAS_DEFAULT_PING_ATTEMPTS 31
+
 enum {
     CONT,
     MULTI,
@@ -42,5 +45,6 @@ CURLcode onas_curl_init(CURL **curl, const char *ipaddr, int64_t port, int64_t t
 int onas_get_clamd_version(struct onas_context **ctx);
 cl_error_t onas_setup_client(struct onas_context **ctx);
 int onas_check_remote(struct onas_context **ctx, cl_error_t *err);
+int16_t onas_ping_clamd(struct onas_context **ctx);
 
 #endif
