@@ -30,10 +30,11 @@
 #include <sys/param.h>
 #endif
 
-#include "shared/misc.h"
+#include "misc.h"
 
-struct RCVLN {
+struct onas_rcvln {
     char buf[PATH_MAX + 1024];
+    int sockd;
     CURL *curl;
     CURLcode curlcode;
     size_t retlen;
@@ -42,7 +43,8 @@ struct RCVLN {
 };
 
 int onas_sendln(CURL *curl, const void *line, size_t len, int64_t timeout_ms);
-void onas_recvlninit(struct RCVLN *s, CURL *curl);
-int onas_recvln(struct RCVLN *rcv_data, char **ret_bol, char **ret_eol, int64_t timeout_ms);
+void onas_recvlninit(struct onas_rcvln *s, CURL *curl, int sockd);
+int onas_recvln(struct onas_rcvln *rcv_data, char **ret_bol, char **ret_eol, int64_t timeout_ms);
+int onas_fd_recvln(struct onas_rcvln *rcv_data, char **ret_bol, char **ret_eol, int64_t timeout_ms);
 
 #endif
