@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2019-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Mickey Sola
  *
@@ -34,31 +34,32 @@
 #include <errno.h>
 #include <stdbool.h>
 
-#if defined(FANOTIFY)
+#if defined(HAVE_SYS_FANOTIFY_H)
 #include <sys/fanotify.h>
 #include <sys/inotify.h>
 #endif
 
-#include "../fanotif/fanotif.h"
+// libclamav
+#include "clamav.h"
+#include "scanners.h"
 
+// shared
+#include "optparser.h"
+#include "output.h"
+
+// clamd
+#include "server.h"
+#include "clamd_others.h"
+#include "scanner.h"
+
+#include "../fanotif/fanotif.h"
 #include "hash.h"
 #include "inotif.h"
-
 #include "../scan/thread.h"
-#include "../scan/queue.h"
+#include "../scan/onas_queue.h"
 #include "../misc/utils.h"
 
-#include "libclamav/clamav.h"
-#include "libclamav/scanners.h"
-
-#include "shared/optparser.h"
-#include "shared/output.h"
-
-#include "clamd/server.h"
-#include "clamd/others.h"
-#include "clamd/scanner.h"
-
-#if defined(FANOTIFY)
+#if defined(HAVE_SYS_FANOTIFY_H)
 
 static int onas_ddd_init_ht(uint32_t ht_size);
 static int onas_ddd_init_wdlt(uint64_t nwatches);
@@ -71,6 +72,7 @@ static int onas_ddd_unwatch_hierarchy(const char *pathname, size_t len, int fd,
 
 static void onas_ddd_handle_in_moved_to(struct onas_context *ctx, const char *path, const char *child_path, const struct inotify_event *event, int wd, uint64_t in_mask);
 static void onas_ddd_handle_in_create(struct onas_context *ctx, const char *path, const char *child_path, const struct inotify_event *event, int wd, uint64_t in_mask);
+static void onas_ddd_handle_in_close_write(struct onas_context *ctx, const char *child_path);
 static void onas_ddd_handle_in_moved_from(struct onas_context *ctx, const char *path, const char *child_path, const struct inotify_event *event, int wd);
 static void onas_ddd_handle_in_delete(struct onas_context *ctx, const char *path, const char *child_path, const struct inotify_event *event, int wd);
 static void onas_ddd_handle_extra_scanning(struct onas_context *ctx, const char *pathname, int extra_options);
@@ -359,9 +361,10 @@ void *onas_ddd_th(void *arg)
 {
     struct onas_context *ctx = (struct onas_context *)arg;
     sigset_t sigset;
-    struct sigaction act;
     const struct optstruct *pt;
-    uint64_t in_mask = IN_ONLYDIR | IN_MOVE | IN_DELETE | IN_CREATE;
+    const struct optstruct *pt_tmpdir;
+    const char *clamd_tmpdir;
+    uint64_t in_mask = IN_ONLYDIR | IN_MOVE | IN_DELETE | IN_CREATE | IN_CLOSE_WRITE;
     fd_set rfds;
     char buf[4096];
     ssize_t bread;
@@ -409,15 +412,31 @@ void *onas_ddd_th(void *arg)
         return NULL;
     }
 
+    pt_tmpdir = optget(ctx->clamdopts, "TemporaryDirectory");
+    if (pt_tmpdir->enabled) {
+        clamd_tmpdir = pt_tmpdir->strarg;
+    } else {
+        clamd_tmpdir = cli_gettmpdir();
+    }
+
     if ((pt = optget(ctx->clamdopts, "OnAccessIncludePath"))->enabled) {
 
         while (pt) {
             if (!strcmp(pt->strarg, "/")) {
-                logg("!ClamInotif: not including path '%s' while DDD is enabled\n", pt->strarg);
-                logg("!ClamInotif: please use the OnAccessMountPath option to watch '%s'\n", pt->strarg);
+                logg("!ClamInotif: Not watching path '%s' while DDD is enabled\n", pt->strarg);
+                logg("!ClamInotif: Please use the OnAccessMountPath option to watch '%s'\n", pt->strarg);
                 pt = (struct optstruct *)pt->nextarg;
                 continue;
             }
+
+            if (0 == strcmp(clamd_tmpdir, pt->strarg)) {
+                logg("!ClamInotif: Not watching path '%s'\n", pt->strarg);
+                logg("!ClamInotif: ClamOnAcc should not watch the directory clamd is using for temp files\n");
+                logg("!ClamInotif: Consider setting TemporaryDirectory in clamd.conf to a different directory.\n");
+                pt = (struct optstruct *)pt->nextarg;
+                continue;
+            }
+
             if (onas_ht_get(ddd_ht, pt->strarg, strlen(pt->strarg), NULL) != CL_SUCCESS) {
                 if (onas_ht_add_hierarchy(ddd_ht, pt->strarg)) {
                     logg("!ClamInotif: can't include '%s'\n", pt->strarg);
@@ -445,6 +464,21 @@ void *onas_ddd_th(void *arg)
         idx = 0;
         while (NULL != include_list[idx]) {
             if (onas_ht_get(ddd_ht, include_list[idx], strlen(include_list[idx]), NULL) != CL_SUCCESS) {
+                if (!strcmp(include_list[idx], "/")) {
+                    logg("!ClamInotif: Not watching path '%s' while DDD is enabled\n", include_list[idx]);
+                    logg("!ClamInotif: Please use the OnAccessMountPath option to watch '%s'\n", include_list[idx]);
+                    pt = (struct optstruct *)pt->nextarg;
+                    continue;
+                }
+
+                if (0 == strcmp(clamd_tmpdir, include_list[idx])) {
+                    logg("!ClamInotif: Not watching path '%s'\n", include_list[idx]);
+                    logg("!ClamInotif: ClamOnAcc should not watch the directory clamd is using for temp files\n");
+                    logg("!ClamInotif: Consider setting TemporaryDirectory in clamd.conf to a different directory.\n");
+                    pt = (struct optstruct *)pt->nextarg;
+                    continue;
+                }
+
                 if (onas_ht_add_hierarchy(ddd_ht, include_list[idx])) {
                     logg("!ClamInotif: can't include '%s'\n", include_list[idx]);
                     return NULL;
@@ -499,13 +533,22 @@ void *onas_ddd_th(void *arg)
         }
     }
 
+    /* Also remove the clamd temp directory, in case its parent directory was watched */
+    logg("*Excluding temp directory: %s\n", clamd_tmpdir);
+    if (onas_ht_rm_hierarchy(ddd_ht, clamd_tmpdir, strlen(clamd_tmpdir), 0)) {
+        logg("*ClamInotif: NVM, didn't actually need to exclude '%s'\n", clamd_tmpdir);
+    } else {
+        logg("ClamInotif: excluding '%s' (and all sub-directories)\n", clamd_tmpdir);
+    }
+
     /* Watch provided paths recursively */
     if ((pt = optget(ctx->clamdopts, "OnAccessIncludePath"))->enabled) {
         while (pt) {
             errno        = 0;
             size_t ptlen = strlen(pt->strarg);
             if (onas_ht_get(ddd_ht, pt->strarg, ptlen, NULL) == CL_SUCCESS) {
-                if (err = onas_ddd_watch(pt->strarg, ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask)) {
+                err = onas_ddd_watch(pt->strarg, ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask);
+                if (err) {
 
                     if (0 == errno) {
                         logg("!ClamInotif: could not watch path '%s', %d\n ", pt->strarg, err);
@@ -536,7 +579,8 @@ void *onas_ddd_th(void *arg)
             errno          = 0;
             uint64_t ptlen = strlen(include_list[idx]);
             if (onas_ht_get(ddd_ht, include_list[idx], ptlen, NULL) == CL_SUCCESS) {
-                if (err = onas_ddd_watch(include_list[idx], ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask)) {
+                err = onas_ddd_watch(include_list[idx], ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask);
+                if (err) {
                     if (0 == errno) {
                         logg("!ClamInotif: could not watch path '%s', %d\n ", include_list[idx], err);
                     } else {
@@ -616,6 +660,9 @@ void *onas_ddd_th(void *arg)
                 } else if (event->mask & IN_CREATE) {
                     onas_ddd_handle_in_create(ctx, path, child_path, event, wd, in_mask);
 
+                } else if (event->mask & IN_CLOSE_WRITE) {
+                    onas_ddd_handle_in_close_write(ctx, child_path);
+
                 } else if (event->mask & IN_MOVED_TO) {
                     onas_ddd_handle_in_moved_to(ctx, path, child_path, event, wd, in_mask);
                 }
@@ -665,26 +712,29 @@ static void onas_ddd_handle_in_create(struct onas_context *ctx,
                                       const char *path, const char *child_path, const struct inotify_event *event, int wd, uint64_t in_mask)
 {
 
+    if (!(event->mask & IN_ISDIR)){
+        return;
+    }
+
+    if (optget(ctx->clamdopts, "OnAccessExtraScanning")->enabled) {
+        logg("*ClamInotif: CREATE - adding %s to %s with wd:%d\n", child_path, path, wd);
+        onas_ddd_handle_extra_scanning(ctx, child_path, ONAS_SCTH_B_DIR);
+    }
+
+    onas_ht_add_hierarchy(ddd_ht, child_path);
+    onas_ddd_watch(child_path, ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask);
+
+    return;
+}
+
+static void onas_ddd_handle_in_close_write(struct onas_context *ctx, const char *child_path)
+{
     struct stat s;
 
     if (optget(ctx->clamdopts, "OnAccessExtraScanning")->enabled) {
         if (stat(child_path, &s) == 0 && S_ISREG(s.st_mode)) {
             onas_ddd_handle_extra_scanning(ctx, child_path, ONAS_SCTH_B_FILE);
-
-        } else if (event->mask & IN_ISDIR) {
-            logg("*ClamInotif: CREATE - adding %s to %s with wd:%d\n", child_path, path, wd);
-            onas_ddd_handle_extra_scanning(ctx, child_path, ONAS_SCTH_B_DIR);
-
-            onas_ht_add_hierarchy(ddd_ht, child_path);
-            onas_ddd_watch(child_path, ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask);
         }
-    } else {
-        if (stat(child_path, &s) == 0 && S_ISREG(s.st_mode)) return;
-        if (!(event->mask & IN_ISDIR)) return;
-
-        logg("*ClamInotif: MOVED_TO - adding %s to %s with wd:%d\n", child_path, path, wd);
-        onas_ht_add_hierarchy(ddd_ht, child_path);
-        onas_ddd_watch(child_path, ctx->fan_fd, ctx->fan_mask, onas_in_fd, in_mask);
     }
 
     return;
@@ -750,6 +800,8 @@ static void onas_ddd_handle_extra_scanning(struct onas_context *ctx, const char
 
 static void onas_ddd_exit(void *arg)
 {
+    UNUSEDPARAM(arg);
+
     logg("*ClamInotif: onas_ddd_exit()\n");
 
     if (onas_in_fd) {
