@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2019-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Mickey Sola
  *
@@ -21,8 +21,12 @@
 #ifndef __CLAMD_ONAS_OTHERS_H
 #define __CLAMD_ONAS_OTHERS_H
 
-#include "shared/optparser.h"
-#include "libclamav/clamav.h"
+// libclamav
+#include "clamav.h"
+
+// shared
+#include "optparser.h"
+
 #include "../clamonacc.h"
 
 typedef enum {
@@ -31,7 +35,7 @@ typedef enum {
     CHK_SELF
 } cli_check_t;
 
-#if defined(FANOTIFY)
+#if defined(HAVE_SYS_FANOTIFY_H)
 int onas_fan_checkowner(int pid, const struct optstruct *opts);
 #endif
 char **onas_get_opt_list(const char *fname, int *num_entries, cl_error_t *err);
