@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+#include <locale.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -39,18 +40,20 @@
 #include <sys/resource.h>
 #endif
 
+// libclamav
+#include "clamav.h"
 #include "others.h"
+#include "str.h"
+
+// shared
+#include "misc.h"
+#include "output.h"
+#include "actions.h"
+#include "optparser.h"
+
 #include "global.h"
 #include "manager.h"
 
-#include "shared/misc.h"
-#include "shared/output.h"
-#include "shared/actions.h"
-#include "shared/optparser.h"
-
-#include "libclamav/str.h"
-#include "libclamav/clamav.h"
-
 void help(void);
 
 struct s_info info;
@@ -62,6 +65,9 @@ int main(int argc, char **argv)
     int ds, dms, ret;
     double mb, rmb;
     struct timeval t1, t2;
+    time_t date_start, date_end;
+
+    char buffer[26];
 #ifndef _WIN32
     sigset_t sigset;
 #endif
@@ -71,11 +77,16 @@ int main(int argc, char **argv)
     if (check_flevel())
         exit(2);
 
-#if !defined(_WIN32) && !defined(C_BEOS)
+#if !defined(_WIN32)
+    if (!setlocale(LC_CTYPE, "")) {
+        mprintf("^Failed to set locale\n");
+    }
+#if !defined(C_BEOS)
     sigemptyset(&sigset);
     sigaddset(&sigset, SIGXFSZ);
     sigprocmask(SIG_SETMASK, &sigset, NULL);
-#endif
+#endif /* !C_BEOS */
+#endif /* !_WIN32 */
 
     cl_initialize_crypto();
 
@@ -154,13 +165,16 @@ int main(int argc, char **argv)
 
     memset(&info, 0, sizeof(struct s_info));
 
+    date_start = time(NULL);
     gettimeofday(&t1, NULL);
 
     ret = scanmanager(opts);
 
     if (!optget(opts, "no-summary")->enabled) {
-        gettimeofday(&t2, NULL);
+        struct tm tmp;
 
+        date_end = time(NULL);
+        gettimeofday(&t2, NULL);
         ds  = t2.tv_sec - t1.tv_sec;
         dms = t2.tv_usec - t1.tv_usec;
         ds -= (dms < 0) ? (1) : (0);
@@ -184,6 +198,26 @@ int main(int argc, char **argv)
         rmb = info.rblocks * (CL_COUNT_PRECISION / 1024) / 1024.0;
         logg("Data read: %2.2lf MB (ratio %.2f:1)\n", rmb, info.rblocks ? (double)info.blocks / (double)info.rblocks : 0);
         logg("Time: %u.%3.3u sec (%u m %u s)\n", ds, dms / 1000, ds / 60, ds % 60);
+
+#ifdef _WIN32
+        if (0 != localtime_s(&tmp, &date_start)) {
+#else
+        if (!localtime_r(&date_start, &tmp)) {
+#endif
+            logg("!Failed to get local time for Start Date.\n");
+        }
+        strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
+        logg("Start Date: %s\n", buffer);
+
+#ifdef _WIN32
+        if (0 != localtime_s(&tmp, &date_end)) {
+#else
+        if (!localtime_r(&date_end, &tmp)) {
+#endif
+            logg("!Failed to get local time for End Date.\n");
+        }
+        strftime(buffer, sizeof(buffer), "%Y:%m:%d %H:%M:%S", &tmp);
+        logg("End Date:   %s\n", buffer);
     }
 
     optfree(opts);
@@ -198,7 +232,7 @@ void help(void)
     mprintf("\n");
     mprintf("                       Clam AntiVirus: Scanner %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2020 Cisco Systems, Inc.\n");
+    printf("           (C) 2021 Cisco Systems, Inc.\n");
     mprintf("\n");
     mprintf("    clamscan [options] [file/directory/-]\n");
     mprintf("\n");
@@ -237,6 +271,8 @@ void help(void)
     mprintf("\n");
     mprintf("    --bytecode[=yes(*)/no]               Load bytecode from the database\n");
     mprintf("    --bytecode-unsigned[=yes/no(*)]      Load unsigned bytecode\n");
+    mprintf("                                         **Caution**: You should NEVER run bytecode signatures from untrusted sources.\n");
+    mprintf("                                         Doing so may result in arbitrary code execution.\n");
     mprintf("    --bytecode-timeout=N                 Set bytecode timeout (in milliseconds)\n");
     mprintf("    --statistics[=none(*)/bytecode/pcre] Collect and print execution statistics\n");
     mprintf("    --detect-pua[=yes/no(*)]             Detect Possibly Unwanted Applications\n");
@@ -246,6 +282,7 @@ void help(void)
     mprintf("    --structured-ssn-format=X            SSN format (0=normal,1=stripped,2=both)\n");
     mprintf("    --structured-ssn-count=N             Min SSN count to generate a detect\n");
     mprintf("    --structured-cc-count=N              Min CC count to generate a detect\n");
+    mprintf("    --structured-cc-mode=X               CC mode (0=credit debit and private label, 1=credit cards only\n");
     mprintf("    --scan-mail[=yes(*)/no]              Scan mail files\n");
     mprintf("    --phishing-sigs[=yes(*)/no]          Enable email signature-based phishing detection\n");
     mprintf("    --phishing-scan-urls[=yes(*)/no]     Enable URL signature-based phishing detection\n");
@@ -262,6 +299,7 @@ void help(void)
     mprintf("    --scan-hwp3[=yes(*)/no]              Scan HWP3 files\n");
     mprintf("    --scan-archive[=yes(*)/no]           Scan archive files (supported by libclamav)\n");
     mprintf("    --alert-broken[=yes/no(*)]           Alert on broken executable files (PE & ELF)\n");
+    mprintf("    --alert-broken-media[=yes/no(*)]     Alert on broken graphics files (JPEG, TIFF, PNG, GIF)\n");
     mprintf("    --alert-encrypted[=yes/no(*)]        Alert on encrypted archives and documents\n");
     mprintf("    --alert-encrypted-archive[=yes/no(*)] Alert on encrypted archives\n");
     mprintf("    --alert-encrypted-doc[=yes/no(*)]    Alert on encrypted documents\n");
@@ -273,7 +311,7 @@ void help(void)
     mprintf("    --nocerts                            Disable authenticode certificate chain verification in PE files\n");
     mprintf("    --dumpcerts                          Dump authenticode certificate chain in PE files\n");
     mprintf("\n");
-    mprintf("    --max-scantime=#n                    Scan time longer than this will be skipped and assumed clean\n");
+    mprintf("    --max-scantime=#n                    Scan time longer than this will be skipped and assumed clean (milliseconds)\n");
     mprintf("    --max-filesize=#n                    Files larger than this will be skipped and assumed clean\n");
     mprintf("    --max-scansize=#n                    The maximum amount of data to scan for each container file (**)\n");
     mprintf("    --max-files=#n                       The maximum number of files to scan for each container file (**)\n");
