@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *  Copyright (C) 2002-2007 Tomasz Kojm <tkojm@clamav.net>
  *
@@ -22,8 +22,6 @@
 #include "clamav-config.h"
 #endif
 
-#include "libfreshclam/libfreshclam.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -51,15 +49,19 @@
 #endif
 
 #include "target.h"
+
+// libclamav
 #include "clamav.h"
-#include "libfreshclam/libfreshclam.h"
+#include "others.h"
+#include "str.h"
 
-#include "libclamav/others.h"
-#include "libclamav/str.h"
+// shared
+#include "optparser.h"
+#include "output.h"
+#include "misc.h"
 
-#include "shared/optparser.h"
-#include "shared/output.h"
-#include "shared/misc.h"
+// libfreshclam
+#include "libfreshclam.h"
 
 #include "execute.h"
 #include "notify.h"
@@ -120,18 +122,35 @@ sighandler(int sig)
     return;
 }
 
-static void writepid(const char *pidfile)
+static int writepid(const char *pidfile)
 {
     FILE *fd;
     int old_umask;
-    old_umask = umask(0006);
+    old_umask = umask(0002);
     if ((fd = fopen(pidfile, "w")) == NULL) {
         logg("!Can't save PID to file %s: %s\n", pidfile, strerror(errno));
+        return 1;
     } else {
         fprintf(fd, "%d\n", (int)getpid());
         fclose(fd);
     }
     umask(old_umask);
+
+#ifndef _WIN32
+    /*If the file has already been created by a different user, it will just be
+     * rewritten by us, but not change the ownership, so do that explicitly.
+     */
+    if (0 == geteuid()) {
+        struct passwd *pw = getpwuid(0);
+        int ret           = lchown(pidfile, pw->pw_uid, pw->pw_gid);
+        if (ret) {
+            logg("!Can't change ownership of PID file %s '%s'\n", pidfile, strerror(errno));
+            return 1;
+        }
+    }
+#endif /*_WIN32 */
+
+    return 0;
 }
 
 static void help(void)
@@ -139,7 +158,7 @@ static void help(void)
     printf("\n");
     printf("                      Clam AntiVirus: Database Updater %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2020 Cisco Systems, Inc.\n");
+    printf("           (C) 2021 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    freshclam [options]\n");
     printf("\n");
@@ -163,11 +182,10 @@ static void help(void)
     printf("    --no-dns                             Force old non-DNS verification method\n");
     printf("    --checks=#n          -c #n           Number of checks per day, 1 <= n <= 50\n");
     printf("    --datadir=DIRECTORY                  Download new databases into DIRECTORY\n");
-#ifdef BUILD_CLAMD
     printf("    --daemon-notify[=/path/clamd.conf]   Send RELOAD command to clamd\n");
-#endif
     printf("    --local-address=IP   -a IP           Bind to IP for HTTP downloads\n");
-    printf("    --on-update-execute=COMMAND          Execute COMMAND after successful update\n");
+    printf("    --on-update-execute=COMMAND          Execute COMMAND after successful update.\n");
+    printf("                                         Use EXIT_1 to return 1 after successful database update.\n");
     printf("    --on-error-execute=COMMAND           Execute COMMAND if errors occurred\n");
     printf("    --on-outdated-execute=COMMAND        Execute COMMAND when software is outdated\n");
     printf("    --update-db=DBNAME                   Only update database DBNAME\n");
@@ -212,7 +230,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
     fc_error_t ret;
     fc_ctx *fc_context = (fc_ctx *)context;
 
-#ifndef WIN32
+#ifndef _WIN32
     char firstline[256];
     char lastline[256];
     int pipefd[2];
@@ -233,7 +251,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
     logg("Testing database: '%s' ...\n", dbFilename);
 
     if (fc_context->bTestDatabases) {
-#ifdef WIN32
+#ifdef _WIN32
 
         __try {
             ret = fc_test_database(dbFilename, fc_context->bBytecodeEnabled);
@@ -243,7 +261,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
             ret = FC_ETESTFAIL;
         }
         if (FC_SUCCESS != ret) {
-            logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+            logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
             status = FC_ETESTFAIL;
             goto done;
         }
@@ -258,11 +276,19 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
             logg("^pipe() failed: %s\n", strerror(errno));
             ret = fc_test_database(dbFilename, fc_context->bBytecodeEnabled);
             if (FC_SUCCESS != ret) {
-                logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                 status = FC_ETESTFAIL;
                 goto done;
             }
         } else {
+            /*
+             * Attempt to test database in a child process.
+             */
+
+            /* We need to be able to wait for the child process ourselves.
+             * We'll re-enable wait in the global handler when we're done. */
+            g_sigchildWait = 0;
+
             switch (pid = fork()) {
                 case -1: {
                     /*
@@ -276,10 +302,11 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
                     /* Test the database without forking. */
                     ret = fc_test_database(dbFilename, fc_context->bBytecodeEnabled);
                     if (FC_SUCCESS != ret) {
-                        logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                        logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                         status = FC_ETESTFAIL;
                         goto done;
                     }
+                    break;
                 }
                 case 0: {
                     /*
@@ -340,7 +367,7 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
                     if (WIFEXITED(stat_loc)) {
                         ret = (fc_error_t)WEXITSTATUS(stat_loc);
                         if (FC_SUCCESS != ret) {
-                            logg("^Database load exited with \"%s\" (%d)\n", fc_strerror(ret), ret);
+                            logg("^Database load exited with \"%s\"\n", fc_strerror(ret));
                             status = FC_ETESTFAIL;
                             goto done;
                         }
@@ -373,6 +400,7 @@ done:
         logg("!Database test FAILED.\n");
     }
 
+    /* Re-enable the global handler's child process wait */
     g_sigchildWait = 1;
 
     return status;
@@ -450,13 +478,6 @@ static fc_error_t get_server_node(
     status     = FC_SUCCESS;
 
 done:
-
-    if (FC_SUCCESS != status) {
-        if (NULL != url) {
-            free(url);
-        }
-    }
-
     return status;
 }
 
@@ -534,7 +555,7 @@ static void free_string_list(char **stringList, uint32_t nListItems)
 /**
  * @brief Get the database server list object
  *
- * @param opts          Freshclam options struct.
+ * @param opts          FreshClam options struct.
  * @param serverList    [out] List of servers.
  * @param nServers      [out] Number of servers in list.
  * @param bPrivate      [out] Non-zero if PrivateMirror servers were selected.
@@ -632,68 +653,6 @@ done:
     return status;
 }
 
-/**
- * @brief Switch users to the DatabaseOwner, if specified.
- *
- * @param dbowner     A user account that will have write permissions in the database directory.
- * @return fc_error_t FC_SUCCESS if success.
- * @return fc_error_t FC_EARG if success.
- */
-static fc_error_t switch_user(const char *dbowner)
-{
-    fc_error_t status = FC_EARG;
-
-#ifdef HAVE_PWD_H
-    struct passwd *user;
-
-    if (NULL == dbowner) {
-        logg("*No DatabaseOwner specified. Freshclam wlil run as the current user.\n");
-        status = FC_SUCCESS;
-        goto done;
-    }
-
-    if (!geteuid()) {
-        if ((user = getpwnam(dbowner)) == NULL) {
-            logg("^Can't get information about user %s.\n", dbowner);
-            status = FC_ECONFIG;
-            goto done;
-        }
-
-#ifdef HAVE_INITGROUPS
-        if (initgroups(dbowner, user->pw_gid)) {
-            logg("^initgroups() failed.\n");
-            status = FC_ECONFIG;
-            goto done;
-        }
-#elif HAVE_SETGROUPS
-        if (setgroups(1, &user->pw_gid)) {
-            logg("^setgroups() failed.\n");
-            status = FC_ECONFIG;
-            goto done;
-        }
-#endif
-
-        if (setgid(user->pw_gid)) {
-            logg("^setgid(%d) failed.\n", (int)user->pw_gid);
-            status = FC_ECONFIG;
-            goto done;
-        }
-
-        if (setuid(user->pw_uid)) {
-            logg("^setuid(%d) failed.\n", (int)user->pw_uid);
-            status = FC_ECONFIG;
-            goto done;
-        }
-    }
-#endif /* HAVE_PWD_H */
-
-    status = FC_SUCCESS;
-
-done:
-
-    return status;
-}
-
 /**
  * @brief Get a list of strings for a given repeatable opt argument.
  *
@@ -749,9 +708,11 @@ static fc_error_t initialize(struct optstruct *opts)
     fc_error_t status = FC_EARG;
     cl_error_t cl_init_retcode;
     fc_config fcConfig;
-    char *tempDirectory = NULL;
+    char *tempDirectory                = NULL;
     const struct optstruct *logFileOpt = NULL;
 
+    STATBUF statbuf;
+
     memset(&fcConfig, 0, sizeof(fc_config));
 
     if (NULL == opts) {
@@ -769,16 +730,73 @@ static fc_error_t initialize(struct optstruct *opts)
         }
     }
 
-#ifdef HAVE_PWD_H
     /*
-     * freshclam shouldn't work with root privileges.
-     * Drop privileges to the DatabaseOwner user, if specified.
+     * Verify that the database directory exists.
+     * Create database directory if missing.
      */
-    ret = switch_user(optget(opts, "DatabaseOwner")->strarg);
-    if (FC_SUCCESS != ret) {
-        logg("!Failed to switch to %s user.\n", optget(opts, "DatabaseOwner")->strarg);
-        status = ret;
-        goto done;
+    fcConfig.databaseDirectory = optget(opts, "DatabaseDirectory")->strarg;
+
+    if (LSTAT(fcConfig.databaseDirectory, &statbuf) == -1) {
+#ifdef HAVE_PWD_H
+        struct passwd *user;
+#endif
+
+        logg("Creating missing database directory: %s\n", fcConfig.databaseDirectory);
+
+        if (0 != mkdir(fcConfig.databaseDirectory, 0755)) {
+            logg("!Failed to create database directory: %s\n", fcConfig.databaseDirectory);
+            logg("Manually prepare the database directory, or re-run freshclam with higher privileges.\n");
+            status = FC_EDBDIRACCESS;
+            goto done;
+        }
+
+#ifdef HAVE_PWD_H
+        if (!geteuid()) {
+            /* Running as root user, will assign ownership of database directory to DatabaseOwner */
+            errno = 0;
+            if ((user = getpwnam(optget(opts, "DatabaseOwner")->strarg)) == NULL) {
+                logg("ERROR: Failed to get information about user \"%s\".\n",
+                     optget(opts, "DatabaseOwner")->strarg);
+                if (errno == 0) {
+                    logg("Create the \"%s\" user account for freshclam to use, or set the DatabaseOwner config option in freshclam.conf to a different user.\n",
+                         optget(opts, "DatabaseOwner")->strarg);
+                    logg("For more information, see https://www.clamav.net/documents/installing-clamav-on-unix-linux-macos-from-source\n");
+                } else {
+                    logg("An unexpected error occurred when attempting to query the \"%s\" user account.\n",
+                         optget(opts, "DatabaseOwner")->strarg);
+                }
+                status = FC_EDBDIRACCESS;
+                goto done;
+            }
+
+            if (chown(fcConfig.databaseDirectory, user->pw_uid, user->pw_gid)) {
+                logg("!Failed to change database directory ownership to user %s. Error: %s\n", optget(opts, "DatabaseOwner")->strarg, strerror(errno));
+                status = FC_EDBDIRACCESS;
+                goto done;
+            }
+
+            logg("Assigned ownership of database directory to user \"%s\".\n", optget(opts, "DatabaseOwner")->strarg);
+        }
+#endif
+    }
+
+#ifdef HAVE_PWD_H
+    /* Drop database privileges here if we are not planning on daemonizing.  If
+     * we are, we should wait until after we craete the PidFile to drop
+     * privileges.  That way, it is owned by root (or whoever started freshclam),
+     * and no one can change it.  */
+    if (!optget(opts, "daemon")->enabled) {
+        /*
+         * freshclam shouldn't work with root privileges.
+         * Drop privileges to the DatabaseOwner user, if specified.
+         * Pass NULL for the log file name, because it hasn't been created yet.
+         */
+        ret = drop_privileges(optget(opts, "DatabaseOwner")->strarg, NULL);
+        if (ret) {
+            logg("!Failed to switch to %s user.\n", optget(opts, "DatabaseOwner")->strarg);
+            status = FC_ECONFIG;
+            goto done;
+        }
     }
 #endif /* HAVE_PWD_H */
 
@@ -856,10 +874,8 @@ static fc_error_t initialize(struct optstruct *opts)
     if ((optget(opts, "LocalIPAddress"))->enabled)
         fcConfig.localIP = (optget(opts, "LocalIPAddress"))->strarg;
 
-    fcConfig.databaseDirectory = optget(opts, "DatabaseDirectory")->strarg;
-
     /* Select a path for the temp directory:  databaseDirectory/tmp */
-    tempDirectory = cli_gentemp_with_prefix(fcConfig.databaseDirectory, "tmp");
+    tempDirectory          = cli_gentemp_with_prefix(fcConfig.databaseDirectory, "tmp");
     fcConfig.tempDirectory = tempDirectory;
 
     /* Store the path of the temp directory so we can delete it later. */
@@ -1313,7 +1329,7 @@ static fc_error_t executeIfNewVersion(
             }
             version++;
         }
-        char *modifiedCommand = (char *)malloc(strlen(command) + strlen(version) + 10);
+        modifiedCommand = (char *)malloc(strlen(command) + strlen(version) + 10);
         if (NULL == modifiedCommand) {
             logg("!executeIfNewVersion: Can't allocate memory for modifiedCommand\n");
             status = FC_EMEM;
@@ -1447,7 +1463,7 @@ fc_error_t perform_database_update(
             (void *)fc_context,
             &nUpdated);
         if (FC_SUCCESS != ret) {
-            logg("!Database update process failed: %s (%d)\n", fc_strerror(ret), ret);
+            logg("!Database update process failed: %s\n", fc_strerror(ret));
             status = ret;
             goto done;
         }
@@ -1464,7 +1480,7 @@ fc_error_t perform_database_update(
             (void *)fc_context,
             &nUpdated);
         if (FC_SUCCESS != ret) {
-            logg("!Database update process failed: %s (%d)\n", fc_strerror(ret), ret);
+            logg("!Database update process failed: %s\n", fc_strerror(ret));
             status = ret;
             goto done;
         }
@@ -1474,20 +1490,11 @@ fc_error_t perform_database_update(
     }
 
     if (0 < nTotalUpdated) {
-#ifdef BUILD_CLAMD
-        if (NULL != notifyClamd)
+        if (NULL != notifyClamd) {
             notify(notifyClamd);
-#endif
-
-        if (NULL != onUpdateExecute) {
-            execute("OnUpdateExecute", onUpdateExecute, bDaemonized);
         }
     }
 
-    if ((NULL != newVersion) && (NULL != onOutdatedExecute)) {
-        executeIfNewVersion(onOutdatedExecute, newVersion, bDaemonized);
-    }
-
     status = FC_SUCCESS;
 
 done:
@@ -1498,6 +1505,20 @@ done:
             cli_rmdirs(g_freshclamTempDirectory);
         }
     }
+
+    if (FC_SUCCESS == status) {
+        /* Run Execute commands after we clean up the temp directory,
+         * in case they want us to EXIT */
+        if (0 < nTotalUpdated) {
+            if (NULL != onUpdateExecute) {
+                execute("OnUpdateExecute", onUpdateExecute, bDaemonized);
+            }
+        }
+        if ((NULL != newVersion) && (NULL != onOutdatedExecute)) {
+            executeIfNewVersion(onOutdatedExecute, newVersion, bDaemonized);
+        }
+    }
+
     if (NULL != dnsUpdateInfo) {
         free(dnsUpdateInfo);
     }
@@ -1530,6 +1551,11 @@ int main(int argc, char **argv)
 
     int bPrune = 1;
 
+#ifdef HAVE_PWD_H
+    const struct optstruct *logFileOpt = NULL;
+    const char *logFileName            = NULL;
+#endif /* HAVE_PWD_H */
+
     fc_ctx fc_context = {0};
 
 #ifndef _WIN32
@@ -1537,6 +1563,7 @@ int main(int argc, char **argv)
     struct sigaction oldact;
 #endif
     int i;
+    pid_t parentPid = getpid();
 
     if (check_flevel())
         exit(FC_EINIT);
@@ -1674,16 +1701,6 @@ int main(int argc, char **argv)
             status = ret;
             goto done;
         }
-        if (optget(opts, "SafeBrowsing")->enabled) {
-            if (FC_SUCCESS != (ret = string_list_add("safebrowsing", &optInList, &nOptIns))) {
-                free_string_list(optInList, nOptIns);
-                optInList = NULL;
-
-                mprintf("!Failed to add safebrowsing to list of opt-in databases.\n");
-                status = ret;
-                goto done;
-            }
-        }
 
         /*
          * Collect list of database opt-outs.
@@ -1698,8 +1715,10 @@ int main(int argc, char **argv)
         }
         if (!optget(opts, "Bytecode")->enabled) {
             if (FC_SUCCESS != (ret = string_list_add("bytecode", &optOutList, &nOptOuts))) {
-                free_string_list(optOutList, nOptOuts);
+                free_string_list(optInList, nOptIns);
                 optInList = NULL;
+                free_string_list(optOutList, nOptOuts);
+                optOutList = NULL;
 
                 mprintf("!Failed to add bytecode to list of opt-out databases.\n");
                 status = ret;
@@ -1836,7 +1855,7 @@ int main(int argc, char **argv)
 #ifndef _WIN32
         /* fork into background */
         if (g_foreground == 0) {
-            if (daemonize() == -1) {
+            if (-1 == daemonize_parent_wait(NULL, NULL)) {
                 logg("!daemonize() failed\n");
                 status = FC_EFAILEDUPDATE;
                 goto done;
@@ -1848,9 +1867,41 @@ int main(int argc, char **argv)
         /* Write PID of daemon process to pidfile. */
         if ((opt = optget(opts, "PidFile"))->enabled) {
             g_pidfile = opt->strarg;
-            writepid(g_pidfile);
+            if (writepid(g_pidfile)) {
+                status = FC_EINIT;
+                goto done;
+            }
         }
 
+#ifndef _WIN32
+        /* Signal the parent process that we have successfully
+         * written the PidFile.  If it does not get this signal, it
+         * will wait for our exit status (and we don't exit in daemon mode).
+         */
+        if (parentPid != getpid()) { //we have been daemonized
+            daemonize_signal_parent(parentPid);
+        }
+#endif
+
+#ifdef HAVE_PWD_H
+        /*  Get the log file name to pass it into drop_privileges.  */
+        logFileOpt = optget(opts, "UpdateLogFile");
+        if (logFileOpt->enabled) {
+            logFileName = logFileOpt->strarg;
+        }
+
+        /*
+         * freshclam shouldn't work with root privileges.
+         * Drop privileges to the DatabaseOwner user, if specified.
+         */
+        ret = drop_privileges(optget(opts, "DatabaseOwner")->strarg, logFileName);
+        if (0 != ret) {
+            logg("!Failed to switch to %s user.\n", optget(opts, "DatabaseOwner")->strarg);
+            status = FC_ECONFIG;
+            goto done;
+        }
+#endif /* HAVE_PWD_H */
+
         g_active_children = 0;
 
         logg("#freshclam daemon %s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ")\n", get_version());
@@ -1881,7 +1932,7 @@ int main(int argc, char **argv)
             alarm(0);
 #endif
 
-            if (ret > 1) {
+            if (ret > FC_UPTODATE) {
                 if ((opt = optget(opts, "OnErrorExecute"))->enabled)
                     arg = opt->strarg;
 
@@ -1889,6 +1940,14 @@ int main(int argc, char **argv)
                     execute("OnErrorExecute", arg, optget(opts, "daemon")->enabled);
 
                 arg = NULL;
+
+                if (FC_EFORBIDDEN == ret) {
+                    /* We're being actively blocked, which is a fatal error. Exit. */
+                    logg("^FreshClam was forbidden from downloading a database.\n");
+                    logg("^This is fatal. Retrying later won't help. Exiting now.\n");
+                    status = ret;
+                    goto done;
+                }
             }
 
             logg("#--------------------------------------\n");
@@ -1951,6 +2010,9 @@ done:
     if (NULL != urlDatabaseList) {
         free_string_list(urlDatabaseList, nUrlDatabases);
     }
+    if (NULL != serverList) {
+        free_string_list(serverList, nServers);
+    }
     if (NULL != opts) {
         optfree(opts);
     }
