@@ -1,7 +1,7 @@
 /*
  *  Load, verify and execute ClamAV bytecode.
  *
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include "fmap.h"
 #include "bytecode_detect.h"
+#include "platform.h"
 
 struct cli_dbio;
 struct cli_bc_ctx;
@@ -105,7 +106,7 @@ void cli_bytecode_context_destroy(struct cli_bc_ctx *ctx);
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern int have_clamjit;
+extern LIBCLAMAV_EXPORT int have_clamjit;
 #ifdef __cplusplus
 }
 #endif
