@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -66,6 +66,10 @@ typedef enum cli_file {
     CL_TYPE_SIS,
     CL_TYPE_SCRENC,
     CL_TYPE_GRAPHICS,
+    CL_TYPE_GIF,
+    CL_TYPE_PNG,
+    CL_TYPE_JPEG,
+    CL_TYPE_TIFF,
     CL_TYPE_RIFF,
     CL_TYPE_BINHEX,
     CL_TYPE_TNEF,
@@ -142,8 +146,9 @@ struct cli_matched_type {
 cli_file_t cli_ftcode(const char *name);
 const char *cli_ftname(cli_file_t code);
 void cli_ftfree(const struct cl_engine *engine);
-cli_file_t cli_filetype(const unsigned char *buf, size_t buflen, const struct cl_engine *engine);
-cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine, cli_file_t basetype);
+cli_file_t cli_compare_ftm_file(const unsigned char *buf, size_t buflen, const struct cl_engine *engine);
+cli_file_t cli_compare_ftm_partition(const unsigned char *buf, size_t buflen, const struct cl_engine *engine);
+cli_file_t cli_determine_fmap_type(fmap_t *map, const struct cl_engine *engine, cli_file_t basetype);
 int cli_addtypesigs(struct cl_engine *engine);
 
 #endif
