@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -48,123 +48,6 @@
 #include "others.h"
 #include "str.h"
 
-static inline unsigned int fmap_align_items(unsigned int sz, unsigned int al);
-static inline unsigned int fmap_align_to(unsigned int sz, unsigned int al);
-static inline unsigned int fmap_which_page(fmap_t *m, size_t at);
-
-#ifndef _WIN32
-/* pread proto here in order to avoid the use of XOPEN and BSD_SOURCE
-   which may in turn prevent some mmap constants to be defined */
-ssize_t pread(int fd, void *buf, size_t count, off_t offset);
-
-/* vvvvv POSIX STUFF BELOW vvvvv */
-static off_t pread_cb(void *handle, void *buf, size_t count, off_t offset)
-{
-    return pread((int)(ssize_t)handle, buf, count, offset);
-}
-
-fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty)
-{
-    STATBUF st;
-    fmap_t *m;
-
-    *empty = 0;
-    if (FSTAT(fd, &st)) {
-        cli_warnmsg("fmap: fstat failed\n");
-        return NULL;
-    }
-
-    if (!len) len = st.st_size - offset; /* bound checked later */
-    if (!len) {
-        cli_dbgmsg("fmap: attempted void mapping\n");
-        *empty = 1;
-        return NULL;
-    }
-    if (!CLI_ISCONTAINED(0, st.st_size, offset, len)) {
-        cli_warnmsg("fmap: attempted oof mapping\n");
-        return NULL;
-    }
-    m = cl_fmap_open_handle((void *)(ssize_t)fd, offset, len, pread_cb, 1);
-    if (!m)
-        return NULL;
-    m->mtime        = st.st_mtime;
-    m->handle_is_fd = 1;
-    return m;
-}
-#else
-/* vvvvv WIN32 STUFF BELOW vvvvv */
-static void unmap_win32(fmap_t *m)
-{ /* WIN32 */
-    UnmapViewOfFile(m->data);
-    CloseHandle(m->mh);
-    free((void *)m);
-}
-
-fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty)
-{ /* WIN32 */
-    unsigned int pages, mapsz, hdrsz;
-    int pgsz = cli_getpagesize();
-    STATBUF st;
-    fmap_t *m;
-    const void *data;
-    HANDLE fh;
-    HANDLE mh;
-
-    *empty = 0;
-    if (FSTAT(fd, &st)) {
-        cli_warnmsg("fmap: fstat failed\n");
-        return NULL;
-    }
-    if (offset < 0 || offset != fmap_align_to(offset, pgsz)) {
-        cli_warnmsg("fmap: attempted mapping with unaligned offset\n");
-        return NULL;
-    }
-    if (!len) len = st.st_size - offset; /* bound checked later */
-    if (!len) {
-        cli_dbgmsg("fmap: attempted void mapping\n");
-        *empty = 1;
-        return NULL;
-    }
-    if (!CLI_ISCONTAINED(0, st.st_size, offset, len)) {
-        cli_warnmsg("fmap: attempted oof mapping\n");
-        return NULL;
-    }
-
-    pages = fmap_align_items(len, pgsz);
-    hdrsz = fmap_align_to(sizeof(fmap_t), pgsz);
-
-    if ((fh = (HANDLE)_get_osfhandle(fd)) == INVALID_HANDLE_VALUE) {
-        cli_errmsg("fmap: cannot get a valid handle for descriptor %d\n", fd);
-        return NULL;
-    }
-    if (!(mh = CreateFileMapping(fh, NULL, PAGE_READONLY, (DWORD)((len >> 31) >> 1), (DWORD)len, NULL))) {
-        cli_errmsg("fmap: cannot create a map of descriptor %d\n", fd);
-        CloseHandle(fh);
-        return NULL;
-    }
-    if (!(data = MapViewOfFile(mh, FILE_MAP_READ, (DWORD)((offset >> 31) >> 1), (DWORD)(offset), len))) {
-        cli_errmsg("fmap: cannot map file descriptor %d\n", fd);
-        CloseHandle(mh);
-        CloseHandle(fh);
-        return NULL;
-    }
-    if (!(m = cl_fmap_open_memory(data, len))) {
-        cli_errmsg("fmap: cannot allocate fmap_t\n", fd);
-        CloseHandle(mh);
-        CloseHandle(fh);
-        return NULL;
-    }
-    m->handle       = (void *)(size_t)fd;
-    m->handle_is_fd = 1;
-    m->fh           = fh;
-    m->mh           = mh;
-    m->unmap        = unmap_win32;
-    return m;
-}
-#endif /* _WIN32 */
-
-/* vvvvv SHARED STUFF BELOW vvvvv */
-
 #define FM_MASK_COUNT 0x3fffffff
 #define FM_MASK_PAGED 0x40000000
 #define FM_MASK_SEEN 0x80000000
@@ -200,38 +83,305 @@ pthread_mutex_t fmap_mutex = PTHREAD_MUTEX_INITIALIZER;
 #define MADV_DONTFORK 0
 #endif
 
-#define fmap_bitmap (&m->placeholder_for_bitmap)
+#define fmap_bitmap (m->bitmap)
+
+static inline uint64_t fmap_align_items(uint64_t sz, uint64_t al);
+static inline uint64_t fmap_align_to(uint64_t sz, uint64_t al);
+static inline uint64_t fmap_which_page(fmap_t *m, size_t at);
 
 static const void *handle_need(fmap_t *m, size_t at, size_t len, int lock);
 static void handle_unneed_off(fmap_t *m, size_t at, size_t len);
 static const void *handle_need_offstr(fmap_t *m, size_t at, size_t len_hint);
 static const void *handle_gets(fmap_t *m, char *dst, size_t *at, size_t max_len);
+
 static void unmap_mmap(fmap_t *m);
 static void unmap_malloc(fmap_t *m);
 
-extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
-                                      clcb_pread pread_cb, int use_aging)
-{
-    unsigned int pages, mapsz, hdrsz;
-    cl_fmap_t *m;
-    int pgsz = cli_getpagesize();
+#ifndef _WIN32
+/* pread proto here in order to avoid the use of XOPEN and BSD_SOURCE
+   which may in turn prevent some mmap constants to be defined */
+ssize_t pread(int fd, void *buf, size_t count, off_t offset);
 
-    if ((off_t)offset < 0 || offset != fmap_align_to(offset, pgsz)) {
+/* vvvvv POSIX STUFF BELOW vvvvv */
+static off_t pread_cb(void *handle, void *buf, size_t count, off_t offset)
+{
+    return pread((int)(ssize_t)handle, buf, count, offset);
+}
+
+fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name)
+{
+    STATBUF st;
+    fmap_t *m = NULL;
+
+    *empty = 0;
+    if (FSTAT(fd, &st)) {
+        cli_warnmsg("fmap: fstat failed\n");
+        return NULL;
+    }
+
+    if (!len) len = st.st_size - offset; /* bound checked later */
+    if (!len) {
+        cli_dbgmsg("fmap: attempted void mapping\n");
+        *empty = 1;
+        return NULL;
+    }
+    if (!CLI_ISCONTAINED(0, st.st_size, offset, len)) {
+        cli_warnmsg("fmap: attempted oof mapping\n");
+        return NULL;
+    }
+    m = cl_fmap_open_handle((void *)(ssize_t)fd, offset, len, pread_cb, 1);
+    if (!m)
+        return NULL;
+    m->mtime = st.st_mtime;
+
+    if (NULL != name) {
+        m->name = cli_strdup(name);
+        if (NULL == m->name) {
+            funmap(m);
+            return NULL;
+        }
+    }
+
+    return m;
+}
+#else
+/* vvvvv WIN32 STUFF BELOW vvvvv */
+static void unmap_win32(fmap_t *m)
+{
+    if (NULL != m) {
+        if (NULL != m->data) {
+            UnmapViewOfFile(m->data);
+        }
+        if (NULL != m->mh) {
+            CloseHandle(m->mh);
+        }
+        if (NULL != m->name) {
+            free(m->name);
+        }
+        free((void *)m);
+    }
+}
+
+fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name)
+{ /* WIN32 */
+    uint64_t pages, mapsz;
+    int pgsz = cli_getpagesize();
+    STATBUF st;
+    fmap_t *m = NULL;
+    const void *data;
+    HANDLE fh;
+    HANDLE mh;
+
+    *empty = 0;
+    if (FSTAT(fd, &st)) {
+        cli_warnmsg("fmap: fstat failed\n");
+        return NULL;
+    }
+    if (offset < 0 || offset != fmap_align_to(offset, pgsz)) {
         cli_warnmsg("fmap: attempted mapping with unaligned offset\n");
         return NULL;
     }
+    if (!len) len = st.st_size - offset; /* bound checked later */
     if (!len) {
         cli_dbgmsg("fmap: attempted void mapping\n");
+        *empty = 1;
         return NULL;
     }
-    if (offset >= len) {
+    if (!CLI_ISCONTAINED(0, st.st_size, offset, len)) {
         cli_warnmsg("fmap: attempted oof mapping\n");
         return NULL;
     }
 
     pages = fmap_align_items(len, pgsz);
-    hdrsz = fmap_align_to(sizeof(fmap_t) + (pages - 1) * sizeof(uint64_t), pgsz); /* fmap_t includes 1 bitmap slot, hence (pages-1) */
-    mapsz = pages * pgsz + hdrsz;
+
+    if ((fh = (HANDLE)_get_osfhandle(fd)) == INVALID_HANDLE_VALUE) {
+        cli_errmsg("fmap: cannot get a valid handle for descriptor %d\n", fd);
+        return NULL;
+    }
+    if (!(mh = CreateFileMapping(fh, NULL, PAGE_READONLY, (DWORD)((len >> 31) >> 1), (DWORD)len, NULL))) {
+        cli_errmsg("fmap: cannot create a map of descriptor %d\n", fd);
+        return NULL;
+    }
+    if (!(data = MapViewOfFile(mh, FILE_MAP_READ, (DWORD)((offset >> 31) >> 1), (DWORD)(offset), len))) {
+        cli_errmsg("fmap: cannot map file descriptor %d\n", fd);
+        CloseHandle(mh);
+        return NULL;
+    }
+    if (!(m = cl_fmap_open_memory(data, len))) {
+        cli_errmsg("fmap: cannot allocate fmap_t\n", fd);
+        UnmapViewOfFile(data);
+        CloseHandle(mh);
+        return NULL;
+    }
+    m->handle       = (void *)(size_t)fd;
+    m->handle_is_fd = 1; /* This is probably(?) needed so `fmap_fd()` can return the file descriptor. */
+    m->fh           = fh;
+    m->mh           = mh;
+    m->unmap        = unmap_win32;
+
+    if (NULL != name) {
+        m->name = cli_strdup(name);
+        if (NULL == m->name) {
+            funmap(m);
+            return NULL;
+        }
+    }
+
+    return m;
+}
+#endif /* _WIN32 */
+
+/* vvvvv SHARED STUFF BELOW vvvvv */
+
+fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char *name)
+{
+    cl_error_t status        = CL_ERROR;
+    cl_fmap_t *duplicate_map = NULL;
+
+    if (NULL == map) {
+        cli_warnmsg("fmap_duplicate: map is NULL!\n");
+        goto done;
+    }
+
+    duplicate_map = cli_malloc(sizeof(cl_fmap_t));
+    if (!duplicate_map) {
+        cli_warnmsg("fmap_duplicate: map allocation failed\n");
+        goto done;
+    }
+
+    /* Duplicate the state of the original map */
+    memcpy(duplicate_map, map, sizeof(cl_fmap_t));
+
+    if (offset > map->len) {
+        /* invalid offset, exceeds length of map */
+        cli_warnmsg("fmap_duplicate: requested offset exceeds end of map\n");
+        goto done;
+    }
+
+    if (offset > 0 || length < map->len) {
+        /* Caller requested a window into the current map, not the whole map */
+        unsigned char hash[16] = {'\0'};
+
+        /* Set the new nested offset and (nested) length for the new map */
+        /* can't change offset because then we'd have to discard/move cached
+         * data, instead use nested_offset to reuse the already cached data */
+        duplicate_map->nested_offset += offset;
+        duplicate_map->len = MIN(length, map->len - offset);
+
+        if (!CLI_ISCONTAINED2(map->nested_offset, map->len,
+                              duplicate_map->nested_offset, duplicate_map->len)) {
+            size_t len1, len2;
+            len1 = map->nested_offset + map->len;
+            len2 = duplicate_map->nested_offset + duplicate_map->len;
+            cli_warnmsg("fmap_duplicate: internal map error: %zu, %zu; %zu, %zu\n",
+                        map->nested_offset, len1,
+                        duplicate_map->offset, len2);
+        }
+
+        /* Calculate the fmap hash to be used by the FP check later */
+        if (CL_SUCCESS != fmap_get_MD5(hash, duplicate_map)) {
+            cli_warnmsg("fmap_duplicate: failed to get fmap MD5\n");
+            goto done;
+        }
+        memcpy(duplicate_map->maphash, hash, 16);
+    }
+
+    if (NULL != name) {
+        duplicate_map->name = cli_strdup(name);
+        if (NULL == duplicate_map->name) {
+            free(duplicate_map);
+            return NULL;
+        }
+    } else {
+        duplicate_map->name = NULL;
+    }
+
+    status = CL_SUCCESS;
+
+done:
+    if (CL_SUCCESS != status) {
+        if (NULL != duplicate_map) {
+            free(duplicate_map);
+            duplicate_map = NULL;
+        }
+    }
+
+    return duplicate_map;
+}
+
+void free_duplicate_fmap(cl_fmap_t *map)
+{
+    if (NULL != map) {
+        if (NULL != map->name) {
+            free(map->name);
+            map->name = NULL;
+        }
+        free(map);
+    }
+}
+
+static void unmap_handle(fmap_t *m)
+{
+    if (NULL != m) {
+        if (NULL != m->data) {
+            if (m->aging) {
+                unmap_mmap(m);
+            } else {
+                free((void *)m->data);
+            }
+            m->data = NULL;
+        }
+        if (NULL != m->bitmap) {
+            free(m->bitmap);
+            m->bitmap = NULL;
+        }
+        if (NULL != m->name) {
+            free(m->name);
+        }
+        free((void *)m);
+    }
+}
+
+extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
+                                      clcb_pread pread_cb, int use_aging)
+{
+    cl_error_t status = CL_EMEM;
+    uint64_t pages;
+    size_t mapsz, bitmap_size;
+    cl_fmap_t *m = NULL;
+    int pgsz     = cli_getpagesize();
+
+    unsigned char hash[16] = {'\0'};
+
+    if ((off_t)offset < 0 || offset != fmap_align_to(offset, pgsz)) {
+        cli_warnmsg("fmap: attempted mapping with unaligned offset\n");
+        goto done;
+    }
+    if (!len) {
+        cli_dbgmsg("fmap: attempted void mapping\n");
+        goto done;
+    }
+    if (offset >= len) {
+        cli_warnmsg("fmap: attempted oof mapping\n");
+        goto done;
+    }
+
+    pages = fmap_align_items(len, pgsz);
+
+    bitmap_size = pages * sizeof(uint64_t);
+    mapsz       = pages * pgsz;
+
+    m = cli_calloc(1, sizeof(fmap_t));
+    if (!m) {
+        cli_warnmsg("fmap: map header allocation failed\n");
+        goto done;
+    }
+
+    m->bitmap = cli_calloc(1, bitmap_size);
+    if (!m->bitmap) {
+        cli_warnmsg("fmap: map header allocation failed\n");
+        goto done;
+    }
 
 #ifndef ANONYMOUS_MAP
     use_aging = 0;
@@ -239,29 +389,26 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
 #ifdef ANONYMOUS_MAP
     if (use_aging) {
         fmap_lock;
-        if ((m = (fmap_t *)mmap(NULL, mapsz, PROT_READ | PROT_WRITE, MAP_PRIVATE | /*FIXME: MAP_POPULATE is ~8% faster but more memory intensive */ ANONYMOUS_MAP, -1, 0)) == MAP_FAILED) {
-            m = NULL;
+        if ((m->data = (fmap_t *)mmap(NULL,
+                                      mapsz,
+                                      PROT_READ | PROT_WRITE, MAP_PRIVATE | /* FIXME: MAP_POPULATE is ~8% faster but more memory intensive */ ANONYMOUS_MAP,
+                                      -1,
+                                      0)) == MAP_FAILED) {
+            m->data = NULL;
         } else {
 #if HAVE_MADVISE
-            madvise((void *)m, mapsz, MADV_RANDOM | MADV_DONTFORK);
+            madvise((void *)m->data, mapsz, MADV_RANDOM | MADV_DONTFORK);
 #endif /* madvise */
-            /* fault the header while we still have the lock - we DO context switch here a lot here :@ */
-            memset(fmap_bitmap, 0, sizeof(uint32_t) * pages);
         }
         fmap_unlock;
     }
 #endif /* ANONYMOUS_MAP */
     if (!use_aging) {
-        m = (fmap_t *)cli_malloc(mapsz);
-        if (!(m)) {
-            cli_warnmsg("fmap: map allocation failed\n");
-            return NULL;
-        }
-        memset(m, 0, hdrsz);
+        m->data = (fmap_t *)cli_malloc(mapsz);
     }
-    if (!m) {
+    if (!m->data) {
         cli_warnmsg("fmap: map allocation failed\n");
-        return NULL;
+        goto done;
     }
     m->handle          = handle;
     m->pread_cb        = pread_cb;
@@ -271,15 +418,30 @@ extern cl_fmap_t *cl_fmap_open_handle(void *handle, size_t offset, size_t len,
     m->len             = len; /* m->nested_offset + m->len = m->real_len */
     m->real_len        = len;
     m->pages           = pages;
-    m->hdrsz           = hdrsz;
     m->pgsz            = pgsz;
     m->paged           = 0;
     m->dont_cache_flag = 0;
-    m->unmap           = use_aging ? unmap_mmap : unmap_malloc;
+    m->unmap           = unmap_handle;
     m->need            = handle_need;
     m->need_offstr     = handle_need_offstr;
     m->gets            = handle_gets;
     m->unneed_off      = handle_unneed_off;
+    m->handle_is_fd    = 1;
+
+    /* Calculate the fmap hash to be used by the FP check later */
+    if (CL_SUCCESS != fmap_get_MD5(hash, m)) {
+        cli_warnmsg("fmap: failed to get MD5\n");
+        goto done;
+    }
+    memcpy(m->maphash, hash, 16);
+
+    status = CL_SUCCESS;
+
+done:
+    if (CL_SUCCESS != status) {
+        unmap_handle(m);
+        m = NULL;
+    }
     return m;
 }
 
@@ -288,10 +450,10 @@ static void fmap_aging(fmap_t *m)
 #ifdef ANONYMOUS_MAP
     if (!m->aging) return;
     if (m->paged * m->pgsz > UNPAGE_THRSHLD_HI) { /* we alloc'd too much */
-        unsigned int i, avail = 0, freeme[2048], maxavail = MIN(sizeof(freeme) / sizeof(*freeme), m->paged - UNPAGE_THRSHLD_LO / m->pgsz) - 1;
+        uint64_t i, avail = 0, freeme[2048], maxavail = MIN(sizeof(freeme) / sizeof(*freeme), m->paged - UNPAGE_THRSHLD_LO / m->pgsz) - 1;
 
         for (i = 0; i < m->pages; i++) {
-            uint32_t s = fmap_bitmap[i];
+            uint64_t s = fmap_bitmap[i];
             if ((s & (FM_MASK_PAGED | FM_MASK_LOCKED)) == FM_MASK_PAGED) {
                 /* page is paged and not locked: dec age */
                 if (s & FM_MASK_COUNT) fmap_bitmap[i]--;
@@ -302,7 +464,7 @@ static void fmap_aging(fmap_t *m)
                     avail++;
                 } else {
                     /* Insert sort onto a stack'd array - same performance as quickselect */
-                    unsigned int insert_to = MIN(maxavail, avail) - 1, age = fmap_bitmap[i] & FM_MASK_COUNT;
+                    uint64_t insert_to = MIN(maxavail, avail) - 1, age = fmap_bitmap[i] & FM_MASK_COUNT;
                     if (avail <= maxavail || (fmap_bitmap[freeme[maxavail]] & FM_MASK_COUNT) > age) {
                         while ((fmap_bitmap[freeme[insert_to]] & FM_MASK_COUNT) > age) {
                             freeme[insert_to + 1] = freeme[insert_to];
@@ -318,7 +480,7 @@ static void fmap_aging(fmap_t *m)
             char *lastpage  = NULL;
             char *firstpage = NULL;
             for (i = 0; i < avail; i++) {
-                char *pptr = (char *)m + freeme[i] * m->pgsz + m->hdrsz;
+                char *pptr = (char *)m->data + freeme[i] * m->pgsz;
                 /* we mark the page as seen */
                 fmap_bitmap[freeme[i]] = FM_MASK_SEEN;
                 /* and we mmap the page over so the kernel knows there's nothing good in there */
@@ -351,15 +513,15 @@ static void fmap_aging(fmap_t *m)
 #endif
 }
 
-static int fmap_readpage(fmap_t *m, uint64_t first_page, uint32_t count, uint32_t lock_count)
+static int fmap_readpage(fmap_t *m, uint64_t first_page, uint64_t count, uint64_t lock_count)
 {
     size_t readsz = 0, eintr_off;
     char *pptr    = NULL, errtxt[256];
-    uint32_t sbitmap;
+    uint64_t sbitmap;
     uint64_t i, page = first_page, force_read = 0;
 
-    if ((size_t)(m->real_len) > (size_t)(UINT_MAX)) {
-        cli_dbgmsg("fmap_readage: size of file exceeds total prefaultible page size (unpacked file is too large)\n");
+    if ((uint64_t)(m->real_len) > (uint64_t)(m->pages * m->pgsz)) {
+        cli_dbgmsg("fmap_readpage: size of file exceeds total prefaultible page size (unpacked file is too large)\n");
         return 1;
     }
 
@@ -368,7 +530,7 @@ static int fmap_readpage(fmap_t *m, uint64_t first_page, uint32_t count, uint32_
         /* Not worth checking if the page is already paged, just ping each */
         /* Also not worth reusing the loop below */
         volatile char faultme;
-        faultme = ((char *)m)[(first_page + i) * m->pgsz + m->hdrsz];
+        faultme = ((char *)m->data)[(first_page + i) * m->pgsz];
     }
     fmap_unlock;
     for (i = 0; i <= count; i++, page++) {
@@ -411,7 +573,7 @@ static int fmap_readpage(fmap_t *m, uint64_t first_page, uint32_t count, uint32_
         if (force_read) {
             /* we have some pending reads to perform */
             if (m->handle_is_fd) {
-                unsigned int j;
+                uint64_t j;
                 int _fd = (int)(ptrdiff_t)m->handle;
                 for (j = first_page; j < page; j++) {
                     if (fmap_bitmap[j] & FM_MASK_SEEN) {
@@ -451,7 +613,7 @@ static int fmap_readpage(fmap_t *m, uint64_t first_page, uint32_t count, uint32_
                     cli_strerror(errno, errtxt, sizeof(errtxt));
                     cli_errmsg("fmap_readpage: pread error: %s\n", errtxt);
                 } else {
-                    cli_warnmsg("fmap_readpage: pread fail: asked for %lu bytes @ offset %lu, got %lu\n", (long unsigned int)readsz, (long unsigned int)target_offset, (long unsigned int)got);
+                    cli_warnmsg("fmap_readpage: pread fail: asked for %zu bytes @ offset %zu, got %zd\n", readsz, (size_t)target_offset, got);
                 }
                 return 1;
             }
@@ -465,7 +627,7 @@ static int fmap_readpage(fmap_t *m, uint64_t first_page, uint32_t count, uint32_
         /* page is not already paged */
         if (!pptr) {
             /* set a new start for pending reads if we don't have one */
-            pptr       = (char *)m + page * m->pgsz + m->hdrsz;
+            pptr       = (char *)m->data + page * m->pgsz;
             first_page = page;
         }
         if ((page == m->pages - 1) && (m->real_len % m->pgsz))
@@ -506,14 +668,13 @@ static const void *handle_need(fmap_t *m, size_t at, size_t len, int lock)
     if (fmap_readpage(m, first_page, last_page - first_page + 1, lock_count))
         return NULL;
 
-    ret = (char *)m;
-    ret += at + m->hdrsz;
+    ret = (char *)m->data + at;
     return (void *)ret;
 }
 
-static void fmap_unneed_page(fmap_t *m, unsigned int page)
+static void fmap_unneed_page(fmap_t *m, uint64_t page)
 {
-    uint32_t s = fmap_bitmap[page];
+    uint64_t s = fmap_bitmap[page];
 
     if ((s & (FM_MASK_PAGED | FM_MASK_LOCKED)) == (FM_MASK_PAGED | FM_MASK_LOCKED)) {
         /* page is paged and locked: check lock count */
@@ -532,7 +693,7 @@ static void fmap_unneed_page(fmap_t *m, unsigned int page)
 
 static void handle_unneed_off(fmap_t *m, size_t at, size_t len)
 {
-    unsigned int i, first_page, last_page;
+    uint64_t i, first_page, last_page;
     if (!m->aging) return;
     if (!len) {
         cli_warnmsg("fmap_unneed: attempted void unneed\n");
@@ -556,23 +717,28 @@ static void handle_unneed_off(fmap_t *m, size_t at, size_t len)
 static void unmap_mmap(fmap_t *m)
 {
 #ifdef ANONYMOUS_MAP
-    size_t len = m->pages * m->pgsz + m->hdrsz;
+    size_t len = m->pages * m->pgsz;
     fmap_lock;
-    if (munmap((void *)m, len) == -1) /* munmap() failed */
-        cli_warnmsg("funmap: unable to unmap memory segment at address: %p with length: %zu\n", (void *)m, len);
+    if (munmap((void *)m->data, len) == -1) /* munmap() failed */
+        cli_warnmsg("funmap: unable to unmap memory segment at address: %p with length: %zu\n", (void *)m->data, len);
     fmap_unlock;
 #endif
 }
 
 static void unmap_malloc(fmap_t *m)
 {
-    free((void *)m);
+    if (NULL != m) {
+        if (NULL != m->name) {
+            free(m->name);
+        }
+        free((void *)m);
+    }
 }
 
 static const void *handle_need_offstr(fmap_t *m, size_t at, size_t len_hint)
 {
-    unsigned int i, first_page, last_page;
-    void *ptr = (void *)((char *)m + m->hdrsz + at);
+    uint64_t i, first_page, last_page;
+    void *ptr = (void *)((char *)m->data + at);
 
     if (!len_hint || len_hint > m->real_len - at)
         len_hint = m->real_len - at;
@@ -586,8 +752,8 @@ static const void *handle_need_offstr(fmap_t *m, size_t at, size_t len_hint)
     last_page  = fmap_which_page(m, at + len_hint - 1);
 
     for (i = first_page; i <= last_page; i++) {
-        char *thispage = (char *)m + m->hdrsz + i * m->pgsz;
-        unsigned int scanat, scansz;
+        char *thispage = (char *)m->data + i * m->pgsz;
+        uint64_t scanat, scansz;
 
         if (fmap_readpage(m, i, 1, 1)) {
             last_page = i - 1;
@@ -611,9 +777,11 @@ static const void *handle_need_offstr(fmap_t *m, size_t at, size_t len_hint)
 
 static const void *handle_gets(fmap_t *m, char *dst, size_t *at, size_t max_len)
 {
-    unsigned int i, first_page, last_page;
-    char *src = (void *)((char *)m + m->hdrsz + *at), *endptr = NULL;
-    size_t len = MIN(max_len - 1, m->real_len - *at), fullen = len;
+    uint64_t i, first_page, last_page;
+    char *src     = (void *)((char *)m->data + *at);
+    char *endptr  = NULL;
+    size_t len    = MIN(max_len - 1, m->real_len - *at);
+    size_t fullen = len;
 
     if (!len || !CLI_ISCONTAINED(0, m->real_len, *at, len))
         return NULL;
@@ -624,8 +792,8 @@ static const void *handle_gets(fmap_t *m, char *dst, size_t *at, size_t max_len)
     last_page  = fmap_which_page(m, *at + len - 1);
 
     for (i = first_page; i <= last_page; i++) {
-        char *thispage = (char *)m + m->hdrsz + i * m->pgsz;
-        unsigned int scanat, scansz;
+        char *thispage = (char *)m->data + i * m->pgsz;
+        uint64_t scanat, scansz;
 
         if (fmap_readpage(m, i, 1, 0))
             return NULL;
@@ -663,8 +831,10 @@ static void mem_unneed_off(fmap_t *m, size_t at, size_t len);
 static const void *mem_need_offstr(fmap_t *m, size_t at, size_t len_hint);
 static const void *mem_gets(fmap_t *m, char *dst, size_t *at, size_t max_len);
 
-extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len)
+fmap_t *fmap_open_memory(const void *start, size_t len, const char *name)
 {
+    unsigned char hash[16] = {'\0'};
+
     int pgsz     = cli_getpagesize();
     cl_fmap_t *m = cli_calloc(1, sizeof(*m));
     if (!m) {
@@ -681,9 +851,33 @@ extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len)
     m->need_offstr = mem_need_offstr;
     m->gets        = mem_gets;
     m->unneed_off  = mem_unneed_off;
+
+    if (NULL != name) {
+        m->name = cli_strdup(name);
+        if (NULL == m->name) {
+            free(m);
+            return NULL;
+        }
+    }
+
+    /* Calculate the fmap hash to be used by the FP check later */
+    if (CL_SUCCESS != fmap_get_MD5(hash, m)) {
+        if (NULL != m->name) {
+            free(m->name);
+        }
+        free(m);
+        return NULL;
+    }
+    memcpy(m->maphash, hash, 16);
+
     return m;
 }
 
+extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len)
+{
+    return (cl_fmap_t *)fmap_open_memory(start, len, NULL);
+}
+
 static const void *mem_need(fmap_t *m, size_t at, size_t len, int lock)
 { /* WIN32 */
     UNUSEDPARAM(lock);
@@ -741,23 +935,23 @@ static const void *mem_gets(fmap_t *m, char *dst, size_t *at, size_t max_len)
     return dst;
 }
 
-fmap_t *fmap(int fd, off_t offset, size_t len)
+fmap_t *fmap(int fd, off_t offset, size_t len, const char *name)
 {
     int unused;
-    return fmap_check_empty(fd, offset, len, &unused);
+    return fmap_check_empty(fd, offset, len, &unused, name);
 }
 
-static inline unsigned int fmap_align_items(unsigned int sz, unsigned int al)
+static inline uint64_t fmap_align_items(uint64_t sz, uint64_t al)
 {
     return sz / al + (sz % al != 0);
 }
 
-static inline unsigned int fmap_align_to(unsigned int sz, unsigned int al)
+static inline uint64_t fmap_align_to(uint64_t sz, uint64_t al)
 {
     return al * fmap_align_items(sz, al);
 }
 
-static inline unsigned int fmap_which_page(fmap_t *m, size_t at)
+static inline uint64_t fmap_which_page(fmap_t *m, size_t at)
 {
     return at / m->pgsz;
 }
@@ -790,8 +984,8 @@ cl_error_t fmap_dump_to_file(fmap_t *map, const char *filepath, const char *tmpd
         } else if ((start_offset != 0) && (end_offset != map->real_len)) {
             /* If we're only dumping a portion of the file, inlcude the offsets in the prefix,...
 			 * e.g. tmp filename will become something like:  filebase.500-1200.<randhex> */
-            uint32_t prefix_len = strlen(filebase) + 1 + SIZE_T_CHARLEN + 1 + SIZE_T_CHARLEN + 1;
-            prefix              = malloc(prefix_len);
+            size_t prefix_len = strlen(filebase) + 1 + SIZE_T_CHARLEN + 1 + SIZE_T_CHARLEN + 1;
+            prefix            = malloc(prefix_len);
             if (NULL == prefix) {
                 cli_errmsg("fmap_dump_to_file: Failed to allocate memory for tempfile prefix.\n");
                 if (NULL != filebase)
@@ -860,8 +1054,13 @@ cl_error_t fmap_dump_to_file(fmap_t *map, const char *filepath, const char *tmpd
 int fmap_fd(fmap_t *m)
 {
     int fd;
-    if (!m->handle_is_fd)
+    if (NULL == m) {
+        cli_errmsg("fmap_fd: Attempted to get fd for NULL fmap\n");
         return -1;
+    }
+    if (!m->handle_is_fd) {
+        return -1;
+    }
     fd = (int)(ptrdiff_t)m->handle;
     lseek(fd, 0, SEEK_SET);
     return fd;
@@ -871,3 +1070,40 @@ extern void cl_fmap_close(cl_fmap_t *map)
 {
     funmap(map);
 }
+
+cl_error_t fmap_get_MD5(unsigned char *hash, fmap_t *map)
+{
+    size_t todo, at = 0;
+    void *hashctx;
+
+    todo = map->len;
+
+    hashctx = cl_hash_init("md5");
+    if (!(hashctx)) {
+        cli_errmsg("ctx_get_MD5: error initializing new md5 hash!\n");
+        return CL_ERROR;
+    }
+
+    while (todo) {
+        const void *buf;
+        size_t readme = todo < 1024 * 1024 * 10 ? todo : 1024 * 1024 * 10;
+
+        if (!(buf = fmap_need_off_once(map, at, readme))) {
+            cl_hash_destroy(hashctx);
+            return CL_EREAD;
+        }
+
+        todo -= readme;
+        at += readme;
+
+        if (cl_update_hash(hashctx, (void *)buf, readme)) {
+            cl_hash_destroy(hashctx);
+            cli_errmsg("ctx_get_MD5: error reading while generating hash!\n");
+            return CL_EREAD;
+        }
+    }
+
+    cl_finish_hash(hashctx, hash);
+
+    return CL_CLEAN;
+}
