@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -46,13 +46,12 @@ struct cl_fmap {
 
     /* internal */
     time_t mtime;
-    unsigned int pages;
-    uint64_t hdrsz;
+    uint64_t pages;
     uint64_t pgsz;
-    unsigned int paged;
-    unsigned short aging;
-    unsigned short dont_cache_flag;
-    unsigned short handle_is_fd;
+    uint64_t paged;
+    uint16_t aging;
+    uint16_t dont_cache_flag;
+    uint16_t handle_is_fd;
 
     /* memory interface */
     const void *data;
@@ -82,12 +81,71 @@ struct cl_fmap {
     HANDLE mh;
 #endif
     unsigned char maphash[16];
-    uint32_t placeholder_for_bitmap;
+    uint64_t *bitmap;
+    char *name;
 };
 
-fmap_t *fmap(int fd, off_t offset, size_t len);
-fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty);
+/**
+ * @brief Create a new fmap given a file descriptor.
+ *
+ * @param fd        File descriptor of file to be mapped.
+ * @param offset    Offset into file for start of map.
+ * @param len       Length from offset for size of map.
+ * @param name      (optional) Original name of the file (to set fmap name metadata)
+ * @return fmap_t*  The newly created fmap.  Free it with `funmap()`
+ */
+fmap_t *fmap(int fd, off_t offset, size_t len, const char *name);
 
+/**
+ * @brief Create  new fmap given a file descriptor.
+ *
+ * This variant of fmap() provides a boolean output variable to indicate on
+ * failure if the failure was because the file is empty (not really a failure).
+ *
+ * @param fd        File descriptor of file to be mapped.
+ * @param offset    Offset into file for start of map.
+ * @param len       Length from offset for size of map.
+ * @param empty     [out] Boolean will be non-zero if the file couldn't be mapped because it is empty.
+ * @param name      (optional) Original name of the file (to set fmap name metadata)
+ * @return fmap_t*  The newly created fmap.  Free it with `funmap()`
+ */
+fmap_t *fmap_check_empty(int fd, off_t offset, size_t len, int *empty, const char *name);
+
+/**
+ * @brief Create a new fmap given a buffer.
+ *
+ * @param start     Start of a buffer that the fmap will reference.
+ * @param len       Length of the buffer.
+ * @param name      (optional) Original name of the file (to set fmap name metadata)
+ * @return fmap_t*
+ */
+fmap_t *fmap_open_memory(const void *start, size_t len, const char *name);
+
+/**
+ * @brief Create a new fmap view into another fmap.
+ *
+ * @param map       The parent fmap.
+ * @param offset    Offset for the start of the new fmap into the parent fmap.
+ * @param length    Length of the data from the offset for the new fmap.
+ * @param name      (optional) Original name of the file (to set fmap name metadata)
+ * @return fmap_t*  NULL if failure or a special fmap that the caller must free with free_duplicate_fmap()
+ */
+fmap_t *fmap_duplicate(cl_fmap_t *map, size_t offset, size_t length, const char *name);
+
+/**
+ * @brief Deallocate a _duplicated_ fmap.  Does not unmap the mapped region.
+ *
+ * This function should be used instead of `free()` to cleanup the optional fmap name.
+ *
+ * @param m The map to be free'd.
+ */
+void free_duplicate_fmap(cl_fmap_t *map);
+
+/**
+ * @brief Unmap/deallocate an fmap.
+ *
+ * @param m The map to be free'd.
+ */
 static inline void funmap(fmap_t *m)
 {
     m->unmap(m);
@@ -105,9 +163,7 @@ static inline const void *fmap_need_off_once(fmap_t *m, size_t at, size_t len)
 
 static inline size_t fmap_ptr2off(const fmap_t *m, const void *ptr)
 {
-    return (m->data ? (const char *)ptr - (const char *)m->data
-                    : (const char *)ptr - (const char *)m - m->hdrsz) -
-           m->nested_offset;
+    return (size_t)((const char *)ptr - (const char *)m->data) - m->nested_offset;
 }
 
 static inline const void *fmap_need_ptr(fmap_t *m, const void *ptr, size_t len)
@@ -131,7 +187,7 @@ static inline void fmap_unneed_ptr(fmap_t *m, const void *ptr, size_t len)
 }
 
 /**
- * @brief Read bytes from fmap at offset into destinatio buffer.
+ * @brief Read bytes from fmap at offset into destination buffer.
  *
  * @param m         fmap
  * @param dst       destination buffer
@@ -217,4 +273,6 @@ cl_error_t fmap_dump_to_file(fmap_t *map, const char *filepath, const char *tmpd
  */
 int fmap_fd(fmap_t *m);
 
+cl_error_t fmap_get_MD5(unsigned char *hash, fmap_t *map);
+
 #endif
