@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2011-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB <acab@clamav.net>
@@ -26,6 +26,7 @@
 #include "iso9660.h"
 #include "fmap.h"
 #include "str.h"
+#include "entconv.h"
 #include "hashtab.h"
 
 typedef struct {
@@ -58,7 +59,7 @@ static int iso_scan_file(const iso9660_t *iso, unsigned int block, unsigned int
     char *tmpf;
     int fd, ret = CL_SUCCESS;
 
-    if (cli_gentempfd(iso->ctx->engine->tmpdir, &tmpf, &fd) != CL_SUCCESS)
+    if (cli_gentempfd(iso->ctx->sub_tmpdir, &tmpf, &fd) != CL_SUCCESS)
         return CL_ETMPFILE;
 
     cli_dbgmsg("iso_scan_file: dumping to %s\n", tmpf);
@@ -81,7 +82,7 @@ static int iso_scan_file(const iso9660_t *iso, unsigned int block, unsigned int
     }
 
     if (!len)
-        ret = cli_magic_scandesc(fd, tmpf, iso->ctx);
+        ret = cli_magic_scan_desc(fd, tmpf, iso->ctx, iso->buf);
 
     close(fd);
     if (!iso->ctx->engine->keeptmp) {
@@ -104,7 +105,7 @@ static char *iso_string(iso9660_t *iso, const void *src, unsigned int len)
         memcpy(iso->buf, src, len);
         iso->buf[len]     = '\0';
         iso->buf[len + 1] = '\0';
-        utf8              = cli_utf16_to_utf8(iso->buf, len, UTF16_BE);
+        utf8              = cli_utf16_to_utf8(iso->buf, len, E_UTF16_BE);
         uutf8             = utf8 ? utf8 : "";
         strncpy(iso->buf, uutf8, sizeof(iso->buf));
         iso->buf[sizeof(iso->buf) - 1] = '\0';
