@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2011-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm <tkojm@clamav.net>
@@ -24,7 +24,6 @@
 
 #include "others.h"
 
-int cli_parsejpeg(cli_ctx *ctx);
-int cli_parsegif(cli_ctx *ctx);
+cl_error_t cli_parsejpeg(cli_ctx *ctx);
 
 #endif
