@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm <tkojm@clamav.net>
@@ -407,7 +407,7 @@ int cli_scanmacho(cli_ctx *ctx, struct cli_exe_info *fileinfo)
                     sections[sect].rva = EC32(section.addr, conv);
                     sections[sect].vsz = EC32(section.size, conv);
                     sections[sect].raw = EC32(section.offset, conv);
-                    if ((uint64_t) 1 << EC32(section.align, conv) > INT32_MAX) {
+                    if ((uint64_t)1 << EC32(section.align, conv) > INT32_MAX) {
                         cli_dbgmsg("cli_scanmacho: Section aligned is malformed\n");
                         free(sections);
                         RETURN_BROKEN;
@@ -562,7 +562,7 @@ int cli_scanmacho_unibin(cli_ctx *ctx)
         cli_dbgmsg("UNIBIN: Binary %u of %u\n", i + 1, fat_header.nfats);
         cli_dbgmsg("UNIBIN: File offset: %u\n", fat_arch.offset);
         cli_dbgmsg("UNIBIN: File size: %u\n", fat_arch.size);
-        ret = cli_map_scan(map, fat_arch.offset, fat_arch.size, ctx, CL_TYPE_ANY);
+        ret = cli_magic_scan_nested_fmap_type(map, fat_arch.offset, fat_arch.size, ctx, CL_TYPE_ANY, NULL);
         if (ret == CL_VIRUS)
             break;
     }
@@ -602,7 +602,7 @@ int cli_unpackmacho(cli_ctx *ctx)
                     cli_dbgmsg("cli_scanmacho: Unpacked and rebuilt executable\n");
                 lseek(ndesc, 0, SEEK_SET);
                 cli_dbgmsg("***** Scanning rebuilt Mach-O file *****\n");
-                if (cli_magic_scandesc(ndesc, tempfile, ctx) == CL_VIRUS) {
+                if (cli_magic_scan_desc(ndesc, tempfile, ctx, NULL) == CL_VIRUS) {
                     close(ndesc);
                     CLI_TMPUNLK();
                     free(tempfile);
