@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Kevin Lin <klin@sourcefire.com>
  *
@@ -34,7 +34,7 @@
 #include "clamav.h"
 #include "others.h"
 #include "mbr.h"
-#include "prtn_intxn.h"
+#include "partition_intersection.h"
 #include "scanners.h"
 #include "dconf.h"
 
@@ -64,17 +64,17 @@ enum MBR_STATE {
     SEEN_EMPTY
 };
 
-static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba,
+static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, size_t extlba,
                            size_t extlbasize, size_t sectorsize);
 static int mbr_check_mbr(struct mbr_boot_record *record, size_t maplen, size_t sectorsize);
 static int mbr_check_ebr(struct mbr_boot_record *record);
-static int mbr_primary_prtn_intxn(cli_ctx *ctx, struct mbr_boot_record mbr, size_t sectorsize);
-static int mbr_extended_prtn_intxn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t sectorsize);
+static int mbr_primary_partition_intersection(cli_ctx *ctx, struct mbr_boot_record mbr, size_t sectorsize);
+static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t sectorsize);
 
 int cli_mbr_check(const unsigned char *buff, size_t len, size_t maplen)
 {
     struct mbr_boot_record mbr;
-    off_t mbr_base    = 0;
+    size_t mbr_base   = 0;
     size_t sectorsize = 512;
 
     if (len < sectorsize) {
@@ -94,7 +94,7 @@ int cli_mbr_check(const unsigned char *buff, size_t len, size_t maplen)
 int cli_mbr_check2(cli_ctx *ctx, size_t sectorsize)
 {
     struct mbr_boot_record mbr;
-    off_t pos = 0, mbr_base = 0;
+    size_t pos = 0, mbr_base = 0;
     size_t maplen;
 
     if (!ctx || !ctx->fmap) {
@@ -140,7 +140,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     struct mbr_boot_record mbr;
     enum MBR_STATE state = SEEN_NOTHING;
     int ret = CL_CLEAN, detection = CL_CLEAN;
-    off_t pos = 0, mbr_base = 0, partoff = 0;
+    size_t pos = 0, mbr_base = 0, partoff = 0;
     unsigned i = 0, prtncount = 0;
     size_t maplen, partsize;
 
@@ -184,7 +184,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     }
 
     /* MBR is valid, examine bootstrap code */
-    ret = cli_map_scan(*ctx->fmap, 0, sectorsize, ctx, CL_TYPE_ANY);
+    ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, 0, sectorsize, ctx, CL_TYPE_ANY, NULL);
     if (ret != CL_CLEAN) {
         if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
             detection = CL_VIRUS;
@@ -194,7 +194,7 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
 
     /* check that the partition table has no intersections - HEURISTICS */
     if (SCAN_HEURISTIC_PARTITION_INTXN && (ctx->dconf->other & OTHER_CONF_PRTNINTXN)) {
-        ret = mbr_primary_prtn_intxn(ctx, mbr, sectorsize);
+        ret = mbr_primary_partition_intersection(ctx, mbr, sectorsize);
         if (ret != CL_CLEAN) {
             if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
                 detection = CL_VIRUS;
@@ -210,10 +210,10 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
         cli_dbgmsg("MBR Partition Entry %u:\n", i);
         cli_dbgmsg("Status: %u\n", mbr.entries[i].status);
         cli_dbgmsg("Type: %x\n", mbr.entries[i].type);
-        cli_dbgmsg("Blocks: [%u, +%u), ([%lu, +%lu))\n",
+        cli_dbgmsg("Blocks: [%u, +%u), ([%zu, +%zu))\n",
                    mbr.entries[i].firstLBA, mbr.entries[i].numLBA,
-                   (unsigned long)(mbr.entries[i].firstLBA * sectorsize),
-                   (unsigned long)(mbr.entries[i].numLBA * sectorsize));
+                   mbr.entries[i].firstLBA * sectorsize,
+                   mbr.entries[i].numLBA * sectorsize);
 
         /* Handle MBR entry based on type */
         if (mbr.entries[i].type == MBR_EMPTY) {
@@ -239,8 +239,8 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
 
             partoff  = mbr.entries[i].firstLBA * sectorsize;
             partsize = mbr.entries[i].numLBA * sectorsize;
-            mbr_parsemsg("cli_map_scan: [%u, +%u)\n", partoff, partsize);
-            ret = cli_map_scan(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY);
+            mbr_parsemsg("cli_magic_scan_nested_fmap_type: [%u, +%u)\n", partoff, partsize);
+            ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
             if (ret != CL_CLEAN) {
                 if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
                     detection = CL_VIRUS;
@@ -257,12 +257,12 @@ int cli_scanmbr(cli_ctx *ctx, size_t sectorsize)
     return detection;
 }
 
-static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t extlbasize, size_t sectorsize)
+static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t extlbasize, size_t sectorsize)
 {
     struct mbr_boot_record ebr;
     enum MBR_STATE state = SEEN_NOTHING;
     int ret = CL_CLEAN, detection = CL_CLEAN;
-    off_t pos = 0, mbr_base = 0, logiclba = 0, extoff = 0, partoff = 0;
+    size_t pos = 0, mbr_base = 0, logiclba = 0, extoff = 0, partoff = 0;
     size_t partsize, extsize;
     unsigned i = 0, j = 0;
 
@@ -381,7 +381,7 @@ static int mbr_scanextprtn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size
                         return CL_EFORMAT;
                     }
 
-                    ret = cli_map_scan(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY);
+                    ret = cli_magic_scan_nested_fmap_type(*ctx->fmap, partoff, partsize, ctx, CL_TYPE_PART_ANY, NULL);
                     if (ret != CL_CLEAN) {
                         if (SCAN_ALLMATCHES && (ret == CL_VIRUS))
                             detection = CL_VIRUS;
@@ -424,7 +424,7 @@ void mbr_convert_to_host(struct mbr_boot_record *record)
 static int mbr_check_mbr(struct mbr_boot_record *record, size_t maplen, size_t sectorsize)
 {
     unsigned i      = 0;
-    off_t partoff   = 0;
+    size_t partoff  = 0;
     size_t partsize = 0;
 
     for (i = 0; i < MBR_MAX_PARTITION_ENTRIES; ++i) {
@@ -481,21 +481,21 @@ static int mbr_check_ebr(struct mbr_boot_record *record)
 }
 
 /* this includes the overall bounds of extended partitions */
-static int mbr_primary_prtn_intxn(cli_ctx *ctx, struct mbr_boot_record mbr, size_t sectorsize)
+static int mbr_primary_partition_intersection(cli_ctx *ctx, struct mbr_boot_record mbr, size_t sectorsize)
 {
-    prtn_intxn_list_t prtncheck;
+    partition_intersection_list_t prtncheck;
     unsigned i = 0, pitxn = 0, prtncount = 0;
     int ret = CL_CLEAN, tmp = CL_CLEAN;
 
-    prtn_intxn_list_init(&prtncheck);
+    partition_intersection_list_init(&prtncheck);
 
     for (i = 0; i < MBR_MAX_PARTITION_ENTRIES && prtncount < ctx->engine->maxpartitions; ++i) {
         if (mbr.entries[i].type == MBR_EMPTY) {
             /* empty partition entry */
             prtncount++;
         } else {
-            tmp = prtn_intxn_list_check(&prtncheck, &pitxn, mbr.entries[i].firstLBA,
-                                        mbr.entries[i].numLBA);
+            tmp = partition_intersection_list_check(&prtncheck, &pitxn, mbr.entries[i].firstLBA,
+                                                    mbr.entries[i].numLBA);
             if (tmp != CL_CLEAN) {
                 if (tmp == CL_VIRUS) {
                     cli_dbgmsg("cli_scanmbr: detected intersection with partitions "
@@ -514,17 +514,17 @@ static int mbr_primary_prtn_intxn(cli_ctx *ctx, struct mbr_boot_record mbr, size
 
             if (mbr.entries[i].type == MBR_EXTENDED) {
                 /* check the logical partitions */
-                tmp = mbr_extended_prtn_intxn(ctx, &prtncount,
-                                              mbr.entries[i].firstLBA, sectorsize);
+                tmp = mbr_extended_partition_intersection(ctx, &prtncount,
+                                                          mbr.entries[i].firstLBA, sectorsize);
                 if (tmp != CL_CLEAN) {
                     if (SCAN_ALLMATCHES && (tmp == CL_VIRUS)) {
                         ret = tmp;
                         tmp = 0;
                     } else if (tmp == CL_VIRUS) {
-                        prtn_intxn_list_free(&prtncheck);
+                        partition_intersection_list_free(&prtncheck);
                         return CL_VIRUS;
                     } else {
-                        prtn_intxn_list_free(&prtncheck);
+                        partition_intersection_list_free(&prtncheck);
                         return tmp;
                     }
                 }
@@ -535,23 +535,23 @@ static int mbr_primary_prtn_intxn(cli_ctx *ctx, struct mbr_boot_record mbr, size
     }
 
 leave:
-    prtn_intxn_list_free(&prtncheck);
+    partition_intersection_list_free(&prtncheck);
     return ret;
 }
 
 /* checks internal logical partitions */
-static int mbr_extended_prtn_intxn(cli_ctx *ctx, unsigned *prtncount, off_t extlba, size_t sectorsize)
+static int mbr_extended_partition_intersection(cli_ctx *ctx, unsigned *prtncount, size_t extlba, size_t sectorsize)
 {
     struct mbr_boot_record ebr;
-    prtn_intxn_list_t prtncheck;
+    partition_intersection_list_t prtncheck;
     unsigned i, pitxn;
     int ret = CL_CLEAN, tmp = CL_CLEAN, mbr_base = 0;
-    off_t pos = 0, logiclba = 0;
+    size_t pos = 0, logiclba = 0;
     int virus_found = 0;
 
     mbr_base = sectorsize - sizeof(struct mbr_boot_record);
 
-    prtn_intxn_list_init(&prtncheck);
+    partition_intersection_list_init(&prtncheck);
 
     logiclba = 0;
     i        = 0;
@@ -562,7 +562,7 @@ static int mbr_extended_prtn_intxn(cli_ctx *ctx, unsigned *prtncount, off_t extl
         pos += (logiclba * sectorsize) + mbr_base;
         if (fmap_readn(*ctx->fmap, &ebr, pos, sizeof(ebr)) != sizeof(ebr)) {
             cli_dbgmsg("cli_scanebr: Invalid extended boot record\n");
-            prtn_intxn_list_free(&prtncheck);
+            partition_intersection_list_free(&prtncheck);
             return CL_EFORMAT;
         }
 
@@ -573,7 +573,7 @@ static int mbr_extended_prtn_intxn(cli_ctx *ctx, unsigned *prtncount, off_t extl
         (*prtncount)++;
 
         /* assume that logical record is first and extended is second */
-        tmp = prtn_intxn_list_check(&prtncheck, &pitxn, logiclba, ebr.entries[0].numLBA);
+        tmp = partition_intersection_list_check(&prtncheck, &pitxn, logiclba, ebr.entries[0].numLBA);
         if (tmp != CL_CLEAN) {
             if (tmp == CL_VIRUS) {
                 cli_dbgmsg("cli_scanebr: detected intersection with partitions "
@@ -604,7 +604,7 @@ static int mbr_extended_prtn_intxn(cli_ctx *ctx, unsigned *prtncount, off_t extl
     } while (logiclba != 0 && (*prtncount) < ctx->engine->maxpartitions);
 
 leave:
-    prtn_intxn_list_free(&prtncheck);
+    partition_intersection_list_free(&prtncheck);
     if (virus_found)
         return CL_VIRUS;
     return ret;
