@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -93,7 +93,7 @@ struct msexp_hdr {
         return CL_SUCCESS;                           \
     w = 0;
 
-int cli_msexpand(cli_ctx *ctx, int ofd)
+cl_error_t cli_msexpand(cli_ctx *ctx, int ofd)
 {
     const struct msexp_hdr *hdr;
     uint8_t i, mask, bits;
