@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog
@@ -40,6 +40,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <stdbool.h>
 
 #include "clamav.h"
 #include "others.h"
@@ -111,6 +112,7 @@ typedef struct ole2_header_tag {
     struct uniq *U;
     fmap_t *map;
     int has_vba;
+    int has_xlm;
     hwp5_header_t *is_hwp;
 } ole2_header_t;
 
@@ -227,27 +229,27 @@ int ole2_list_delete(ole2_list_t *list)
 
 static unsigned char magic_id[] = {0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1};
 
-static char *
-get_property_name2(char *name, int size)
+char *
+cli_ole2_get_property_name2(const char *name, int size)
 {
     int i, j;
     char *newname;
 
-    if (*name == 0 || size <= 0 || size > 128) {
+    if ((name[0] == 0 && name[1] == 0) || size <= 0 || size > 128) {
         return NULL;
     }
     newname = (char *)cli_malloc(size * 7);
     if (!newname) {
-        cli_errmsg("OLE2 [get_property_name2]: Unable to allocate memory for newname: %u\n", size * 7);
+        cli_errmsg("OLE2 [cli_ole2_get_property_name2]: Unable to allocate memory for newname: %u\n", size * 7);
         return NULL;
     }
     j = 0;
     /* size-2 to ignore trailing NULL */
     for (i = 0; i < size - 2; i += 2) {
-        if ((!(name[i] & 0x80)) && isprint(name[i])) {
+        if ((!(name[i] & 0x80)) && isprint(name[i]) && name[i + 1] == 0) {
             newname[j++] = tolower(name[i]);
         } else {
-            if (name[i] < 10 && name[i] >= 0) {
+            if (name[i] < 10 && name[i] >= 0 && name[i + 1] == 0) {
                 newname[j++] = '_';
                 newname[j++] = name[i] + '0';
             } else {
@@ -293,7 +295,7 @@ get_property_name(char *name, int size)
         oname += 2;
         if (u > 0x1040) {
             free(newname);
-            return get_property_name2(name, size);
+            return cli_ole2_get_property_name2(name, size);
         }
         lo = u % 64;
         u >>= 6;
@@ -391,9 +393,9 @@ ole2_read_block(ole2_header_t *hdr, void *buff, unsigned int size, int32_t block
         return FALSE;
     }
     /* other methods: (blockno+1) * 512 or (blockno * block_size) + 512; */
-    if (( (uint64_t) blockno << hdr->log2_big_block_size) < (INT32_MAX - MAX(512, (uint64_t) 1 << hdr->log2_big_block_size) )) {
-	/* 512 is header size */
-	offset = (blockno << hdr->log2_big_block_size) + MAX(512, 1 << hdr->log2_big_block_size);
+    if (((uint64_t)blockno << hdr->log2_big_block_size) < (INT32_MAX - MAX(512, (uint64_t)1 << hdr->log2_big_block_size))) {
+        /* 512 is header size */
+        offset = (blockno << hdr->log2_big_block_size) + MAX(512, 1 << hdr->log2_big_block_size);
         offend = offset + size;
     } else {
         offset = INT32_MAX - size;
@@ -723,7 +725,7 @@ ole2_walk_property_tree(ole2_header_t *hdr, const char *dir, int32_t prop_index,
 #if HAVE_JSON
                     if (SCAN_COLLECT_METADATA && (ctx->wrkproperty != NULL)) {
                         if (!json_object_object_get_ex(ctx->wrkproperty, "DigitalSignatures", NULL)) {
-                            name = get_property_name2(prop_block[idx].name, prop_block[idx].name_size);
+                            name = cli_ole2_get_property_name2(prop_block[idx].name, prop_block[idx].name_size);
                             if (name) {
                                 if (!strcmp(name, "_xmlsignatures") || !strcmp(name, "_signatures")) {
                                     cli_jsonbool(ctx->wrkproperty, "HasDigitalSignatures", 1);
@@ -811,8 +813,9 @@ handler_writefile(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx
         cli_dbgmsg("OLE2 [handler_writefile]: property name too long: %d\n", prop->name_size);
         return CL_SUCCESS;
     }
-    name = get_property_name2(prop->name, prop->name_size);
+    name = cli_ole2_get_property_name2(prop->name, prop->name_size);
     if (name) {
+        cli_dbgmsg("Storing %s in uniq\n", name);
         if (CL_SUCCESS != uniq_add(hdr->U, name, strlen(name), &hash, &cnt)) {
             free(name);
             cli_dbgmsg("OLE2 [handler_writefile]: too many property names added to uniq store.\n");
@@ -830,7 +833,7 @@ handler_writefile(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx
     if (name)
         free(name);
 
-    ofd = open(newname, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, S_IRWXU);
+    ofd = open(newname, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, S_IRUSR | S_IWUSR);
     if (ofd < 0) {
         cli_errmsg("OLE2 [handler_writefile]: failed to create file: %s\n", newname);
         return CL_SUCCESS;
@@ -918,6 +921,227 @@ handler_writefile(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx
     return CL_SUCCESS;
 }
 
+enum biff_parser_states {
+    BIFF_PARSER_INITIAL,
+    BIFF_PARSER_EXPECTING_2ND_TAG_BYTE,
+    BIFF_PARSER_EXPECTING_1ST_LENGTH_BYTE,
+    BIFF_PARSER_EXPECTING_2ND_LENGTH_BYTE,
+    BIFF_PARSER_NAME_RECORD,
+    BIFF_PARSER_BOUNDSHEET_RECORD,
+    BIFF_PARSER_DATA,
+};
+
+struct biff_parser_state {
+    enum biff_parser_states state;
+    uint16_t opcode;
+    uint16_t length;
+    uint16_t data_offset;
+    uint8_t tmp;
+};
+
+/**
+ * Scan through a buffer of BIFF records and find PARSERNAME, BOUNDSHEET records (Which indicate XLM  macros).
+ * BIFF streams follow the format OOLLDDDDDDDDD..., where OO is the opcode (little endian 16 bit value),
+ * LL is the data length (little endian 16 bit value), followed by LL bytes of data. Records are defined in
+ * the MICROSOFT OFFICE EXCEL 97-2007 BINARY FILE FORMAT SPECIFICATION.
+ *
+ * \param state The parser state.
+ * \param buff The buffer.
+ * \param len The buffer's size in bytes.
+ * \param ctx The ClamAV context for emitting JSON about the document.
+ * \returns true if a macro has been found, false otherwise.
+ */
+static bool
+scan_biff_for_xlm_macros(struct biff_parser_state *state, unsigned char *buff, size_t len, cli_ctx *ctx)
+{
+    size_t i;
+    bool found_macro = false;
+
+    for (i = 0; i < len; ++i) {
+        switch (state->state) {
+            case BIFF_PARSER_INITIAL:
+                state->opcode = buff[i];
+                state->state  = BIFF_PARSER_EXPECTING_2ND_TAG_BYTE;
+                break;
+            case BIFF_PARSER_EXPECTING_2ND_TAG_BYTE:
+                state->opcode |= buff[i] << 8;
+                state->state = BIFF_PARSER_EXPECTING_1ST_LENGTH_BYTE;
+                break;
+            case BIFF_PARSER_EXPECTING_1ST_LENGTH_BYTE:
+                state->length = buff[i];
+                state->state  = BIFF_PARSER_EXPECTING_2ND_LENGTH_BYTE;
+                break;
+            case BIFF_PARSER_EXPECTING_2ND_LENGTH_BYTE:
+                state->length |= buff[i] << 8;
+                state->data_offset = 0;
+                switch (state->opcode) {
+                    case 0x85:
+                        state->state = BIFF_PARSER_BOUNDSHEET_RECORD;
+                        break;
+                    case 0x18:
+                        state->state = BIFF_PARSER_NAME_RECORD;
+                        break;
+                    default:
+                        state->state = BIFF_PARSER_DATA;
+                        break;
+                }
+                if (state->length == 0) {
+                    state->state = BIFF_PARSER_INITIAL;
+                }
+                break;
+            default:
+                switch (state->state) {
+                    case BIFF_PARSER_NAME_RECORD:
+#if HAVE_JSON
+                        if (state->data_offset == 0) {
+                            state->tmp = buff[i] & 0x20;
+                        } else if ((state->data_offset == 14 || state->data_offset == 15) && state->tmp) {
+                            if (buff[i] == 1 || buff[i] == 2) {
+                                if (SCAN_COLLECT_METADATA && (ctx->wrkproperty != NULL)) {
+                                    json_object *indicators = cli_jsonarray(ctx->wrkproperty, "MacroIndicators");
+                                    if (indicators) {
+                                        cli_jsonstr(indicators, NULL, "autorun");
+                                    } else {
+                                        cli_dbgmsg("[scan_biff_for_xlm_macros] Failed to add \"autorun\" entry to MacroIndicators JSON array\n");
+                                    }
+                                }
+                            }
+
+                            if (buff[i] != 0) {
+                                state->tmp = 0;
+                            }
+                        }
+#endif
+                        break;
+                    case BIFF_PARSER_BOUNDSHEET_RECORD:
+                        if (state->data_offset == 4) {
+                            state->tmp = buff[i];
+                        } else if (state->data_offset == 5 && buff[i] == 1) { //Excel 4.0 macro sheet
+                            cli_dbgmsg("[scan_biff_for_xlm_macros] Found XLM macro sheet\n");
+#if HAVE_JSON
+                            if (SCAN_COLLECT_METADATA && (ctx->wrkproperty != NULL)) {
+                                cli_jsonbool(ctx->wrkproperty, "HasMacros", 1);
+                                json_object *macro_languages = cli_jsonarray(ctx->wrkproperty, "MacroLanguages");
+                                if (macro_languages) {
+                                    cli_jsonstr(macro_languages, NULL, "XLM");
+                                } else {
+                                    cli_dbgmsg("[scan_biff_for_xlm_macros] Failed to add \"XLM\" entry to MacroLanguages JSON array\n");
+                                }
+                                if (state->tmp == 1 || state->tmp == 2) {
+                                    json_object *indicators = cli_jsonarray(ctx->wrkproperty, "MacroIndicators");
+                                    if (indicators) {
+                                        cli_jsonstr(indicators, NULL, "hidden");
+                                    } else {
+                                        cli_dbgmsg("[scan_biff_for_xlm_macros] Failed to add \"hidden\" entry to MacroIndicators JSON array\n");
+                                    }
+                                }
+                            }
+#endif
+                            found_macro = true;
+                        }
+                        break;
+                    case BIFF_PARSER_DATA:
+                        break;
+                    default:
+                        //Should never arrive here
+                        cli_dbgmsg("[scan_biff_for_xlm_macros] Unexpected state value %d\n", (int)state->state);
+                        break;
+                }
+                state->data_offset += 1;
+
+                if (state->data_offset >= state->length) {
+                    state->state = BIFF_PARSER_INITIAL;
+                }
+        }
+    }
+    return found_macro;
+}
+
+/**
+ * Scan for XLM (Excel 4.0) macro sheets in an OLE2 Workbook stream.
+ * The stream should be encoded with <= BIFF8.
+ */
+static int
+scan_for_xlm_macros(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
+{
+    unsigned char *buff = NULL;
+    int32_t current_block;
+    size_t len, offset;
+    bitset_t *blk_bitset = NULL;
+    struct biff_parser_state state;
+    bool found_macro = false;
+
+    UNUSEDPARAM(ctx);
+    UNUSEDPARAM(dir);
+
+    if (prop->type != 2) {
+        /* Not a file */
+        goto done;
+    }
+
+    state.state   = BIFF_PARSER_INITIAL;
+    state.length  = 0;
+    current_block = prop->start_block;
+    len           = prop->size;
+
+    buff = (unsigned char *)cli_malloc(1 << hdr->log2_big_block_size);
+    if (!buff) {
+        cli_errmsg("OLE2 [scan_for_xlm_macros]: Unable to allocate memory for buff: %u\n", 1 << hdr->log2_big_block_size);
+        goto done;
+    }
+    blk_bitset = cli_bitset_init();
+    if (!blk_bitset) {
+        cli_errmsg("OLE2 [scan_for_xlm_macros]: init bitset failed\n");
+        goto done;
+    }
+    while ((current_block >= 0) && (len > 0)) {
+        if (current_block > (int32_t)hdr->max_block_no) {
+            cli_dbgmsg("OLE2 [scan_for_xlm_macros]: Max block number for file size exceeded: %d\n", current_block);
+            goto done;
+        }
+        /* Check we aren't in a loop */
+        if (cli_bitset_test(blk_bitset, (unsigned long)current_block)) {
+            /* Loop in block list */
+            cli_dbgmsg("OLE2 [scan_for_xlm_macros]: Block list loop detected\n");
+            goto done;
+        }
+        if (!cli_bitset_set(blk_bitset, (unsigned long)current_block)) {
+            goto done;
+        }
+        if (prop->size < (int64_t)hdr->sbat_cutoff) {
+            /* Small block file */
+            if (!ole2_get_sbat_data_block(hdr, buff, current_block)) {
+                cli_dbgmsg("OLE2 [scan_for_xlm_macros]: ole2_get_sbat_data_block failed\n");
+                goto done;
+            }
+            /* buff now contains the block with N small blocks in it */
+            offset = (1 << hdr->log2_small_block_size) * (current_block % (1 << (hdr->log2_big_block_size - hdr->log2_small_block_size)));
+
+            found_macro = scan_biff_for_xlm_macros(&state, &buff[offset], MIN(len, 1 << hdr->log2_small_block_size), ctx) || found_macro;
+            len -= MIN(len, 1 << hdr->log2_small_block_size);
+            current_block = ole2_get_next_sbat_block(hdr, current_block);
+        } else {
+            /* Big block file */
+            if (!ole2_read_block(hdr, buff, 1 << hdr->log2_big_block_size, current_block)) {
+                goto done;
+            }
+
+            found_macro   = scan_biff_for_xlm_macros(&state, buff, MIN(len, (1 << hdr->log2_big_block_size)), ctx) || found_macro;
+            current_block = ole2_get_next_block_number(hdr, current_block);
+            len -= MIN(len, (1 << hdr->log2_big_block_size));
+        }
+    }
+
+done:
+    if (buff) {
+        free(buff);
+    }
+    if (blk_bitset) {
+        cli_bitset_free(blk_bitset);
+    }
+    return found_macro;
+}
+
 /* enum file Handler - checks for VBA presence */
 static int
 handler_enum(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
@@ -929,7 +1153,7 @@ handler_enum(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx
 #if HAVE_JSON
     json_object *arrobj, *strmobj;
 
-    name = get_property_name2(prop->name, prop->name_size);
+    name = cli_ole2_get_property_name2(prop->name, prop->name_size);
     if (name) {
         if (SCAN_COLLECT_METADATA && ctx->wrkproperty != NULL) {
             arrobj = cli_jsonarray(ctx->wrkproperty, "Streams");
@@ -958,7 +1182,7 @@ handler_enum(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx
 
     if (!hdr->has_vba) {
         if (!name)
-            name = get_property_name2(prop->name, prop->name_size);
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
         if (name) {
             if (!strcmp(name, "_vba_project") || !strcmp(name, "powerpoint document") || !strcmp(name, "worddocument") || !strcmp(name, "_1_ole10native"))
                 hdr->has_vba = 1;
@@ -971,7 +1195,7 @@ handler_enum(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx
      */
     if (!hdr->is_hwp) {
         if (!name)
-            name = get_property_name2(prop->name, prop->name_size);
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
         if (name) {
             if (!strcmp(name, "fileheader")) {
                 hwp_check = (unsigned char *)cli_calloc(1, 1 << hdr->log2_big_block_size);
@@ -1034,6 +1258,16 @@ handler_enum(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx
         }
     }
 
+    if (!hdr->has_xlm) {
+        if (!name) {
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
+        }
+
+        if (name && (strcmp(name, "workbook") == 0 || strcmp(name, "book") == 0)) {
+            hdr->has_xlm = scan_for_xlm_macros(hdr, prop, dir, ctx);
+        }
+    }
+
     if (name)
         free(name);
     return ret;
@@ -1093,7 +1327,7 @@ static cl_error_t scan_mso_stream(int fd, cli_ctx *ctx)
             return CL_ESTAT;
         }
 
-        input = fmap(fd, 0, statbuf.st_size);
+        input = fmap(fd, 0, statbuf.st_size, NULL);
         if (!input) {
             cli_dbgmsg("scan_mso_stream: Failed to get fmap for input stream\n");
             return CL_EMAP;
@@ -1101,7 +1335,7 @@ static cl_error_t scan_mso_stream(int fd, cli_ctx *ctx)
     }
 
     /* reserve tempfile for output and scanning */
-    if ((ret = cli_gentempfd(ctx->engine->tmpdir, &tmpname, &ofd)) != CL_SUCCESS) {
+    if ((ret = cli_gentempfd(ctx->sub_tmpdir, &tmpname, &ofd)) != CL_SUCCESS) {
         cli_errmsg("scan_mso_stream: Can't generate temporary file\n");
         funmap(input);
         return ret;
@@ -1192,7 +1426,7 @@ static cl_error_t scan_mso_stream(int fd, cli_ctx *ctx)
     }
 
     /* scanning inflated stream */
-    ret = cli_magic_scandesc(ofd, tmpname, ctx);
+    ret = cli_magic_scan_desc(ofd, tmpname, ctx, NULL);
 
     /* clean-up */
 mso_end:
@@ -1226,10 +1460,10 @@ handler_otf(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
     }
     print_ole2_property(prop);
 
-    if (!(tempfile = cli_gentemp(ctx ? ctx->engine->tmpdir : NULL)))
+    if (!(tempfile = cli_gentemp(ctx ? ctx->sub_tmpdir : NULL)))
         return CL_EMEM;
 
-    if ((ofd = open(tempfile, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IRWXU)) < 0) {
+    if ((ofd = open(tempfile, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, S_IRUSR | S_IWUSR)) < 0) {
         cli_dbgmsg("OLE2: Can't create file %s\n", tempfile);
         free(tempfile);
         return CL_ECREAT;
@@ -1239,7 +1473,7 @@ handler_otf(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
 
     if (cli_debug_flag) {
         if (!name)
-            name = get_property_name2(prop->name, prop->name_size);
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
         cli_dbgmsg("OLE2 [handler_otf]: Dumping '%s' to '%s'\n", name, tempfile);
     }
 
@@ -1347,7 +1581,7 @@ handler_otf(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
     /* JSON Output Summary Information */
     if (SCAN_COLLECT_METADATA && (ctx->properties != NULL)) {
         if (!name)
-            name = get_property_name2(prop->name, prop->name_size);
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
         if (name) {
             if (!strncmp(name, "_5_summaryinformation", 21)) {
                 cli_dbgmsg("OLE2: detected a '_5_summaryinformation' stream\n");
@@ -1385,7 +1619,7 @@ handler_otf(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
 
     if (hdr->is_hwp) {
         if (!name)
-            name = get_property_name2(prop->name, prop->name_size);
+            name = cli_ole2_get_property_name2(prop->name, prop->name_size);
         ret = cli_scanhwp5_stream(ctx, hdr->is_hwp, name, ofd, tempfile);
     } else if (is_mso < 0) {
         ret = CL_ESEEK;
@@ -1394,7 +1628,7 @@ handler_otf(ole2_header_t *hdr, property_t *prop, const char *dir, cli_ctx *ctx)
         ret = scan_mso_stream(ofd, ctx);
     } else {
         /* Normal File Scan */
-        ret = cli_magic_scandesc(ofd, tempfile, ctx);
+        ret = cli_magic_scan_desc(ofd, tempfile, ctx, NULL);
     }
     if (name)
         free(name);
@@ -1474,7 +1708,7 @@ ole2_read_header(int fd, ole2_header_t *hdr)
 }
 #endif
 
-int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **vba)
+int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **files, int *has_vba, int *has_xlm)
 {
     ole2_header_t hdr;
     int ret = CL_CLEAN;
@@ -1567,6 +1801,7 @@ int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **vba)
 
     /* PASS 1 : Count files and check for VBA */
     hdr.has_vba = 0;
+    hdr.has_xlm = 0;
     ret         = ole2_walk_property_tree(&hdr, NULL, 0, handler_enum, 0, &file_count, ctx, &scansize);
     cli_bitset_free(hdr.bitset);
     hdr.bitset = NULL;
@@ -1585,7 +1820,7 @@ int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **vba)
     }
 
     /* If there's no VBA we scan OTF */
-    if (hdr.has_vba) {
+    if (hdr.has_vba || hdr.has_xlm) {
         /* PASS 2/A : VBA scan */
         cli_dbgmsg("OLE2: VBA project found\n");
         if (!(hdr.U = uniq_init(file_count))) {
@@ -1595,8 +1830,14 @@ int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **vba)
         }
         file_count = 0;
         ole2_walk_property_tree(&hdr, dirname, 0, handler_writefile, 0, &file_count, ctx, &scansize2);
-        ret  = CL_CLEAN;
-        *vba = hdr.U;
+        ret    = CL_CLEAN;
+        *files = hdr.U;
+        if (has_vba) {
+            *has_vba = hdr.has_vba;
+        }
+        if (has_xlm) {
+            *has_xlm = hdr.has_xlm;
+        }
     } else {
         cli_dbgmsg("OLE2: no VBA projects found\n");
         /* PASS 2/B : OTF scan */
