@@ -1,11 +1,11 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog
- * 
+ *
  *  Summary: Extract component parts of OLE2 files (e.g. MS Office Documents).
- * 
+ *
  *  Acknowledgements: Some ideas and algorithms were based upon OpenOffice and libgsf.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include "others.h"
 #include "uniq.h"
 
-int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **);
+int cli_ole2_extract(const char *dirname, cli_ctx *ctx, struct uniq **files, int *has_vba, int *has_xlm);
+char *cli_ole2_get_property_name2(const char *name, int size);
 
 #endif
