@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2014-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Kevin Lin
  *
@@ -27,7 +27,7 @@
 
 #include "others.h"
 
-int cli_ooxml_filetype(cli_ctx *, fmap_t *);
-int cli_process_ooxml(cli_ctx *, int);
+cli_file_t cli_ooxml_filetype(cli_ctx *, fmap_t *);
+cl_error_t cli_process_ooxml(cli_ctx *, int);
 
 #endif
