@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Trog
@@ -173,9 +173,11 @@ static void *get_module_function(lt_dlhandle handle, const char *name)
 static void *load_module(const char *name, const char *featurename)
 {
     static const char *suffixes[] = {
+#ifndef _WIN32
         LT_MODULE_EXT "." LIBCLAMAV_FULLVER,
         PASTE(LT_MODULE_EXT ".", LIBCLAMAV_MAJORVER),
         LT_MODULE_EXT,
+#endif
         "." LT_LIBEXT};
 
     const char *searchpath;
@@ -196,7 +198,7 @@ static void *load_module(const char *name, const char *featurename)
 #ifdef _WIN32
         rhandle = LoadLibraryA(modulename);
 #else  // !_WIN32
-        rhandle = dlopen(modulename, RTLD_NOW);
+        rhandle         = dlopen(modulename, RTLD_NOW);
 #endif // !_WIN32
         if (rhandle) {
             break;
@@ -415,6 +417,8 @@ const char *cl_strerror(int clerror)
             return "Scanner still active";
         case CL_ESTATE:
             return "Bad state (engine not initialized, or already initialized)";
+        case CL_ERROR:
+            return "Unspecified error";
         case CL_VERIFIED:
             return "The scanned object was verified and deemed trusted";
         default:
@@ -438,7 +442,7 @@ cl_error_t cl_init(unsigned int initoptions)
         rarload();
     }
 #else
-        rarload();
+    rarload();
 #endif
 
     gettimeofday(&tv, (struct timezone *)0);
@@ -1261,9 +1265,9 @@ int cli_unlink(const char *pathname)
         }
         return 0;
 #else
-            char err[128];
-            cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
-            return 1;
+        char err[128];
+        cli_warnmsg("cli_unlink: unlink failure - %s\n", cli_strerror(errno, err, sizeof(err)));
+        return 1;
 #endif
     }
     return 0;
@@ -1449,75 +1453,75 @@ int cli_rmdirs(const char *name)
     return rc;
 }
 #else
-    int cli_rmdirs(const char *dirname)
-    {
-        DIR *dd;
-        struct dirent *dent;
-        STATBUF maind, statbuf;
-        char *path;
-        char err[128];
+int cli_rmdirs(const char *dirname)
+{
+    DIR *dd;
+    struct dirent *dent;
+    STATBUF maind, statbuf;
+    char *path;
+    char err[128];
 
-        chmod(dirname, 0700);
-        if ((dd = opendir(dirname)) != NULL) {
-            while (CLAMSTAT(dirname, &maind) != -1) {
-                if (!rmdir(dirname)) break;
-                if (errno != ENOTEMPTY && errno != EEXIST && errno != EBADF) {
-                    cli_errmsg("cli_rmdirs: Can't remove temporary directory %s: %s\n", dirname, cli_strerror(errno, err, sizeof(err)));
-                    closedir(dd);
-                    return -1;
-                }
+    chmod(dirname, 0700);
+    if ((dd = opendir(dirname)) != NULL) {
+        while (CLAMSTAT(dirname, &maind) != -1) {
+            if (!rmdir(dirname)) break;
+            if (errno != ENOTEMPTY && errno != EEXIST && errno != EBADF) {
+                cli_errmsg("cli_rmdirs: Can't remove temporary directory %s: %s\n", dirname, cli_strerror(errno, err, sizeof(err)));
+                closedir(dd);
+                return -1;
+            }
 
-                while ((dent = readdir(dd))) {
-                    if (dent->d_ino) {
-                        if (strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
-                            path = cli_malloc(strlen(dirname) + strlen(dent->d_name) + 2);
-                            if (!path) {
-                                cli_errmsg("cli_rmdirs: Unable to allocate memory for path %llu\n", (long long unsigned)(strlen(dirname) + strlen(dent->d_name) + 2));
-                                closedir(dd);
-                                return -1;
-                            }
+            while ((dent = readdir(dd))) {
+                if (dent->d_ino) {
+                    if (strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
+                        path = cli_malloc(strlen(dirname) + strlen(dent->d_name) + 2);
+                        if (!path) {
+                            cli_errmsg("cli_rmdirs: Unable to allocate memory for path %llu\n", (long long unsigned)(strlen(dirname) + strlen(dent->d_name) + 2));
+                            closedir(dd);
+                            return -1;
+                        }
 
-                            sprintf(path, "%s" PATHSEP "%s", dirname, dent->d_name);
+                        sprintf(path, "%s" PATHSEP "%s", dirname, dent->d_name);
 
-                            /* stat the file */
-                            if (LSTAT(path, &statbuf) != -1) {
-                                if (S_ISDIR(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)) {
-                                    if (rmdir(path) == -1) { /* can't be deleted */
-                                        if (errno == EACCES) {
-                                            cli_errmsg("cli_rmdirs: Can't remove some temporary directories due to access problem.\n");
-                                            closedir(dd);
-                                            free(path);
-                                            return -1;
-                                        }
-                                        if (cli_rmdirs(path)) {
-                                            cli_warnmsg("cli_rmdirs: Can't remove nested directory %s\n", path);
-                                            free(path);
-                                            closedir(dd);
-                                            return -1;
-                                        }
+                        /* stat the file */
+                        if (LSTAT(path, &statbuf) != -1) {
+                            if (S_ISDIR(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)) {
+                                if (rmdir(path) == -1) { /* can't be deleted */
+                                    if (errno == EACCES) {
+                                        cli_errmsg("cli_rmdirs: Can't remove some temporary directories due to access problem.\n");
+                                        closedir(dd);
+                                        free(path);
+                                        return -1;
                                     }
-                                } else {
-                                    if (cli_unlink(path)) {
+                                    if (cli_rmdirs(path)) {
+                                        cli_warnmsg("cli_rmdirs: Can't remove nested directory %s\n", path);
                                         free(path);
                                         closedir(dd);
                                         return -1;
                                     }
                                 }
+                            } else {
+                                if (cli_unlink(path)) {
+                                    free(path);
+                                    closedir(dd);
+                                    return -1;
+                                }
                             }
-                            free(path);
                         }
+                        free(path);
                     }
                 }
-                rewinddir(dd);
             }
-
-        } else {
-            return -1;
+            rewinddir(dd);
         }
 
-        closedir(dd);
-        return 0;
+    } else {
+        return -1;
     }
+
+    closedir(dd);
+    return 0;
+}
 #endif
 
 /* Implement a generic bitset, trog@clamav.net */
