@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -39,20 +39,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#ifdef HAVE_JSON
+#include <json.h>
+#endif
+
 #include "clamav.h"
 #include "dconf.h"
 #include "filetypes.h"
 #include "fmap.h"
-#include "libclamunrar_iface/unrar_iface.h"
 #include "regex/regex.h"
 #include "bytecode.h"
 #include "bytecode_api.h"
 #include "events.h"
 #include "crtmgr.h"
 
-#ifdef HAVE_JSON
-#include "json-c/json.h"
-#endif
+#include "unrar_iface.h"
 
 #ifdef HAVE_YARA
 #include "yara_clam.h"
@@ -71,7 +72,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 115
+#define CL_FLEVEL 123
 #define CL_FLEVEL_DCONF CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -79,38 +80,36 @@ extern uint8_t cli_debug_flag;
 extern uint8_t cli_always_gen_section_hash;
 
 /*
- * CLI_ISCONTAINED(bb, bb_size, sb, sb_size) checks if sb (sub buffer) is contained
- * within bb (buffer).
+ * CLI_ISCONTAINED(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * within bb (big buffer).
  *
  * bb and sb are pointers (or offsets) for the main buffer and the
  * sub-buffer respectively, and bb_size and sb_size are their sizes
  *
  * The macro can be used to protect against wraps.
  */
-#define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)                                           \
-    (                                                                                       \
-        (size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&                                   \
-        (size_t)(sb_size) <= (size_t)(bb_size) &&                                           \
-        (ptrdiff_t)(sb) >= (ptrdiff_t)(bb) &&                                               \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) <= (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size) && \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) > (ptrdiff_t)(bb) &&                         \
-        (ptrdiff_t)(sb) < (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size))
+#define CLI_ISCONTAINED(bb, bb_size, sb, sb_size)                            \
+    ((size_t)(bb_size) > 0 && (size_t)(sb_size) > 0 &&                       \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                               \
+     (size_t)(sb) >= (size_t)(bb) &&                                         \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb) + (size_t)(bb_size) && \
+     (size_t)(sb) + (size_t)(sb_size) > (size_t)(bb) &&                      \
+     (size_t)(sb) < (size_t)(bb) + (size_t)(bb_size))
 
 /*
- * CLI_ISCONTAINED2(bb, bb_size, sb, sb_size) checks if sb (sub buffer) is contained
- * within bb (buffer).
+ * CLI_ISCONTAINED2(bb, bb_size, sb, sb_size) checks if sb (small buffer) is
+ * within bb (big buffer).
  *
- * CLI_ISCONTAINED2 is the same as CLI_ISCONTAINED except that it allows for sub-
- * buffers with sb_size == 0.
+ * CLI_ISCONTAINED2 is the same as CLI_ISCONTAINED except that it allows for
+ * small-buffers with sb_size == 0.
  */
-#define CLI_ISCONTAINED2(bb, bb_size, sb, sb_size)                                          \
-    (                                                                                       \
-        (size_t)(bb_size) > 0 && (size_t)(sb_size) >= 0 &&                                  \
-        (size_t)(sb_size) <= (size_t)(bb_size) &&                                           \
-        (ptrdiff_t)(sb) >= (ptrdiff_t)(bb) &&                                               \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) <= (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size) && \
-        (ptrdiff_t)(sb) + (ptrdiff_t)(sb_size) >= (ptrdiff_t)(bb) &&                        \
-        (ptrdiff_t)(sb) < (ptrdiff_t)(bb) + (ptrdiff_t)(bb_size))
+#define CLI_ISCONTAINED2(bb, bb_size, sb, sb_size)                           \
+    ((size_t)(bb_size) > 0 &&                                                \
+     (size_t)(sb_size) <= (size_t)(bb_size) &&                               \
+     (size_t)(sb) >= (size_t)(bb) &&                                         \
+     (size_t)(sb) + (size_t)(sb_size) <= (size_t)(bb) + (size_t)(bb_size) && \
+     (size_t)(sb) + (size_t)(sb_size) >= (size_t)(bb) &&                     \
+     (size_t)(sb) <= (size_t)(bb) + (size_t)(bb_size))
 
 #define CLI_MAX_ALLOCATION (182 * 1024 * 1024)
 
@@ -150,8 +149,9 @@ typedef struct cli_ctx_container_tag {
 
 /* internal clamav context */
 typedef struct cli_ctx_tag {
-    char *target_filepath;
-    const char *sub_filepath;
+    char *target_filepath;    /**< (optional) The filepath of the original scan target */
+    const char *sub_filepath; /**< (optional) The filepath of the current file being parsed. May be a temp file. */
+    char *sub_tmpdir;         /**< The directory to store tmp files at this recursion depth. */
     const char **virname;
     unsigned int num_viruses;
     unsigned long int *scanned;
@@ -167,7 +167,7 @@ typedef struct cli_ctx_tag {
     cli_ctx_container *containers; /* set container type after recurse */
     unsigned char handlertype_hash[16];
     struct cli_dconf *dconf;
-    fmap_t **fmap;
+    fmap_t **fmap; /* pointer to current fmap in an allocated array, incremented with recursion depth */
     bitset_t *hook_lsig_matches;
     void *cb_ctx;
     cli_events_t *perf;
@@ -487,7 +487,7 @@ extern cl_unrar_error_t (*cli_unrar_extract_file)(void *hArchive, const char *de
 extern cl_unrar_error_t (*cli_unrar_skip_file)(void *hArchive);
 extern void (*cli_unrar_close)(void *hArchive);
 
-extern int have_rar;
+extern LIBCLAMAV_EXPORT int have_rar;
 
 #define SCAN_ALLMATCHES (ctx->options->general & CL_SCAN_GENERAL_ALLMATCHES)
 #define SCAN_COLLECT_METADATA (ctx->options->general & CL_SCAN_GENERAL_COLLECT_METADATA)
@@ -507,6 +507,7 @@ extern int have_rar;
 #define SCAN_PARSE_PE (ctx->options->parse & CL_SCAN_PARSE_PE)
 
 #define SCAN_HEURISTIC_BROKEN (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN)
+#define SCAN_HEURISTIC_BROKEN_MEDIA (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN_MEDIA)
 #define SCAN_HEURISTIC_EXCEEDS_MAX (ctx->options->heuristic & CL_SCAN_HEURISTIC_EXCEEDS_MAX)
 #define SCAN_HEURISTIC_PHISHING_SSL_MISMATCH (ctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_SSL_MISMATCH)
 #define SCAN_HEURISTIC_PHISHING_CLOAK (ctx->options->heuristic & CL_SCAN_HEURISTIC_PHISHING_CLOAK)
@@ -635,7 +636,42 @@ static inline void cli_writeint32(void *offset, uint32_t value)
 }
 #endif
 
+/**
+ * @brief Append an alert.
+ *
+ * An FP-check will verify that the file is not whitelisted.
+ * The whitelist check does not happen before the scan because file whitelisting
+ * is so infrequent that such action would be detrimental to performance.
+ *
+ * TODO: Replace implementation with severity scale, and severity threshold
+ * wherein signatures that do not meet the threshold are documented in JSON
+ * metadata but do not halt the scan.
+ *
+ * @param ctx       The scan context.
+ * @param virname   The alert name.
+ * @return cl_error_t CL_VIRUS if scan should be halted due to an alert, CL_CLEAN if scan should continue.
+ */
 cl_error_t cli_append_virus(cli_ctx *ctx, const char *virname);
+
+/**
+ * @brief Append a PUA (low severity) alert.
+ *
+ * This function will return CLEAN unless in all-match or Heuristic-precedence
+ * modes. The intention is for the scan to continue in case something more
+ * malicious is found.
+ *
+ * TODO: Replace implementation with severity scale, and severity threshold
+ * wherein signatures that do not meet the threshold are documented in JSON
+ * metadata but do not halt the scan.
+ *
+ * BUG: In normal scan mode (see above), the alert is not FP-checked!
+ *
+ * @param ctx       The scan context.
+ * @param virname   The alert name.
+ * @return cl_error_t CL_VIRUS if scan should be halted due to an alert, CL_CLEAN if scan should continue.
+ */
+cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname);
+
 const char *cli_get_last_virus(const cli_ctx *ctx);
 const char *cli_get_last_virus_str(const cli_ctx *ctx);
 void cli_virus_found_cb(cli_ctx *ctx);
@@ -757,11 +793,16 @@ const char *cli_gettmpdir(void);
 /**
  * @brief Sanitize a relative path, so it cannot have a negative depth.
  *
- * Caller is responsible for freeing the filename.
+ * Caller is responsible for freeing the sanitized filepath.
+ * The optioal sanitized_filebase output param is a pointer into the filepath,
+ * if set, and does not need to be freed.
  *
- * @return char* filename or NULL.
+ * @param filepath                  The filepath to sanitize
+ * @param filepath_len              The length of the filepath
+ * @param[out] sanitized_filebase   Pointer to the basename portion of the sanitized filepath. (optional)
+ * @return char*
  */
-char *cli_sanitize_filepath(const char *filepath, size_t filepath_len);
+char *cli_sanitize_filepath(const char *filepath, size_t filepath_len, char **sanitized_filebase);
 
 /**
  * @brief Generate tempfile filename (no path) with a random MD5 hash.
@@ -772,6 +813,30 @@ char *cli_sanitize_filepath(const char *filepath, size_t filepath_len);
  */
 char *cli_genfname(const char *prefix);
 
+/**
+ * @brief Generate a full tempfile filepath with a provided the name.
+ *
+ * Caller is responsible for freeing the filename.
+ * If the dir is not provided, the engine->tmpdir will be used.
+ *
+ * @param dir 	 Alternative directory. (optional)
+ * @return char* filename or NULL.
+ */
+char *cli_newfilepath(const char *dir, const char *fname);
+
+/**
+ * @brief Generate a full tempfile filepath with a provided the name.
+ *
+ * Caller is responsible for freeing the filename.
+ * If the dir is not provided, the engine->tmpdir will be used.
+ *
+ * @param dir        Alternative temp directory (optional).
+ * @param prefix  	 (Optional) Base filename for new file.
+ * @param[out] name  Allocated filepath, must be freed by caller.
+ * @param[out] fd    File descriptor of open temp file.
+ */
+cl_error_t cli_newfilepathfd(const char *dir, char *fname, char **name, int *fd);
+
 /**
  * @brief Generate a full tempfile filepath with a random MD5 hash and prefix the name, if provided.
  *
@@ -828,7 +893,6 @@ int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
 void cli_qsort_r(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *, const void *), void *arg);
 cl_error_t cli_checktimelimit(cli_ctx *ctx);
-cl_error_t cli_append_possibly_unwanted(cli_ctx *ctx, const char *virname);
 
 /* symlink behaviour */
 #define CLI_FTW_FOLLOW_FILE_SYMLINK 0x01
@@ -865,7 +929,7 @@ struct cli_ftw_cbdata {
  * after an error, we call the callback with reason == error,
  * and if it returns CL_BREAK we break.
  */
-typedef int (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
+typedef cl_error_t (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
 
 /*
  * returns 1 if the path should be skipped and 0 otherwise
