@@ -1,7 +1,7 @@
 /*
  *  Match a string against a list of patterns/regexes.
  *
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -154,10 +154,10 @@ cl_error_t regex_list_match(struct regex_matcher *matcher, char *real_url, const
     struct regex_list *regex;
     size_t real_len, display_len, buffer_len;
 
-    char *buffer         = NULL;
-    char *bufrev         = NULL;
-    cl_error_t rc        = CL_SUCCESS;
-    int filter_search_rc = 0;
+    char *buffer  = NULL;
+    char *bufrev  = NULL;
+    cl_error_t rc = CL_SUCCESS;
+    //int filter_search_rc = 0;
     int root;
     struct cli_ac_data mdata;
     struct cli_ac_result *res = NULL;
@@ -208,15 +208,39 @@ cl_error_t regex_list_match(struct regex_matcher *matcher, char *real_url, const
         return CL_EMEM;
 
     reverse_string(bufrev);
-    filter_search_rc = filter_search(&matcher->filter, (const unsigned char *)bufrev, buffer_len) != -1;
-    if (filter_search_rc == -1) {
-        free(buffer);
-        free(bufrev);
-        /* filter says this suffix doesn't match.
-			 * The filter has false positives, but no false
-			 * negatives */
-        return CL_SUCCESS;
-    }
+    // TODO Add this back in once we improve the regex parsing code that finds
+    // suffixes to add to the filter.
+    //
+    // Reviewing Coverity bug reports we found that the return value to this
+    // filter_search call was effectively being ignored, causing no filtering
+    // to occur. Fixing this issue resulted in a unit test that uses the
+    // following match list regex to fail when searching for `ebay.com`.:
+    //
+    // .+\\.paypal\\.(com|de|fr|it)([/?].*)?:.+\\.ebay\\.(at|be|ca|ch|co\\.uk|de|es|fr|ie|in|it|nl|ph|pl|com(\\.(au|cn|hk|my|sg))?)/
+    //
+    // After investigating further, this is because the regex_list_add_pattern
+    // call, which parses the regex for suffixes and attempts to add these to
+    // the filter, can't handle the `com(\\.(au|cn|hk|my|sg))?` portion of
+    // the regex. As a result, it only adds `ebay.at`, `ebay.be`, `ebay.ca`, up
+    // through `ebay.pl` into the filter). With the commented out code below
+    // uncommented, these suffixes not existing in the filter are treated as
+    // there not being a corresponding regex for ebay.com, causing no regex
+    // rules to be evaluated against the URL.
+    //
+    // We should get the regex parsing code working (and ensure it handles any
+    // other complex cases in daily.cdb) before re-enabling this code. The code
+    // has had no effect for 12+ years at this point, though, so it's probably
+    // safe to wait a bit longer without it.
+    //
+    //filter_search_rc = filter_search(&matcher->filter, (const unsigned char *)bufrev, buffer_len);
+    //if (filter_search_rc == -1) {
+    //    free(buffer);
+    //    free(bufrev);
+    //    /* filter says this suffix doesn't match.
+    //     * The filter has false positives, but no false
+    //     * negatives */
+    //    return CL_SUCCESS;
+    //}
 
     rc = cli_ac_scanbuff((const unsigned char *)bufrev, buffer_len, NULL, (void *)&regex, &res, &matcher->suffixes, &mdata, 0, 0, NULL, AC_SCAN_VIR, NULL);
     free(bufrev);
@@ -448,6 +472,9 @@ cl_error_t load_regex_matcher(struct cl_engine *engine, struct regex_matcher *ma
         if (!*buffer)
             continue; /* skip empty lines */
 
+        if (buffer[0] == '#')
+            continue;
+
         if (functionality_level_check(buffer))
             continue;
 
@@ -629,7 +656,6 @@ static void list_add_tail(struct regex_list_ht *ht, struct regex_list *regex)
     ht->tail = regex;
 }
 
-/* returns 0 on success, clamav error code otherwise */
 static cl_error_t add_pattern_suffix(void *cbdata, const char *suffix, size_t suffix_len, const struct regex_list *iregex)
 {
     struct regex_matcher *matcher = cbdata;
