@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog, Török Edvin
@@ -92,154 +92,6 @@ int cli_check_mydoom_log(cli_ctx *ctx)
     return cli_append_virus(ctx, "Heuristics.Worm.Mydoom.M.log");
 }
 
-static int jpeg_check_photoshop_8bim(cli_ctx *ctx, off_t *off)
-{
-    const unsigned char *buf;
-    uint16_t ntmp;
-    uint8_t nlength, id[2];
-    uint32_t size;
-    off_t offset = *off;
-    int retval;
-    fmap_t *map = *ctx->fmap;
-
-    if (!(buf = fmap_need_off_once(map, offset, 4 + 2 + 1))) {
-        cli_dbgmsg("read bim failed\n");
-        return -1;
-    }
-    if (memcmp(buf, "8BIM", 4) != 0) {
-        cli_dbgmsg("missed 8bim\n");
-        return -1;
-    }
-
-    id[0] = (uint8_t)buf[4];
-    id[1] = (uint8_t)buf[5];
-    cli_dbgmsg("ID: 0x%.2x%.2x\n", id[0], id[1]);
-    nlength = buf[6];
-    ntmp    = nlength + ((((uint16_t)nlength) + 1) & 0x01);
-    offset += 4 + 2 + 1 + ntmp;
-
-    if (fmap_readn(map, &size, offset, 4) != 4) {
-        return -1;
-    }
-    size = special_endian_convert_32(size);
-    if (size == 0) {
-        return -1;
-    }
-    if ((size & 0x01) == 1) {
-        size++;
-    }
-
-    *off = offset + 4 + size;
-    /* Is it a thumbnail image: 0x0409 or 0x040c */
-    if ((id[0] == 0x04) && ((id[1] == 0x09) || (id[1] == 0x0c))) {
-        /* Yes */
-        cli_dbgmsg("found thumbnail\n");
-    } else {
-        /* No - Seek past record */
-        return 0;
-    }
-
-    /* Jump past header */
-    offset += 4 + 28;
-
-    retval = cli_check_jpeg_exploit(ctx, offset);
-    if (retval == 1) {
-        cli_dbgmsg("Exploit found in thumbnail\n");
-    }
-    return retval;
-}
-
-static int jpeg_check_photoshop(cli_ctx *ctx, off_t offset)
-{
-    int retval;
-    const unsigned char *buffer;
-    off_t old;
-    fmap_t *map = *ctx->fmap;
-
-    if (!(buffer = fmap_need_off_once(map, offset, 14))) {
-        return 0;
-    }
-
-    if (memcmp(buffer, "Photoshop 3.0", 14) != 0) {
-        return 0;
-    }
-    offset += 14;
-
-    cli_dbgmsg("Found Photoshop segment\n");
-    do {
-        old    = offset;
-        retval = jpeg_check_photoshop_8bim(ctx, &offset);
-        if (offset <= old)
-            break;
-    } while (retval == 0);
-
-    if (retval == -1) {
-        retval = 0;
-    }
-    return retval;
-}
-
-int cli_check_jpeg_exploit(cli_ctx *ctx, off_t offset)
-{
-    const unsigned char *buffer;
-    int retval;
-    fmap_t *map = *ctx->fmap;
-
-    cli_dbgmsg("in cli_check_jpeg_exploit()\n");
-    if (ctx->recursion > ctx->engine->maxreclevel)
-        return CL_EMAXREC;
-
-    if (!(buffer = fmap_need_off_once(map, offset, 2)))
-        return 0;
-    if ((buffer[0] != 0xff) || (buffer[1] != 0xd8)) {
-        return 0;
-    }
-    offset += 2;
-    for (;;) {
-        off_t new_off;
-        if (!(buffer = fmap_need_off_once(map, offset, 4))) {
-            return 0;
-        }
-        /* Check for multiple 0xFF values, we need to skip them */
-        if ((buffer[0] == 0xff) && (buffer[1] == 0xff)) {
-            offset++;
-            continue;
-        }
-        offset += 4;
-        if ((buffer[0] == 0xff) && (buffer[1] == 0xfe)) {
-            if (buffer[2] == 0x00) {
-                if ((buffer[3] == 0x00) || (buffer[3] == 0x01)) {
-                    return 1;
-                }
-            }
-        }
-        if (buffer[0] != 0xff) {
-            return -1;
-        }
-        if (buffer[1] == 0xda) {
-            /* End of Image marker */
-            return 0;
-        }
-
-        new_off = ((unsigned int)buffer[2] << 8) + buffer[3];
-        if (new_off < 2) {
-            return -1;
-        }
-        new_off -= 2;
-        new_off += offset;
-
-        if (buffer[1] == 0xed) {
-            /* Possible Photoshop file */
-            ctx->recursion++;
-            retval = jpeg_check_photoshop(ctx, offset);
-            ctx->recursion--;
-            if (retval != 0)
-                return retval;
-        }
-        offset = new_off;
-    }
-}
-
 static uint32_t riff_endian_convert_32(uint32_t value, int big_endian)
 {
     if (big_endian)
