@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *  Copyright (C) 2002-2007 Tomasz Kojm <tkojm@clamav.net>
  *
@@ -53,19 +53,22 @@
 #include <curl/curl.h>
 
 #include "target.h"
+
+// libclamav
 #include "clamav.h"
+#include "others.h"
+#include "regex_list.h"
+#include "str.h"
+
+// shared
+#include "cert_util.h"
+#include "output.h"
+#include "misc.h"
+
 #include "libfreshclam.h"
 #include "libfreshclam_internal.h"
 #include "dns.h"
 
-#include "shared/cert_util.h"
-#include "shared/output.h"
-#include "shared/misc.h"
-
-#include "libclamav/others.h"
-#include "libclamav/regex_list.h"
-#include "libclamav/str.h"
-
 /*
  * Private functions
  */
@@ -110,6 +113,10 @@ const char *fc_strerror(fc_error_t fcerror)
             return "Memory allocation error";
         case FC_EARG:
             return "Invalid argument(s)";
+        case FC_EFORBIDDEN:
+            return "Forbidden; Blocked by CDN";
+        case FC_ERETRYLATER:
+            return "Too many requests; Retry later";
         default:
             return "Unknown libfreshclam error code!";
     }
@@ -243,6 +250,17 @@ fc_error_t fc_initialize(fc_config *fcConfig)
 
     g_bCompressLocalDatabase = fcConfig->bCompressLocalDatabase;
 
+    /* Load or create mirrors.dat */
+    if (FC_SUCCESS != load_mirrors_dat()) {
+        logg("*Failed to load mirrors.dat; will create a new mirrors.dat\n");
+
+        if (FC_SUCCESS != new_mirrors_dat()) {
+            logg("^Failed to create a new mirrors.dat!\n");
+            status = FC_EINIT;
+            goto done;
+        }
+    }
+
     status = FC_SUCCESS;
 
 done:
@@ -290,6 +308,10 @@ void fc_cleanup(void)
         free(g_tempDirectory);
         g_tempDirectory = NULL;
     }
+    if (NULL != g_mirrorsDat) {
+        free(g_mirrorsDat);
+        g_mirrorsDat = NULL;
+    }
 }
 
 fc_error_t fc_prune_database_directory(char **databaseList, uint32_t nDatabases)
@@ -577,8 +599,8 @@ fc_error_t fc_update_database(
     fc_error_t status = FC_EARG;
 
     char *dbFilename = NULL;
-    int signo    = 0;
-    long attempt = 1;
+    int signo        = 0;
+    long attempt     = 1;
     uint32_t i;
 
     if ((NULL == database) || (NULL == serverList) || (NULL == bUpdated)) {
@@ -624,8 +646,7 @@ fc_error_t fc_update_database(
                 }
                 case FC_ECONNECTION:
                 case FC_EBADCVD:
-                case FC_EFAILEDGET:
-                case FC_EMIRRORNOTSYNC: {
+                case FC_EFAILEDGET: {
                     if (attempt < g_maxAttempts) {
                         logg("Trying again in 5 secs...\n");
                         sleep(5);
@@ -639,8 +660,52 @@ fc_error_t fc_update_database(
                     }
                     break;
                 }
+                case FC_EMIRRORNOTSYNC: {
+                    logg("Received an older %s CVD than was advertised. We'll retry so the incremental update will ensure we're up-to-date.\n", database);
+                    break;
+                }
+                case FC_EFORBIDDEN: {
+                    logg("^FreshClam received error code 403 from the ClamAV Content Delivery Network (CDN).\n");
+                    logg("This could mean several things:\n");
+                    logg(" 1. You are running an out-of-date version of ClamAV / FreshClam.\n");
+                    logg("    Ensure you are the most updated version by visiting https://www.clamav.net/downloads\n");
+                    logg(" 2. Your network is explicitly denied by the FreshClam CDN.\n");
+                    logg("    In order to rectify this please check that you are:\n");
+                    logg("   a. Running an up-to-date version of FreshClam\n");
+                    logg("   b. Running FreshClam no more than once an hour\n");
+                    logg("   c. If you have checked (a) and (b), please open a ticket at\n");
+                    logg("      https://bugzilla.clamav.net under the 'Mirrors' component\n");
+                    logg("      and we will investigate why your network is blocked.\n");
+                    status = ret;
+                    goto done;
+                    break;
+                }
+                case FC_ERETRYLATER: {
+                    char retry_after_string[26];
+                    struct tm *tm_info;
+                    tm_info = localtime(&g_mirrorsDat->retry_after);
+                    if (NULL == tm_info) {
+                        logg("!Failed to query the local time for the retry-after date!\n");
+                        status = FC_ERROR;
+                        goto done;
+                    }
+                    strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                    logg("^FreshClam received error code 429 from the ClamAV Content Delivery Network (CDN).\n");
+                    logg("This means that you have been rate limited by the CDN.\n");
+                    logg(" 1. Run FreshClam no more than once an hour to check for updates.\n");
+                    logg("    FreshClam should check DNS first to see if an update is needed.\n");
+                    logg(" 2. If you have more than 10 hosts on your network attempting to download,\n");
+                    logg("    it is recommended that you set up a private mirror on your network using\n");
+                    logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+                    logg("    CDN and your own network.\n");
+                    logg(" 3. Please do not open a ticket asking for an exemption from the rate limit,\n");
+                    logg("    it will not be granted.\n");
+                    logg("^You are on cool-down until after: %s\n", retry_after_string);
+                    goto success;
+                    break;
+                }
                 default: {
-                    logg("!Unexpected error when attempting to update database: %s\n", database);
+                    logg("!Unexpected error when attempting to update %s: %s\n", database, fc_strerror(ret));
                     status = ret;
                     goto done;
                 }
@@ -685,6 +750,38 @@ fc_error_t fc_update_databases(
 
     *nUpdated = 0;
 
+    if (g_mirrorsDat->retry_after > 0) {
+        if (g_mirrorsDat->retry_after > time(NULL)) {
+            /* We're on cool-down, try again later. */
+            char retry_after_string[26];
+            struct tm *tm_info;
+            tm_info = localtime(&g_mirrorsDat->retry_after);
+            if (NULL == tm_info) {
+                logg("!Failed to query the local time for the retry-after date!\n");
+                status = FC_ERROR;
+                goto done;
+            }
+            strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+            logg("^FreshClam previously received error code 429 from the ClamAV Content Delivery Network (CDN).\n");
+            logg("This means that you have been rate limited by the CDN.\n");
+            logg(" 1. Run FreshClam no more than once an hour to check for updates.\n");
+            logg("    FreshClam should check DNS first to see if an update is needed.\n");
+            logg(" 2. If you have more than 10 hosts on your network attempting to download,\n");
+            logg("    it is recommended that you set up a private mirror on your network using\n");
+            logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+            logg("    CDN and your own network.\n");
+            logg(" 3. Please do not open a ticket asking for an exemption from the rate limit,\n");
+            logg("    it will not be granted.\n");
+            logg("^You are still on cool-down until after: %s\n", retry_after_string);
+            status = FC_SUCCESS;
+            goto done;
+        } else {
+            g_mirrorsDat->retry_after = 0;
+            logg("^Cool-down expired, ok to try again.\n");
+            save_mirrors_dat();
+        }
+    }
+
     for (i = 0; i < nDatabases; i++) {
         if (FC_SUCCESS != (ret = fc_update_database(
                                databaseList[i],
@@ -695,7 +792,6 @@ fc_error_t fc_update_databases(
                                bScriptedUpdates,
                                context,
                                &bUpdated))) {
-            logg("^fc_update_databases: fc_update_database failed: %s (%d)\n", fc_strerror(ret), ret);
             status = ret;
             goto done;
         }
@@ -774,6 +870,46 @@ fc_error_t fc_download_url_database(
                 }
                 break;
             }
+            case FC_EFORBIDDEN: {
+                logg("^FreshClam received error code 403 from the ClamAV Content Delivery Network (CDN).\n");
+                logg("This could mean several things:\n");
+                logg(" 1. You are running an out-of-date version of ClamAV / FreshClam.\n");
+                logg("    Ensure you are the most updated version by visiting https://www.clamav.net/downloads\n");
+                logg(" 2. Your network is explicitly denied by the FreshClam CDN.\n");
+                logg("    In order to rectify this please check that you are:\n");
+                logg("   a. Running an up-to-date version of FreshClam\n");
+                logg("   b. Running FreshClam no more than once an hour\n");
+                logg("   c. If you have checked (a) and (b), please open a ticket at\n");
+                logg("      https://bugzilla.clamav.net under the 'Mirrors' component\n");
+                logg("      and we will investigate why your network is blocked.\n");
+                status = ret;
+                goto done;
+                break;
+            }
+            case FC_ERETRYLATER: {
+                char retry_after_string[26];
+                struct tm *tm_info;
+                tm_info = localtime(&g_mirrorsDat->retry_after);
+                if (NULL == tm_info) {
+                    logg("!Failed to query the local time for the retry-after date!\n");
+                    status = FC_ERROR;
+                    goto done;
+                }
+                strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+                logg("^FreshClam received error code 429 from the ClamAV Content Delivery Network (CDN).\n");
+                logg("This means that you have been rate limited by the CDN.\n");
+                logg(" 1. Run FreshClam no more than once an hour to check for updates.\n");
+                logg("    FreshClam should check DNS first to see if an update is needed.\n");
+                logg(" 2. If you have more than 10 hosts on your network attempting to download,\n");
+                logg("    it is recommended that you set up a private mirror on your network using\n");
+                logg("    cvdupdate (https://pypi.org/project/cvdupdate/) to save bandwidth on the\n");
+                logg("    CDN and your own network.\n");
+                logg(" 3. Please do not open a ticket asking for an exemption from the rate limit,\n");
+                logg("    it will not be granted.\n");
+                logg("^You are on cool-down until after: %s\n", retry_after_string);
+                goto success;
+                break;
+            }
             default: {
                 logg("Unexpected error when attempting to update from custom database URL: %s\n", urlDatabase);
                 status = ret;
