@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *  Copyright (C) 2002-2007 Tomasz Kojm <tkojm@clamav.net>
  *
@@ -65,31 +65,30 @@
 #include <zlib.h>
 #include <math.h>
 
-#if defined(C_DARWIN) || defined(_WIN32)
-#include <shared/cert_util.h>
-#endif
-
 #include <curl/curl.h>
+#include <openssl/rand.h>
 
 #include "target.h"
 
+// libclamav
+#include "clamav.h"
+#include "others.h"
+#include "str.h"
+#include "cvd.h"
+#include "regex_list.h"
+
+// shared
+#include "optparser.h"
+#include "output.h"
+#include "cdiff.h"
+#include "tar.h"
+#include "clamdcom.h"
+#include "cert_util.h"
+
 #include "libfreshclam.h"
 #include "libfreshclam_internal.h"
 #include "dns.h"
 
-#include "shared/optparser.h"
-#include "shared/output.h"
-#include "shared/cdiff.h"
-#include "shared/tar.h"
-#include "shared/clamdcom.h"
-#include "shared/cert_util.h"
-
-#include "libclamav/clamav.h"
-#include "libclamav/others.h"
-#include "libclamav/str.h"
-#include "libclamav/cvd.h"
-#include "libclamav/regex_list.h"
-
 #define DB_FILENAME_MAX 60
 #define CVD_HEADER_SIZE 512
 
@@ -117,6 +116,263 @@ uint32_t g_requestTimeout = 0;
 
 uint32_t g_bCompressLocalDatabase = 0;
 
+mirrors_dat_v1_t *g_mirrorsDat = NULL;
+
+/** @brief Generate a Version 4 UUID according to RFC-4122
+ *
+ * Uses the openssl RAND_bytes function to generate a Version 4 UUID.
+ *
+ * Copyright 2021 Karthik Velakur with some modifications by the ClamAV team.
+ * License: MIT
+ * From: https://gist.github.com/kvelakur/9069c9896577c3040030
+ *
+ * @param buffer A buffer that is SIZEOF_UUID_V4
+ */
+static void uuid_v4_gen(char *buffer)
+{
+    union {
+        struct
+        {
+            uint32_t time_low;
+            uint16_t time_mid;
+            uint16_t time_hi_and_version;
+            uint8_t clk_seq_hi_res;
+            uint8_t clk_seq_low;
+            uint8_t node[6];
+        };
+        uint8_t __rnd[16];
+    } uuid;
+
+    if (0 >= RAND_bytes(uuid.__rnd, sizeof(uuid.__rnd))) {
+        /* Failed to generate random bytes for new UUID */
+        memset(uuid.__rnd, 0, sizeof(uuid.__rnd));
+        uuid.time_low = (uint32_t)time(NULL);
+    }
+
+    // Refer Section 4.2 of RFC-4122
+    // https://tools.ietf.org/html/rfc4122#section-4.2
+    uuid.clk_seq_hi_res      = (uint8_t)((uuid.clk_seq_hi_res & 0x3F) | 0x80);
+    uuid.time_hi_and_version = (uint16_t)((uuid.time_hi_and_version & 0x0FFF) | 0x4000);
+
+    snprintf(buffer, SIZEOF_UUID_V4, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
+             uuid.time_low, uuid.time_mid, uuid.time_hi_and_version,
+             uuid.clk_seq_hi_res, uuid.clk_seq_low,
+             uuid.node[0], uuid.node[1], uuid.node[2],
+             uuid.node[3], uuid.node[4], uuid.node[5]);
+    buffer[SIZEOF_UUID_V4 - 1] = 0;
+
+    return;
+}
+
+fc_error_t load_mirrors_dat(void)
+{
+    fc_error_t status      = FC_EINIT;
+    int handle             = -1;
+    ssize_t bread          = 0;
+    mirrors_dat_v1_t *mdat = NULL;
+    uint32_t version       = 0;
+    char magic[13]         = {0};
+
+    /* Change directory to database directory */
+    if (chdir(g_databaseDirectory)) {
+        logg("!Can't change dir to %s\n", g_databaseDirectory);
+        status = FC_EDIRECTORY;
+        goto done;
+    }
+    logg("*Current working dir is %s\n", g_databaseDirectory);
+
+    if (-1 == (handle = open("mirrors.dat", O_RDONLY | O_BINARY))) {
+        char currdir[PATH_MAX];
+
+        if (getcwd(currdir, sizeof(currdir)))
+            logg("*Can't open mirrors.dat in %s\n", currdir);
+        else
+            logg("*Can't open mirrors.dat in the current directory\n");
+
+        logg("*It probably doesn't exist yet. That's ok.\n");
+        status = FC_EFILE;
+        goto done;
+    }
+
+    if (strlen(MIRRORS_DAT_MAGIC) != (bread = read(handle, &magic, strlen(MIRRORS_DAT_MAGIC)))) {
+        char error_message[260];
+        cli_strerror(errno, error_message, 260);
+        logg("!Can't read magic from mirrors.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+        goto done;
+    }
+    if (0 != strncmp(magic, MIRRORS_DAT_MAGIC, strlen(MIRRORS_DAT_MAGIC))) {
+        logg("*Magic bytes for mirrors.dat did not match expectations.\n");
+        goto done;
+    }
+
+    if (sizeof(uint32_t) != (bread = read(handle, &version, sizeof(uint32_t)))) {
+        char error_message[260];
+        cli_strerror(errno, error_message, 260);
+        logg("!Can't read version from mirrors.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+        goto done;
+    }
+
+    switch (version) {
+        case 1: {
+            /* Verify that file size is as expected. */
+            off_t file_size = lseek(handle, 0L, SEEK_END);
+
+            if (strlen(MIRRORS_DAT_MAGIC) + sizeof(mirrors_dat_v1_t) != (size_t)file_size) {
+                logg("*mirrors.dat is bigger than expected: %zu != %ld\n", sizeof(mirrors_dat_v1_t), file_size);
+                goto done;
+            }
+
+            /* Rewind to just after the magic bytes and read data struct */
+            lseek(handle, strlen(MIRRORS_DAT_MAGIC), SEEK_SET);
+
+            mdat = malloc(sizeof(mirrors_dat_v1_t));
+            if (NULL == mdat) {
+                logg("!Failed to allocate memory for mirrors.dat\n");
+                status = FC_EMEM;
+                goto done;
+            }
+
+            if (sizeof(mirrors_dat_v1_t) != (bread = read(handle, mdat, sizeof(mirrors_dat_v1_t)))) {
+                char error_message[260];
+                cli_strerror(errno, error_message, 260);
+                logg("!Can't read from mirrors.dat. Bytes read: %zi, error: %s\n", bread, error_message);
+                goto done;
+            }
+
+            /* Got it. */
+            close(handle);
+            handle = -1;
+
+            /* This is the latest version.
+               If we change the format in the future, we may wish to create a new
+               mirrors dat struct, import the relevant bits to the new format,
+               and then save (overwrite) mirrors.dat with the new data. */
+            if (NULL != g_mirrorsDat) {
+                free(g_mirrorsDat);
+            }
+            g_mirrorsDat = mdat;
+            mdat         = NULL;
+            break;
+        }
+        default: {
+            logg("*mirrors.dat version is different than expected: %u != %u\n", 1, version);
+            goto done;
+        }
+    }
+
+    logg("*Loaded mirrors.dat:\n");
+    logg("*  version:    %d\n", g_mirrorsDat->version);
+    logg("*  uuid:       %s\n", g_mirrorsDat->uuid);
+    if (g_mirrorsDat->retry_after > 0) {
+        char retry_after_string[26];
+        struct tm *tm_info = localtime(&g_mirrorsDat->retry_after);
+        if (NULL == tm_info) {
+            logg("!Failed to query the local time for the retry-after date!\n");
+            goto done;
+        }
+        strftime(retry_after_string, 26, "%Y-%m-%d %H:%M:%S", tm_info);
+        logg("*  retry-after: %s\n", retry_after_string);
+    }
+
+    status = FC_SUCCESS;
+
+done:
+    if (-1 != handle) {
+        close(handle);
+    }
+    if (FC_SUCCESS != status) {
+        if (NULL != mdat) {
+            free(mdat);
+        }
+        if (NULL != g_mirrorsDat) {
+            free(g_mirrorsDat);
+            g_mirrorsDat = NULL;
+        }
+    }
+
+    return status;
+}
+
+fc_error_t save_mirrors_dat(void)
+{
+    fc_error_t status = FC_EINIT;
+    int handle        = -1;
+
+    if (NULL == g_mirrorsDat) {
+        logg("!Attempted to save mirrors data to mirrors.dat before initializing it!\n");
+        goto done;
+    }
+
+    if (-1 == (handle = open("mirrors.dat", O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0644))) {
+        char currdir[PATH_MAX];
+
+        if (getcwd(currdir, sizeof(currdir)))
+            logg("!Can't create mirrors.dat in %s\n", currdir);
+        else
+            logg("!Can't create mirrors.dat in the current directory\n");
+
+        logg("Hint: The database directory must be writable for UID %d or GID %d\n", getuid(), getgid());
+        status = FC_EDBDIRACCESS;
+        goto done;
+    }
+    if (-1 == write(handle, MIRRORS_DAT_MAGIC, strlen(MIRRORS_DAT_MAGIC))) {
+        logg("!Can't write to mirrors.dat\n");
+    }
+    if (-1 == write(handle, g_mirrorsDat, sizeof(mirrors_dat_v1_t))) {
+        logg("!Can't write to mirrors.dat\n");
+    }
+
+    logg("*Saved mirrors.dat\n");
+
+    status = FC_SUCCESS;
+done:
+    if (-1 != handle) {
+        close(handle);
+    }
+
+    return status;
+}
+
+fc_error_t new_mirrors_dat(void)
+{
+    fc_error_t status = FC_EINIT;
+
+    mirrors_dat_v1_t *mdat = calloc(1, sizeof(mirrors_dat_v1_t));
+    if (NULL == mdat) {
+        logg("!Failed to allocate memory for mirrors.dat\n");
+        status = FC_EMEM;
+        goto done;
+    }
+
+    mdat->version     = 1;
+    mdat->retry_after = 0;
+    uuid_v4_gen(mdat->uuid);
+
+    if (NULL != g_mirrorsDat) {
+        free(g_mirrorsDat);
+    }
+    g_mirrorsDat = mdat;
+
+    logg("*Creating new mirrors.dat\n");
+
+    if (FC_SUCCESS != save_mirrors_dat()) {
+        logg("!Failed to save mirrors.dat!\n");
+        status = FC_EFILE;
+        goto done;
+    }
+
+    status = FC_SUCCESS;
+
+done:
+    if (FC_SUCCESS != status) {
+        if (NULL != mdat) {
+            free(mdat);
+        }
+        g_mirrorsDat = NULL;
+    }
+    return status;
+}
+
 /**
  * @brief Get DNS text record field # for official databases.
  *
@@ -137,7 +393,7 @@ static int textrecordfield(const char *database)
     return 0;
 }
 
-#if LIBCURL_VERSION_NUM >= 0x073d00
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 61))
 /* In libcurl 7.61.0, support was added for extracting the time in plain
    microseconds. Older libcurl versions are stuck in using 'double' for this
    information so we complicate this example a bit by supporting either
@@ -160,16 +416,30 @@ struct xfer_progress {
     CURL *curl;
 };
 
-static void printBytes(curl_off_t bytes)
+static void printTime(double seconds)
 {
-    if (bytes / (1024 * 1024) > 1) {
-        double megabytes = bytes / (double)(1024 * 1024);
-        fprintf(stdout, "%.02fMiB", megabytes);
-    } else if (bytes / 1024 > 1) {
-        double kilobytes = bytes / (double)(1024);
-        fprintf(stdout, "%.02fKiB", kilobytes);
+    if (seconds >= 3600) {
+        fprintf(stdout, "%2.0fh %02.0fm", trunc(seconds / 3600), trunc(fmod(seconds, 3600.0) / 60));
+    } else if (seconds >= 60) {
+        fprintf(stdout, "%2.0fm %02.0fs", trunc(seconds / 60), trunc(fmod(seconds, 60.0)));
     } else {
-        fprintf(stdout, "%" CURL_FORMAT_CURL_OFF_T "B", bytes);
+        fprintf(stdout, "%6.1fs", seconds);
+    }
+}
+
+static void printBytes(curl_off_t bytes, int bPad)
+{
+    if (bytes >= (1024 * 1024)) {
+        const char *format = bPad ? "%7.02fMiB" : "%.02fMiB";
+        double megabytes   = bytes / (double)(1024 * 1024);
+        fprintf(stdout, format, megabytes);
+    } else if (bytes >= 1024) {
+        const char *format = bPad ? "%7.02fKiB" : "%.02fKiB";
+        double kilobytes   = bytes / (double)(1024);
+        fprintf(stdout, format, kilobytes);
+    } else {
+        const char *format = bPad ? "%9" CURL_FORMAT_CURL_OFF_T "B" : "%" CURL_FORMAT_CURL_OFF_T "B";
+        fprintf(stdout, format, bytes);
     }
 }
 
@@ -187,10 +457,13 @@ static int xferinfo(void *prog,
     TIMETYPE remtime               = 0;
 
     uint32_t i                = 0;
-    uint32_t totalNumDots     = 30;
+    uint32_t totalNumDots     = 25;
     uint32_t numDots          = 0;
     double fractiondownloaded = 0.0;
 
+    UNUSEDPARAM(TotalToUpload);
+    UNUSEDPARAM(NowUploaded);
+
     if ((TotalToDownload <= 0.0) || (xferProg->bComplete)) {
         return 0;
     }
@@ -201,22 +474,35 @@ static int xferinfo(void *prog,
     curl_easy_getinfo(curl, TIMEOPT, &curtime);
 
     xferProg->lastRunTime = curtime;
-    remtime               = (curtime * 1 / fractiondownloaded) - curtime;
 
 #ifndef _WIN32
     fprintf(stdout, "\e[?7l");
 #endif
 #ifdef TIME_IN_US
     if (fractiondownloaded <= 0.0) {
-        fprintf(stdout, "Time: %.1fs ", curtime / 1000000.0);
+        fprintf(stdout, "Time: ");
+        printTime(curtime / 1000000.0);
+        fprintf(stdout, "               ");
     } else {
-        fprintf(stdout, "Time: %.1fs, ETA: %.1fs ", curtime / 1000000.0, remtime / 1000000.0);
+        remtime = (curtime / fractiondownloaded) - curtime;
+        fprintf(stdout, "Time: ");
+        printTime(curtime / 1000000.0);
+        fprintf(stdout, ", ETA: ");
+        printTime(remtime / 1000000.0);
+        fprintf(stdout, " ");
     }
 #else
     if (fractiondownloaded <= 0.0) {
-        fprintf(stdout, "Time: %.1fs ", curtime);
+        fprintf(stdout, "Time: ");
+        printTime(curtime);
+        fprintf(stdout, "               ");
     } else {
-        fprintf(stdout, "Time: %.1fs, ETA: %.1fs ", curtime, remtime);
+        remtime = (curtime / fractiondownloaded) - curtime;
+        fprintf(stdout, "Time: ");
+        printTime(curtime);
+        fprintf(stdout, ", ETA: ");
+        printTime(remtime);
+        fprintf(stdout, " ");
     }
 #endif
 
@@ -226,30 +512,24 @@ static int xferinfo(void *prog,
             for (i = 0; i < numDots - 1; i++) {
                 fprintf(stdout, "=");
             }
-            fprintf(stdout, ">");
-            i++;
         }
+        fprintf(stdout, ">");
+        i++;
     }
     for (; i < totalNumDots; i++) {
-        printf(" ");
+        fprintf(stdout, " ");
     }
 
     fprintf(stdout, "] ");
 
-    if (TotalToUpload > 0.0) {
-        printBytes(NowUploaded);
-        fprintf(stdout, "/");
-        printBytes(TotalToUpload);
-    } else if (TotalToDownload > 0.0) {
-        printBytes(NowDownloaded);
-        fprintf(stdout, "/");
-        printBytes(TotalToDownload);
-    }
+    printBytes(NowDownloaded, 1);
+    fprintf(stdout, "/");
+    printBytes(TotalToDownload, 0);
 
     if (NowDownloaded < TotalToDownload) {
-        fprintf(stdout, "     \r");
+        fprintf(stdout, "\r");
     } else {
-        fprintf(stdout, "     \n");
+        fprintf(stdout, "\n");
         xferProg->bComplete = 1;
     }
 #ifndef _WIN32
@@ -260,7 +540,7 @@ static int xferinfo(void *prog,
     return 0;
 }
 
-#if LIBCURL_VERSION_NUM < 0x072000
+#if (LIBCURL_VERSION_MAJOR < 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR < 32))
 /**
  * Function from curl example code, Copyright (C) 1998 - 2018, Daniel Stenberg, see COPYING.curl for license details
  * Older style progress bar callback shim; for libcurl older than 7.32.0 ( CURLOPT_PROGRESSFUNCTION ).
@@ -306,12 +586,19 @@ static fc_error_t create_curl_handle(
         goto done;
     }
 
-    if (g_userAgent)
+    if (g_userAgent) {
         strncpy(userAgent, g_userAgent, sizeof(userAgent));
-    else
+    } else {
+        /*
+         * Use a randomly generated UUID in the User-Agent
+         * We'll try to load it from a file in the database directory.
+         * If none exists, we'll create a new one and save it to said file.
+         */
         snprintf(userAgent, sizeof(userAgent),
-                 PACKAGE "/%s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ")",
-                 get_version());
+                 PACKAGE "/%s (OS: " TARGET_OS_TYPE ", ARCH: " TARGET_ARCH_TYPE ", CPU: " TARGET_CPU_TYPE ", UUID: %s)",
+                 get_version(),
+                 g_mirrorsDat->uuid);
+    }
     userAgent[sizeof(userAgent) - 1] = 0;
 
     if (mprintf_verbose) {
@@ -426,6 +713,8 @@ static fc_error_t create_curl_handle(
     if (CURLE_OK != curl_easy_setopt(curl, CURLOPT_SSL_CTX_FUNCTION, *sslctx_function)) {
         logg("*create_curl_handle: Failed to set SSL CTX function. Your libcurl may use an SSL backend that does not support CURLOPT_SSL_CTX_FUNCTION.\n");
     }
+#else
+    set_tls_ca_bundle(curl);
 #endif
 
     *curlHandle = curl;
@@ -551,13 +840,11 @@ static fc_error_t remote_cvdhead(
     /*
      * Request CVD header.
      */
-    logg("Reading CVD header (%s): ", cvdfile);
-
     urlLen = strlen(server) + strlen("/") + strlen(cvdfile);
     url    = malloc(urlLen + 1);
     snprintf(url, urlLen + 1, "%s/%s", server, cvdfile);
 
-    logg("*Trying to retrieve CVD header from %s\n", url);
+    logg("Trying to retrieve CVD header from %s\n", url);
 
     if (FC_SUCCESS != (ret = create_curl_handle(
                            bHttpServer, // Set extra HTTP-specific headers.
@@ -578,7 +865,7 @@ static fc_error_t remote_cvdhead(
         prog.curl        = curl;
         prog.bComplete   = 0;
 
-#if LIBCURL_VERSION_NUM >= 0x072000
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 32))
         /* xferinfo was introduced in 7.32.0, no earlier libcurl versions will
        compile as they won't have the symbols around.
 
@@ -708,6 +995,34 @@ static fc_error_t remote_cvdhead(
             status = FC_UPTODATE;
             goto done;
         }
+        case 403: {
+            status = FC_EFORBIDDEN;
+            break;
+        }
+        case 429: {
+            status = FC_ERETRYLATER;
+
+            curl_off_t retry_after = 0;
+
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 66))
+            /* CURLINFO_RETRY_AFTER was introduced in libcurl 7.66 */
+
+            /* Find out how long we should wait before allowing a retry. */
+            curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
+#endif
+
+            if (retry_after > 0) {
+                /* The response gave us a Retry-After date. Use that. */
+                g_mirrorsDat->retry_after = time(NULL) + (time_t)retry_after;
+            } else {
+                /* Try again in no less than 4 hours if the response didn't specify
+                   or if CURLINFO_RETRY_AFTER is not supported. */
+                g_mirrorsDat->retry_after = time(NULL) + 60 * 60 * 4;
+            }
+            (void)save_mirrors_dat();
+
+            break;
+        }
         case 404: {
             if (g_proxyServer)
                 logg("^remote_cvdhead: file not found: %s (Proxy: %s:%u)\n", url, g_proxyServer, g_proxyPort);
@@ -841,7 +1156,7 @@ static fc_error_t downloadFile(
         prog.curl        = curl;
         prog.bComplete   = 0;
 
-#if LIBCURL_VERSION_NUM >= 0x072000
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 32))
         /* xferinfo was introduced in 7.32.0, no earlier libcurl versions will
        compile as they won't have the symbols around.
 
@@ -976,6 +1291,34 @@ static fc_error_t downloadFile(
             status = FC_UPTODATE;
             break;
         }
+        case 403: {
+            status = FC_EFORBIDDEN;
+            break;
+        }
+        case 429: {
+            status = FC_ERETRYLATER;
+
+            curl_off_t retry_after = 0;
+
+#if (LIBCURL_VERSION_MAJOR > 7) || ((LIBCURL_VERSION_MAJOR == 7) && (LIBCURL_VERSION_MINOR >= 66))
+            /* CURLINFO_RETRY_AFTER was introduced in libcurl 7.66 */
+
+            /* Find out how long we should wait before allowing a retry. */
+            curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
+#endif
+
+            if (retry_after > 0) {
+                /* The response gave us a Retry-After date. Use that. */
+                g_mirrorsDat->retry_after = time(NULL) + (time_t)retry_after;
+            } else {
+                /* Try again in no less than 4 hours if the response didn't specify
+                   or if CURLINFO_RETRY_AFTER is not supported. */
+                g_mirrorsDat->retry_after = time(NULL) + 60 * 60 * 4;
+            }
+            (void)save_mirrors_dat();
+
+            break;
+        }
         case 404: {
             if (g_proxyServer)
                 logg("^downloadFile: file not found: %s (Proxy: %s:%u)\n", url, g_proxyServer, g_proxyPort);
@@ -1026,10 +1369,12 @@ static fc_error_t getcvd(
     const char *cvdfile,
     const char *tmpfile,
     char *server,
+    uint32_t ifModifiedSince,
     unsigned int remoteVersion,
     int logerr)
 {
     fc_error_t ret;
+    cl_error_t cl_ret;
     fc_error_t status = FC_EARG;
 
     struct cl_cvd *cvd           = NULL;
@@ -1046,8 +1391,13 @@ static fc_error_t getcvd(
     url    = malloc(urlLen + 1);
     snprintf(url, urlLen + 1, "%s/%s", server, cvdfile);
 
-    if (FC_SUCCESS != (ret = downloadFile(url, tmpfile, 1, logerr, 0))) {
-        logg("%cgetcvd: Can't download %s from %s\n", logerr ? '!' : '^', cvdfile, url);
+    ret = downloadFile(url, tmpfile, 1, logerr, ifModifiedSince);
+    if (ret == FC_UPTODATE) {
+        logg("%s is up-to-date.\n", cvdfile);
+        status = ret;
+        goto done;
+    } else if (ret > FC_UPTODATE) {
+        logg("%cCan't download %s from %s\n", logerr ? '!' : '^', cvdfile, url);
         status = ret;
         goto done;
     }
@@ -1055,56 +1405,42 @@ static fc_error_t getcvd(
     /* Temporarily rename file to correct extension for verification. */
     tmpfile_with_extension = strdup(tmpfile);
     if (!tmpfile_with_extension) {
-        logg("!getcvd: Can't allocate memory for temp file with extension!\n");
+        logg("!Can't allocate memory for temp file with extension!\n");
         status = FC_EMEM;
         goto done;
     }
     strncpy(tmpfile_with_extension + strlen(tmpfile_with_extension) - 4, cvdfile + strlen(cvdfile) - 4, 4);
     if (rename(tmpfile, tmpfile_with_extension) == -1) {
-        logg("!getcvd: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+        logg("!Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
         status = FC_EDBDIRACCESS;
         goto done;
     }
 
-    if ((ret = cl_cvdverify(tmpfile_with_extension))) {
-        logg("!getcvd: Verification: %s\n", cl_strerror(ret));
+    if (CL_SUCCESS != (cl_ret = cl_cvdverify(tmpfile_with_extension))) {
+        logg("!Verification: %s\n", cl_strerror(cl_ret));
         status = FC_EBADCVD;
         goto done;
     }
 
-    if (!(cvd = cl_cvdhead(tmpfile_with_extension))) {
-        logg("!getcvd: Can't read CVD header of new %s database.\n", cvdfile);
+    if (NULL == (cvd = cl_cvdhead(tmpfile_with_extension))) {
+        logg("!Can't read CVD header of new %s database.\n", cvdfile);
         status = FC_EBADCVD;
         goto done;
     }
 
     /* Rename the file back to the original, since verification passed. */
     if (rename(tmpfile_with_extension, tmpfile) == -1) {
-        logg("!getcvd: Can't rename %s to %s: %s\n", tmpfile_with_extension, tmpfile, strerror(errno));
+        logg("!Can't rename %s to %s: %s\n", tmpfile_with_extension, tmpfile, strerror(errno));
         status = FC_EDBDIRACCESS;
         goto done;
     }
 
     if (cvd->version < remoteVersion) {
-        if (cvd->version == remoteVersion - 1) {
-            logg("*The %s database downloaded from %s is one version older than advertised in the DNS TXT record.\n",
-                cvdfile,
-                server);
-
-            /*
-             * Tolerate an off-by-one version mismatch.
-             * Chances are the new version was just published and the CDN is still updating.
-             */
-            status = FC_SUCCESS;
-            goto done;
-        }
-        else {
-            logg("!The %s database downloaded from %s is more than one version older than the version advertised in the DNS TXT record.\n",
-                cvdfile,
-                server);
-            status = FC_EMIRRORNOTSYNC;
-            goto done;
-        }
+        logg("*The %s database downloaded from %s is older than the version advertised in the DNS TXT record.\n",
+             cvdfile,
+             server);
+        status = FC_EMIRRORNOTSYNC;
+        goto done;
     }
 
     status = FC_SUCCESS;
@@ -1120,7 +1456,10 @@ done:
     if (NULL != url) {
         free(url);
     }
-    if (FC_SUCCESS != status) {
+    if (
+        (FC_SUCCESS != status) &&
+        (FC_EMIRRORNOTSYNC != status) /* Keep older version, it's better than nothing. */
+    ) {
         if (NULL != tmpfile) {
             unlink(tmpfile);
         }
@@ -1257,7 +1596,7 @@ static fc_error_t downloadPatch(
         if (ret == FC_EEMPTYFILE) {
             logg("Empty script %s, need to download entire database\n", patch);
         } else {
-            logg("%cgetpatch: Can't download %s from %s\n", logerr ? '!' : '^', patch, url);
+            logg("%cdownloadPatch: Can't download %s from %s\n", logerr ? '!' : '^', patch, url);
         }
         status = ret;
         goto done;
@@ -1354,7 +1693,7 @@ static fc_error_t buildcld(
 {
     fc_error_t status = FC_EARG;
 
-    char olddir[PATH_MAX];
+    char olddir[PATH_MAX] = {0};
     char info[DB_FILENAME_MAX];
     char buff[CVD_HEADER_SIZE + 1];
     char *pt;
@@ -1695,7 +2034,8 @@ static fc_error_t check_for_new_database_version(
     uint32_t *localVersion,
     uint32_t *remoteVersion,
     char **localFilename,
-    char **remoteFilename)
+    char **remoteFilename,
+    uint32_t *localTimestamp)
 {
     fc_error_t ret;
     fc_error_t status = FC_EARG;
@@ -1704,13 +2044,13 @@ static fc_error_t check_for_new_database_version(
     struct cl_cvd *local_database = NULL;
     char *remotename              = NULL;
 
-    uint32_t localver       = 0;
-    uint32_t localTimestamp = 0;
-    uint32_t remotever      = 0;
+    uint32_t localver  = 0;
+    uint32_t remotever = 0;
 
     if ((NULL == database) || (NULL == server) ||
         (NULL == localVersion) || (NULL == remoteVersion) ||
-        (NULL == localFilename) || (NULL == remoteFilename)) {
+        (NULL == localFilename) || (NULL == remoteFilename) ||
+        (NULL == localTimestamp)) {
         logg("!check_for_new_database_version: Invalid args!\n");
         goto done;
     }
@@ -1719,6 +2059,7 @@ static fc_error_t check_for_new_database_version(
     *remoteVersion  = 0;
     *localFilename  = NULL;
     *remoteFilename = NULL;
+    *localTimestamp = 0;
 
     /*
      * Check local database version (if exists)
@@ -1727,8 +2068,8 @@ static fc_error_t check_for_new_database_version(
         logg("*check_for_new_database_version: No local copy of \"%s\" database.\n", database);
     } else {
         logg("*check_for_new_database_version: Local copy of %s found: %s.\n", database, localname);
-        localTimestamp = local_database->stime;
-        localver       = local_database->version;
+        *localTimestamp = local_database->stime;
+        localver        = local_database->version;
     }
 
     /*
@@ -1736,7 +2077,7 @@ static fc_error_t check_for_new_database_version(
      */
     ret = query_remote_database_version(
         database,
-        localTimestamp,
+        *localTimestamp,
         dnsUpdateInfo,
         server,
         bPrivateMirror,
@@ -1754,15 +2095,15 @@ static fc_error_t check_for_new_database_version(
                      database, localver, remotever);
                 break;
             }
-            // else: Fall-through to Up-to-date case.
+            /* fall-through */
         }
         case FC_UPTODATE: {
             if (NULL == local_database) {
-                logg("!check_for_new_database_version: server claims we're up to date, but we don't have a local database!\n");
+                logg("!check_for_new_database_version: server claims we're up-to-date, but we don't have a local database!\n");
                 status = FC_EFAILEDGET;
                 goto done;
             }
-            logg("%s database is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n",
+            logg("%s database is up-to-date (version: %d, sigs: %d, f-level: %d, builder: %s)\n",
                  localname,
                  local_database->version,
                  local_database->sigs,
@@ -1774,6 +2115,12 @@ static fc_error_t check_for_new_database_version(
             remotever = localver;
             break;
         }
+        case FC_EFORBIDDEN: {
+            /* We tried to look up the version using HTTP and were actively blocked. */
+            logg("!check_for_new_database_version: Blocked from using server %s.\n", server);
+            status = FC_EFORBIDDEN;
+            goto done;
+        }
         default: {
             logg("!check_for_new_database_version: Failed to find %s database using server %s.\n", database, server);
             status = FC_EFAILEDGET;
@@ -1834,11 +2181,12 @@ fc_error_t updatedb(
 
     struct cl_cvd *cvd = NULL;
 
-    uint32_t localVersion  = 0;
-    uint32_t remoteVersion = 0;
-    char *localFilename    = NULL;
-    char *remoteFilename   = NULL;
-    char *newLocalFilename = NULL;
+    uint32_t localTimestamp = 0;
+    uint32_t localVersion   = 0;
+    uint32_t remoteVersion  = 0;
+    char *localFilename     = NULL;
+    char *remoteFilename    = NULL;
+    char *newLocalFilename  = NULL;
 
     char *tmpdir  = NULL;
     char *tmpfile = NULL;
@@ -1868,7 +2216,8 @@ fc_error_t updatedb(
                            &localVersion,
                            &remoteVersion,
                            &localFilename,
-                           &remoteFilename))) {
+                           &remoteFilename,
+                           &localTimestamp))) {
         logg("*updatedb: %s database update failed.\n", database);
         status = ret;
         goto done;
@@ -1890,8 +2239,21 @@ fc_error_t updatedb(
         /*
          * Download entire file.
          */
-        ret = getcvd(remoteFilename, tmpfile, server, remoteVersion, logerr);
-        if (FC_SUCCESS != ret) {
+        ret = getcvd(remoteFilename, tmpfile, server, localTimestamp, remoteVersion, logerr);
+        if (FC_UPTODATE == ret) {
+            logg("^Expected newer version of %s database but the server's copy is not newer than our local file (version %d).\n", database, localVersion);
+            if (NULL != localFilename) {
+                /* Received a 304 (not modified), must be up-to-date after all */
+                *dbFilename = cli_strdup(localFilename);
+            }
+            goto up_to_date;
+        } else if (FC_EMIRRORNOTSYNC == ret) {
+            /* Let's accept this older version, but keep the error code.
+             * We'll have fc_update_database() retry using CDIFFs.
+             */
+            logg("*Received an older %s CVD than was advertised. We'll keep it and try updating to the latest version with CDIFFs.\n", database);
+            status = ret;
+        } else if (FC_SUCCESS != ret) {
             status = ret;
             goto done;
         }
@@ -1901,7 +2263,8 @@ fc_error_t updatedb(
         /*
          * Attempt scripted/CDIFF incremental update.
          */
-        ret = FC_SUCCESS;
+        ret                         = FC_SUCCESS;
+        uint32_t numPatchesReceived = 0;
 
         tmpdir = cli_gentemp(g_tempDirectory);
         if (!tmpdir) {
@@ -1942,11 +2305,19 @@ fc_error_t updatedb(
                     break;
                 }
             }
-            if (FC_SUCCESS != ret)
+            if (FC_SUCCESS == ret) {
+                numPatchesReceived += 1;
+            } else {
                 break;
+            }
         }
 
-        if (FC_SUCCESS != ret) {
+        if (
+            (FC_EEMPTYFILE == ret) ||                                 /* Request a new CVD if we got an empty CDIFF.      */
+            (FC_SUCCESS != ret && (                                   /* Or if the incremental update failed:             */
+                                   (0 == numPatchesReceived) &&       /* 1. Ask for the CVD if we didn't get any patches, */
+                                   (localVersion < remoteVersion - 1) /* 2. AND if we're more than 1 version out of date. */
+                                   ))) {
             /*
              * Incremental update failed or intentionally disabled.
              */
@@ -1956,17 +2327,34 @@ fc_error_t updatedb(
                 logg("^Incremental update failed, trying to download %s\n", remoteFilename);
             }
 
-            ret = getcvd(remoteFilename, tmpfile, server, remoteVersion, logerr);
+            ret = getcvd(remoteFilename, tmpfile, server, localTimestamp, remoteVersion, logerr);
             if (FC_SUCCESS != ret) {
-                status = ret;
-                goto done;
+                if (FC_EMIRRORNOTSYNC == status) {
+                    /* Note: We can't retry with CDIFF's if FC_EMIRRORNOTSYNC happened here.
+                     * If we did there could be an infinite loop.
+                     * Best option is to accept the older CVD.
+                     */
+                    logg("^Received an older %s CVD than was advertised. Incremental updates either failed or are disabled, so we'll have to settle for a slightly out-of-date database.\n", database);
+                    status = FC_SUCCESS;
+                } else {
+                    status = ret;
+                    goto done;
+                }
             }
 
             newLocalFilename = cli_strdup(remoteFilename);
+        } else if (0 == numPatchesReceived) {
+            logg("The database server doesn't have the latest patch for the %s database (version %u). The server will likely have updated if you check again in a few hours.\n", database, remoteVersion);
+            goto up_to_date;
         } else {
             /*
              * CDIFFs downloaded; Use CDIFFs to turn old CVD/CLD into new updated CLD.
              */
+            if (numPatchesReceived < remoteVersion - localVersion) {
+                logg("Downloaded %u patches for %s, which is fewer than the %u expected patches.\n", numPatchesReceived, database, remoteVersion - localVersion);
+                logg("We'll settle for this partial-update, at least for now.\n");
+            }
+
             size_t newLocalFilenameLen = 0;
             if (FC_SUCCESS != buildcld(tmpdir, database, tmpfile, g_bCompressLocalDatabase)) {
                 logg("!updatedb: Incremental update failed. Failed to build CLD.\n");
@@ -1996,9 +2384,8 @@ fc_error_t updatedb(
         }
         snprintf(tmpfile_with_extension, tmpfile_with_extension_len + 1, "%s-%s", tmpfile, newLocalFilename);
         if (rename(tmpfile, tmpfile_with_extension) == -1) {
-            free(tmpfile_with_extension);
-
             logg("!updatedb: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+            free(tmpfile_with_extension);
             status = FC_EDBDIRACCESS;
             goto done;
         }
@@ -2064,7 +2451,9 @@ fc_error_t updatedb(
 
 up_to_date:
 
-    status = FC_SUCCESS;
+    if (status != FC_EMIRRORNOTSYNC) {
+        status = FC_SUCCESS;
+    }
 
 done:
 
@@ -2153,7 +2542,7 @@ fc_error_t updatecustomdb(
         remote_dbtime = statbuf.st_mtime;
         dbtime        = (CLAMSTAT(databaseName, &statbuf) != -1) ? statbuf.st_mtime : 0;
         if (dbtime > remote_dbtime) {
-            logg("%s is up to date (version: custom database)\n", databaseName);
+            logg("%s is up-to-date (version: custom database)\n", databaseName);
             goto up_to_date;
         }
 
@@ -2180,7 +2569,7 @@ fc_error_t updatecustomdb(
 
         ret = downloadFile(url, tmpfile, 1, logerr, dbtime);
         if (ret == FC_UPTODATE) {
-            logg("%s is up to date (version: custom database)\n", databaseName);
+            logg("%s is up-to-date (version: custom database)\n", databaseName);
             goto up_to_date;
         } else if (ret > FC_UPTODATE) {
             logg("%cCan't download %s from %s\n", logerr ? '!' : '^', databaseName, url);
@@ -2205,9 +2594,8 @@ fc_error_t updatecustomdb(
         }
         snprintf(tmpfile_with_extension, tmpfile_with_extension_len + 1, "%s-%s", tmpfile, databaseName);
         if (rename(tmpfile, tmpfile_with_extension) == -1) {
-            free(tmpfile_with_extension);
-
             logg("!updatecustomdb: Can't rename %s to %s: %s\n", tmpfile, tmpfile_with_extension, strerror(errno));
+            free(tmpfile_with_extension);
             status = FC_EDBDIRACCESS;
             goto done;
         }
