@@ -1,7 +1,7 @@
 /*
  *  OpenSSL certificate caching.
  *
- *  Copyright (C) 2016-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2016-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Russ Kubik
  *
@@ -27,10 +27,10 @@
 #include <string.h>
 #include <pthread.h>
 
-#include "shared/cert_util.h"
-#include "shared/cert_util_internal.h"
+#include "cert_util.h"
+#include "cert_util_internal.h"
 
-#include "shared/output.h"
+#include "output.h"
 
 static cert_store_t _cert_store = {
     .mutex = PTHREAD_MUTEX_INITIALIZER};
