@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -51,14 +51,12 @@
 #include <syslog.h>
 #endif
 
-#include "output.h"
-#include "libclamav/clamav.h"
-#include "libclamav/others.h"
-#include "libclamav/str.h"
+// libclamav
+#include "clamav.h"
+#include "others.h"
+#include "str.h"
 
-#ifdef CL_NOTHREADS
-#undef CL_THREAD_SAFE
-#endif
+#include "output.h"
 
 #ifdef CL_THREAD_SAFE
 #include <pthread.h>
