@@ -1,7 +1,7 @@
 /*
  *  OpenSSL certificate verification for Windows.
  *
- *  Copyright (C) 2019-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2019-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Micah Snyder
  *
@@ -29,10 +29,10 @@
 
 #include <curl/curl.h>
 
-#include "shared/output.h"
+#include "output.h"
 
-#include "shared/cert_util.h"
-#include "shared/cert_util_internal.h"
+#include "cert_util.h"
+#include "cert_util_internal.h"
 
 cl_error_t cert_store_load(X509 **trusted_certs, size_t trusted_cert_count)
 {
