@@ -33,11 +33,14 @@
 #include <dirent.h>
 #include <ctype.h>
 
-#include "libclamav/clamav.h"
-#include "libclamav/vba_extract.h"
-#include "libclamav/ole2_extract.h"
-#include "libclamav/readdb.h"
-#include "shared/output.h"
+// libclamav
+#include "clamav.h"
+#include "vba_extract.h"
+#include "ole2_extract.h"
+#include "readdb.h"
+
+// shared
+#include "output.h"
 
 #include "vba.h"
 
@@ -114,7 +117,7 @@ cli_ctx *convenience_ctx(int fd)
     ctx->options->general |= CL_SCAN_GENERAL_HEURISTICS;
     ctx->options->parse = ~(0);
 
-    if (!(*ctx->fmap = fmap(fd, 0, 0))) {
+    if (!(*ctx->fmap = fmap(fd, 0, 0, NULL))) {
         printf("convenience_ctx: fmap failed\n");
         goto done;
     }
@@ -1092,6 +1095,7 @@ static int sigtool_scandir(const char *dirname, int hex_output)
     char *dir;
     int ret = CL_CLEAN, desc;
     cli_ctx *ctx;
+    int has_vba = 0, has_xlm = 0;
 
     fname = NULL;
     if ((dd = opendir(dirname)) != NULL) {
@@ -1116,8 +1120,8 @@ static int sigtool_scandir(const char *dirname, int hex_output)
                             }
                         } else {
                             if (S_ISREG(statbuf.st_mode)) {
-                                struct uniq *vba = NULL;
-                                tmpdir           = cli_gettmpdir();
+                                struct uniq *files = NULL;
+                                tmpdir             = cli_gettmpdir();
 
                                 /* generate the temporary directory */
                                 dir = cli_gentemp(tmpdir);
@@ -1151,7 +1155,7 @@ static int sigtool_scandir(const char *dirname, int hex_output)
                                     free(dir);
                                     return 1;
                                 }
-                                if ((ret = cli_ole2_extract(dir, ctx, &vba))) {
+                                if ((ret = cli_ole2_extract(dir, ctx, &files, &has_vba, &has_xlm))) {
                                     printf("ERROR %s\n", cl_strerror(ret));
                                     destroy_ctx(desc, ctx);
                                     cli_rmdirs(dir);
@@ -1161,8 +1165,8 @@ static int sigtool_scandir(const char *dirname, int hex_output)
                                     return ret;
                                 }
 
-                                if (vba)
-                                    sigtool_vba_scandir(dir, hex_output, vba);
+                                if (has_vba && files)
+                                    sigtool_vba_scandir(dir, hex_output, files);
                                 destroy_ctx(desc, ctx);
                                 cli_rmdirs(dir);
                                 free(dir);
