@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2013 Sourcefire, Inc.
  *
  *  Authors: Steven Morgan <smorgan@sourcefire.com>
@@ -19,8 +19,9 @@
  *  MA 02110-1301, USA.
  */
 
-#include "../libclamav/clamav.h"
-#include "../libclamav/fpu.h"
+// libclamav
+#include "clamav.h"
+#include "fpu.h"
 
 /* Helper command line interface to determine fpu endianness in unit test scripts.
  *   parameters: none
